@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2015 MongoDB, Inc.
+ * Copyright 2009-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -99,7 +99,7 @@ static PyObject* _error(char* name) {
 /* Safely downcast from Py_ssize_t to int, setting an
  * exception and returning -1 on error. */
 static int
-_downcast_and_check(Py_ssize_t size, int extra) {
+_downcast_and_check(Py_ssize_t size, uint8_t extra) {
     if (size > BSON_MAX_SIZE || ((BSON_MAX_SIZE - extra) < size)) {
         PyObject* InvalidStringData = _error("InvalidStringData");
         if (InvalidStringData) {
@@ -362,7 +362,11 @@ static int _load_python_objects(PyObject* module) {
         _load_object(&state->BSONInt64, "bson.int64", "Int64") ||
         _load_object(&state->Decimal128, "bson.decimal128", "Decimal128") ||
         _load_object(&state->UUID, "uuid", "UUID") ||
+#if PY_MAJOR_VERSION >= 3
+        _load_object(&state->Mapping, "collections.abc", "Mapping") ||
+#else
         _load_object(&state->Mapping, "collections", "Mapping") ||
+#endif
         _load_object(&state->CodecOptions, "bson.codec_options", "CodecOptions")) {
         return 1;
     }
@@ -384,6 +388,7 @@ static int _load_python_objects(PyObject* module) {
     }
 
     compiled = PyObject_CallFunction(re_compile, "O", empty_string);
+    Py_DECREF(re_compile);
     if (compiled == NULL) {
         state->REType = NULL;
         Py_DECREF(empty_string);
@@ -478,10 +483,14 @@ int convert_codec_options(PyObject* options_obj, void* p) {
  * Return 0 on failure.
  */
 int default_codec_options(struct module_state* state, codec_options_t* options) {
+    PyObject* options_obj = NULL;
     PyObject* codec_options_func = _get_object(
         state->CodecOptions, "bson.codec_options", "CodecOptions");
-    PyObject* options_obj = PyObject_CallFunctionObjArgs(
-        codec_options_func, NULL);
+    if (codec_options_func == NULL) {
+        return 0;
+    }
+    options_obj = PyObject_CallFunctionObjArgs(codec_options_func, NULL);
+    Py_DECREF(codec_options_func);
     if (options_obj == NULL) {
         return 0;
     }
@@ -851,19 +860,15 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
         {
             /* Timestamp */
             PyObject* obj;
-            long i;
+            unsigned long i;
 
             obj = PyObject_GetAttrString(value, "inc");
             if (!obj) {
                 return 0;
             }
-#if PY_MAJOR_VERSION >= 3
-            i = PyLong_AsLong(obj);
-#else
-            i = PyInt_AsLong(obj);
-#endif
+            i = PyLong_AsUnsignedLong(obj);
             Py_DECREF(obj);
-            if (i == -1 && PyErr_Occurred()) {
+            if (i == (unsigned long)-1 && PyErr_Occurred()) {
                 return 0;
             }
             if (!buffer_write_int32(buffer, (int32_t)i)) {
@@ -874,13 +879,9 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
             if (!obj) {
                 return 0;
             }
-#if PY_MAJOR_VERSION >= 3
-            i = PyLong_AsLong(obj);
-#else
-            i = PyInt_AsLong(obj);
-#endif
+            i = PyLong_AsUnsignedLong(obj);
             Py_DECREF(obj);
-            if (i == -1 && PyErr_Occurred()) {
+            if (i == (unsigned long)-1 && PyErr_Occurred()) {
                 return 0;
             }
             if (!buffer_write_int32(buffer, (int32_t)i)) {
@@ -1197,12 +1198,16 @@ static int _write_element_to_buffer(PyObject* self, buffer_t buffer,
     } else if (PyObject_TypeCheck(value, state->REType)) {
         return _write_regex_to_buffer(buffer, type_byte, value);
     }
-    
-    /* 
+
+    /*
      * Try Mapping and UUID last since we have to import
      * them if we're in a sub-interpreter.
      */
+#if PY_MAJOR_VERSION >= 3
+    mapping_type = _get_object(state->Mapping, "collections.abc", "Mapping");
+#else
     mapping_type = _get_object(state->Mapping, "collections", "Mapping");
+#endif
     if (mapping_type && PyObject_IsInstance(value, mapping_type)) {
         Py_DECREF(mapping_type);
         /* PyObject_IsInstance returns -1 on error */
@@ -1500,8 +1505,13 @@ int write_dict(PyObject* self, buffer_t buffer,
     int length;
     int length_location;
     struct module_state *state = GETSTATE(self);
+#if PY_MAJOR_VERSION >= 3
+    PyObject* mapping_type = _get_object(state->Mapping,
+                                         "collections.abc", "Mapping");
+#else
     PyObject* mapping_type = _get_object(state->Mapping,
                                          "collections", "Mapping");
+#endif
 
     if (mapping_type) {
         if (!PyObject_IsInstance(dict, mapping_type)) {
@@ -1630,62 +1640,35 @@ static PyObject* _cbson_dict_to_bson(PyObject* self, PyObject* args) {
     codec_options_t options;
     buffer_t buffer;
     PyObject* raw_bson_document_bytes_obj;
-    char* raw_bson_document_bytes;
-    Py_ssize_t raw_bson_document_bytes_len;
-    int raw_bson_document_bytes_len_int;
     long type_marker;
 
     if (!PyArg_ParseTuple(args, "ObO&|b", &dict, &check_keys,
                           convert_codec_options, &options, &top_level)) {
         return NULL;
     }
-    buffer = buffer_new();
-    if (!buffer) {
-        destroy_codec_options(&options);
-        PyErr_NoMemory();
-        return NULL;
-    }
 
     /* check for RawBSONDocument */
     type_marker = _type_marker(dict);
     if (type_marker < 0) {
         destroy_codec_options(&options);
-        buffer_free(buffer);
         return NULL;
     } else if (101 == type_marker) {
+        destroy_codec_options(&options);
         raw_bson_document_bytes_obj = PyObject_GetAttrString(dict, "raw");
         if (NULL == raw_bson_document_bytes_obj) {
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            return NULL;
-        }
-#if PY_MAJOR_VERSION >= 3
-        if (-1 == PyBytes_AsStringAndSize(raw_bson_document_bytes_obj,
-                                          &raw_bson_document_bytes,
-                                          &raw_bson_document_bytes_len)) {
-#else
-        if (-1 == PyString_AsStringAndSize(raw_bson_document_bytes_obj,
-                                           &raw_bson_document_bytes,
-                                           &raw_bson_document_bytes_len)) {
-#endif
-            Py_DECREF(raw_bson_document_bytes_obj);
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            return NULL;
-        }
-        raw_bson_document_bytes_len_int = _downcast_and_check(raw_bson_document_bytes_len, 0);
-        if (raw_bson_document_bytes_len_int < 0 ||
-            !buffer_write_bytes(buffer,
-                                raw_bson_document_bytes,
-                                raw_bson_document_bytes_len_int)) {
-            destroy_codec_options(&options);
-            buffer_free(buffer);
-            Py_DECREF(raw_bson_document_bytes_obj);
             return NULL;
         }
+        return raw_bson_document_bytes_obj;
+    }
 
-        Py_DECREF(raw_bson_document_bytes_obj);
-    } else if (!write_dict(self, buffer, dict, check_keys, &options, top_level)) {
+    buffer = buffer_new();
+    if (!buffer) {
+        destroy_codec_options(&options);
+        PyErr_NoMemory();
+        return NULL;
+    }
+
+    if (!write_dict(self, buffer, dict, check_keys, &options, top_level)) {
         destroy_codec_options(&options);
         buffer_free(buffer);
         return NULL;
@@ -1968,13 +1951,13 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
                  * From this point, we hold refs to args, kwargs, and data.
                  * If anything fails, goto uuiderror to clean them up.
                  */
-                if (options->uuid_rep == CSHARP_LEGACY) {
+                if (subtype == 3 && options->uuid_rep == CSHARP_LEGACY) {
                     /* Legacy C# byte order */
                     if ((PyDict_SetItemString(kwargs, "bytes_le", data)) == -1)
                         goto uuiderror;
                 }
                 else {
-                    if (options->uuid_rep == JAVA_LEGACY) {
+                    if (subtype == 3 && options->uuid_rep == JAVA_LEGACY) {
                         /* Convert from legacy java byte order */
                         char big_endian[16];
                         _fix_java(buffer + *position, big_endian);
@@ -1990,7 +1973,7 @@ static PyObject* get_value(PyObject* self, PyObject* name, const char* buffer,
                     }
                     if ((PyDict_SetItemString(kwargs, "bytes", data)) == -1)
                         goto uuiderror;
-                
+
                 }
                 if ((type_to_create = _get_object(state->UUID, "uuid", "UUID"))) {
                     value = PyObject_Call(type_to_create, args, kwargs);
@@ -2666,20 +2649,23 @@ static PyObject* _cbson_element_to_dict(PyObject* self, PyObject* args) {
 static PyObject* _elements_to_dict(PyObject* self, const char* string,
                                    unsigned max,
                                    const codec_options_t* options) {
-    int position = 0;
+    unsigned position = 0;
     PyObject* dict = PyObject_CallObject(options->document_class, NULL);
     if (!dict) {
         return NULL;
     }
     while (position < max) {
-        PyObject* name;
-        PyObject* value;
+        PyObject* name = NULL;
+        PyObject* value = NULL;
+        int new_position;
 
-        position = _element_to_dict(self, string, position, max,
-                                    options, &name, &value);
-        if (position < 0) {
+        new_position = _element_to_dict(
+            self, string, position, max, options, &name, &value);
+        if (new_position < 0) {
             Py_DECREF(dict);
             return NULL;
+        } else {
+            position = (unsigned)new_position;
         }
 
         PyObject_SetItem(dict, name, value);
