@@ -1,4 +1,4 @@
-# Copyright 2010-2015 MongoDB, Inc.
+# Copyright 2010-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,18 +28,21 @@ class MinKey(object):
     def __eq__(self, other):
         return isinstance(other, MinKey)
 
+    def __hash__(self):
+        return hash(self._type_marker)
+
     def __ne__(self, other):
         return not self == other
-    
+
     def __le__(self, dummy):
         return True
-    
+
     def __lt__(self, other):
         return not isinstance(other, MinKey)
 
     def __ge__(self, other):
         return isinstance(other, MinKey)
-    
+
     def __gt__(self, dummy):
         return False
 
