@@ -1,4 +1,4 @@
-/* 
+/*
 
 Copyright (c) 2007-2010  Michael G Schwern
 
@@ -30,7 +30,7 @@ THE SOFTWARE.
 
 Programmers who have available to them 64-bit time values as a 'long
 long' type can use localtime64_r() and gmtime64_r() which correctly
-converts the time even on 32-bit systems. Whether you have 64-bit time 
+converts the time even on 32-bit systems. Whether you have 64-bit time
 values will depend on the operating system.
 
 localtime64_r() is a 64-bit equivalent of localtime_r().
@@ -43,12 +43,10 @@ gmtime64_r() is a 64-bit equivalent of gmtime_r().
     #define _CRT_SECURE_NO_WARNINGS
 #endif
 
-#include <assert.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+/* Including Python.h fixes issues with interpreters built with -std=c99. */
+#include "Python.h"
+
 #include <time.h>
-#include <errno.h>
 #include "time64.h"
 #include "time64_limits.h"
 
@@ -112,7 +110,7 @@ static const int safe_years_low[SOLAR_CYCLE_LENGTH] = {
 #define CHEAT_YEARS 108
 
 #define IS_LEAP(n)      ((!(((n) + 1900) % 400) || (!(((n) + 1900) % 4) && (((n) + 1900) % 100))) != 0)
-#define WRAP(a,b,m)     ((a) = ((a) <  0  ) ? ((b)--, (a) + (m)) : (a))
+#define _TIME64_WRAP(a,b,m)     ((a) = ((a) <  0  ) ? ((b)--, (a) + (m)) : (a))
 
 #ifdef USE_SYSTEM_LOCALTIME
 #    define SHOULD_USE_SYSTEM_LOCALTIME(a)  (       \
@@ -280,7 +278,7 @@ static int check_tm(struct TM *tm)
 
     assert(tm->tm_wday >= 0);
     assert(tm->tm_wday <= 6);
-    
+
     assert(tm->tm_yday >= 0);
     assert(tm->tm_yday <= length_of_year[IS_LEAP(tm->tm_year)]);
 
@@ -356,7 +354,7 @@ static int safe_year(const Year year)
         year_cycle += 17;
 
     year_cycle %= SOLAR_CYCLE_LENGTH;
-    if( year_cycle < 0 ) 
+    if( year_cycle < 0 )
         year_cycle = SOLAR_CYCLE_LENGTH + year_cycle;
 
     assert( year_cycle >= 0 );
@@ -574,9 +572,9 @@ struct TM *gmtime64_r (const Time64_T *in_time, struct TM *p)
     time /= 24;
     v_tm_tday = time;
 
-    WRAP (v_tm_sec, v_tm_min, 60);
-    WRAP (v_tm_min, v_tm_hour, 60);
-    WRAP (v_tm_hour, v_tm_tday, 24);
+    _TIME64_WRAP (v_tm_sec, v_tm_min, 60);
+    _TIME64_WRAP (v_tm_min, v_tm_hour, 60);
+    _TIME64_WRAP (v_tm_hour, v_tm_tday, 24);
 
     v_tm_wday = (int)((v_tm_tday + 4) % 7);
     if (v_tm_wday < 0)
@@ -653,7 +651,7 @@ struct TM *gmtime64_r (const Time64_T *in_time, struct TM *p)
     p->tm_hour = v_tm_hour;
     p->tm_mon  = v_tm_mon;
     p->tm_wday = v_tm_wday;
-    
+
     assert(check_tm(p));
 
     return p;
@@ -737,7 +735,7 @@ struct TM *localtime64_r (const Time64_T *time, struct TM *local_tm)
         local_tm->tm_year++;
     }
 
-    /* GMT is Jan 1st, xx01 year, but localtime is still Dec 31st 
+    /* GMT is Jan 1st, xx01 year, but localtime is still Dec 31st
        in a non-leap xx00.  There is one point in the cycle
        we can't account for which the safe xx00 year is a leap
        year.  So we need to correct for Dec 31st comming out as
@@ -747,7 +745,7 @@ struct TM *localtime64_r (const Time64_T *time, struct TM *local_tm)
         local_tm->tm_yday--;
 
     assert(check_tm(local_tm));
-    
+
     return local_tm;
 }
 
