@@ -28,7 +28,8 @@ master_doc = 'index'
 
 # General information about the project.
 project = u'PyMongo'
-copyright = u'2008 - 2015, MongoDB, Inc.'
+copyright = u'MongoDB, Inc. 2008-present. MongoDB, Mongo, and the leaf logo are registered trademarks of MongoDB, Inc'
+html_show_sphinx = False
 
 # The version info for the project you're documenting, acts as replacement for
 # |version| and |release|, also used in various other places throughout the
@@ -82,7 +83,10 @@ db = client.doctest_test
 # Theme gratefully vendored from CPython source.
 html_theme = "pydoctheme"
 html_theme_path = ["."]
-html_theme_options = {'collapsiblesidebar': True}
+html_theme_options = {
+    'collapsiblesidebar': True,
+    'googletag': False
+}
 
 # Additional static files.
 html_static_path = ['static']
