@@ -1,4 +1,4 @@
-# Copyright 2013-2015 MongoDB, Inc.
+# Copyright 2013-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,12 +15,24 @@
 """Authentication helpers."""
 
 import hmac
+import socket
+
+try:
+    from urllib import quote
+except ImportError:
+    from urllib.parse import quote
 
 HAVE_KERBEROS = True
+_USE_PRINCIPAL = False
 try:
-    import kerberos
+    import winkerberos as kerberos
+    if tuple(map(int, kerberos.__version__.split('.')[:2])) >= (0, 5):
+        _USE_PRINCIPAL = True
 except ImportError:
-    HAVE_KERBEROS = False
+    try:
+        import kerberos
+    except ImportError:
+        HAVE_KERBEROS = False
 
 from base64 import standard_b64decode, standard_b64encode
 from collections import namedtuple
@@ -44,26 +56,35 @@ MongoCredential = namedtuple(
 """A hashable namedtuple of values used for authentication."""
 
 
-GSSAPIProperties = namedtuple('GSSAPIProperties', ['service_name'])
+GSSAPIProperties = namedtuple('GSSAPIProperties',
+                              ['service_name',
+                               'canonicalize_host_name',
+                               'service_realm'])
 """Mechanism properties for GSSAPI authentication."""
 
 
 def _build_credentials_tuple(mech, source, user, passwd, extra):
     """Build and return a mechanism specific credentials tuple.
     """
-    user = _unicode(user)
+    user = _unicode(user) if user is not None else None
+    password = passwd if passwd is None else _unicode(passwd)
     if mech == 'GSSAPI':
         properties = extra.get('authmechanismproperties', {})
         service_name = properties.get('SERVICE_NAME', 'mongodb')
-        props = GSSAPIProperties(service_name=service_name)
-        # No password, source is always $external.
-        return MongoCredential(mech, '$external', user, None, props)
+        canonicalize = properties.get('CANONICALIZE_HOST_NAME', False)
+        service_realm = properties.get('SERVICE_REALM')
+        props = GSSAPIProperties(service_name=service_name,
+                                 canonicalize_host_name=canonicalize,
+                                 service_realm=service_realm)
+        # Source is always $external.
+        return MongoCredential(mech, '$external', user, password, props)
     elif mech == 'MONGODB-X509':
+        # user can be None.
         return MongoCredential(mech, '$external', user, None, None)
     else:
         if passwd is None:
             raise ConfigurationError("A password is required.")
-        return MongoCredential(mech, source, user, _unicode(passwd), None)
+        return MongoCredential(mech, source, user, password, None)
 
 
 if PY3:
@@ -131,6 +152,35 @@ except ImportError:
                 _ui ^= from_bytes(_u1, 'big')
             return to_bytes(_ui, 20, 'big')
 
+try:
+    from hmac import compare_digest
+except ImportError:
+    if PY3:
+        def _xor_bytes(a, b):
+            return a ^ b
+    else:
+        def _xor_bytes(a, b, _ord=ord):
+            return _ord(a) ^ _ord(b)
+
+    # Python 2.x < 2.7.7 and Python 3.x < 3.3
+    # References:
+    #  - http://bugs.python.org/issue14532
+    #  - http://bugs.python.org/issue14955
+    #  - http://bugs.python.org/issue15061
+    def compare_digest(a, b, _xor_bytes=_xor_bytes):
+        left = None
+        right = b
+        if len(a) == len(b):
+            left = a
+            result = 0
+        if len(a) != len(b):
+            left = b
+            result = 1
+
+        for x, y in zip(left, right):
+            result |= _xor_bytes(x, y)
+        return result == 0
+
 
 def _parse_scram_response(response):
     """Split a scram response into key, value pairs."""
@@ -187,7 +237,7 @@ def _authenticate_scram_sha1(credentials, sock_info):
     res = sock_info.command(source, cmd)
 
     parsed = _parse_scram_response(res['payload'])
-    if parsed[b'v'] != server_sig:
+    if not compare_digest(parsed[b'v'], server_sig):
         raise OperationFailure("Server returned an invalid signature.")
 
     # Depending on how it's configured, Cyrus SASL (which the server uses)
@@ -238,12 +288,36 @@ def _authenticate_gssapi(credentials, sock_info):
 
     try:
         username = credentials.username
-        gsn = credentials.mechanism_properties.service_name
+        password = credentials.password
+        props = credentials.mechanism_properties
         # Starting here and continuing through the while loop below - establish
         # the security context. See RFC 4752, Section 3.1, first paragraph.
         host = sock_info.address[0]
-        result, ctx = kerberos.authGSSClientInit(
-            gsn + '@' + host, gssflags=kerberos.GSS_C_MUTUAL_FLAG)
+        if props.canonicalize_host_name:
+            host = socket.getfqdn(host)
+        service = props.service_name + '@' + host
+        if props.service_realm is not None:
+            service = service + '@' + props.service_realm
+
+        if password is not None:
+            if _USE_PRINCIPAL:
+                # Note that, though we use unquote_plus for unquoting URI
+                # options, we use quote here. Microsoft's UrlUnescape (used
+                # by WinKerberos) doesn't support +.
+                principal = ":".join((quote(username), quote(password)))
+                result, ctx = kerberos.authGSSClientInit(
+                    service, principal, gssflags=kerberos.GSS_C_MUTUAL_FLAG)
+            else:
+                if '@' in username:
+                    user, domain = username.split('@', 1)
+                else:
+                    user, domain = username, None
+                result, ctx = kerberos.authGSSClientInit(
+                    service, gssflags=kerberos.GSS_C_MUTUAL_FLAG,
+                    user=user, domain=domain, password=password)
+        else:
+            result, ctx = kerberos.authGSSClientInit(
+                service, gssflags=kerberos.GSS_C_MUTUAL_FLAG)
 
         if result != kerberos.AUTH_GSS_COMPLETE:
             raise OperationFailure('Kerberos context failed to initialize.')
@@ -358,8 +432,13 @@ def _authenticate_x509(credentials, sock_info):
     """Authenticate using MONGODB-X509.
     """
     query = SON([('authenticate', 1),
-                 ('mechanism', 'MONGODB-X509'),
-                 ('user', credentials.username)])
+                 ('mechanism', 'MONGODB-X509')])
+    if credentials.username is not None:
+        query['user'] = credentials.username
+    elif sock_info.max_wire_version < 5:
+        raise ConfigurationError(
+            "A username is required for MONGODB-X509 authentication "
+            "when connected to MongoDB versions older than 3.4.")
     sock_info.command('$external', query)
 
 
