@@ -1,4 +1,4 @@
-# Copyright 2014-2015 MongoDB, Inc.
+# Copyright 2014-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you
 # may not use this file except in compliance with the License.  You
@@ -21,16 +21,18 @@ from pymongo import common
 from pymongo.errors import ConfigurationError
 from pymongo.monitoring import _EventListeners
 from pymongo.pool import PoolOptions
-from pymongo.read_preferences import make_read_preference
+from pymongo.read_concern import ReadConcern
+from pymongo.read_preferences import (make_read_preference,
+                                      read_pref_mode_from_name)
 from pymongo.ssl_support import get_ssl_context
 from pymongo.write_concern import WriteConcern
 
 
 def _parse_credentials(username, password, database, options):
     """Parse authentication credentials."""
-    if username is None:
-        return None
     mechanism = options.get('authmechanism', 'DEFAULT')
+    if username is None and mechanism != 'MONGODB-X509':
+        return None
     source = options.get('authsource', database or 'admin')
     return _build_credentials_tuple(
         mechanism, source, username, password, options)
@@ -41,9 +43,11 @@ def _parse_read_preference(options):
     if 'read_preference' in options:
         return options['read_preference']
 
-    mode = options.get('readpreference', 0)
+    name = options.get('readpreference', 'primary')
+    mode = read_pref_mode_from_name(name)
     tags = options.get('readpreferencetags')
-    return make_read_preference(mode, tags)
+    max_staleness = options.get('maxstalenessseconds', -1)
+    return make_read_preference(mode, tags, max_staleness)
 
 
 def _parse_write_concern(options):
@@ -55,6 +59,12 @@ def _parse_write_concern(options):
     return WriteConcern(concern, wtimeout, j, fsync)
 
 
+def _parse_read_concern(options):
+    """Parse read concern options."""
+    concern = options.get('readconcernlevel')
+    return ReadConcern(concern)
+
+
 def _parse_ssl_options(options):
     """Parse ssl options."""
     use_ssl = options.get('ssl')
@@ -63,9 +73,11 @@ def _parse_ssl_options(options):
 
     certfile = options.get('ssl_certfile')
     keyfile = options.get('ssl_keyfile')
+    passphrase = options.get('ssl_pem_passphrase')
     ca_certs = options.get('ssl_ca_certs')
     cert_reqs = options.get('ssl_cert_reqs')
     match_hostname = options.get('ssl_match_hostname', True)
+    crlfile = options.get('ssl_crlfile')
 
     ssl_kwarg_keys = [k for k in options
                       if k.startswith('ssl_') and options[k]]
@@ -80,7 +92,8 @@ def _parse_ssl_options(options):
         use_ssl = True
 
     if use_ssl is True:
-        ctx = get_ssl_context(certfile, keyfile, ca_certs, cert_reqs)
+        ctx = get_ssl_context(
+            certfile, keyfile, passphrase, ca_certs, cert_reqs, crlfile)
         return ctx, match_hostname
     return None, match_hostname
 
@@ -88,18 +101,28 @@ def _parse_ssl_options(options):
 def _parse_pool_options(options):
     """Parse connection pool options."""
     max_pool_size = options.get('maxpoolsize', common.MAX_POOL_SIZE)
+    min_pool_size = options.get('minpoolsize', common.MIN_POOL_SIZE)
+    default_idle_seconds = common.validate_timeout_or_none(
+        'maxidletimems', common.MAX_IDLE_TIME_MS)
+    max_idle_time_seconds = options.get('maxidletimems', default_idle_seconds)
+    if max_pool_size is not None and min_pool_size > max_pool_size:
+        raise ValueError("minPoolSize must be smaller or equal to maxPoolSize")
     connect_timeout = options.get('connecttimeoutms', common.CONNECT_TIMEOUT)
-    socket_keepalive = options.get('socketkeepalive', False)
+    socket_keepalive = options.get('socketkeepalive', True)
     socket_timeout = options.get('sockettimeoutms')
     wait_queue_timeout = options.get('waitqueuetimeoutms')
     wait_queue_multiple = options.get('waitqueuemultiple')
     event_listeners = options.get('event_listeners')
+    appname = options.get('appname')
     ssl_context, ssl_match_hostname = _parse_ssl_options(options)
     return PoolOptions(max_pool_size,
+                       min_pool_size,
+                       max_idle_time_seconds,
                        connect_timeout, socket_timeout,
                        wait_queue_timeout, wait_queue_multiple,
                        ssl_context, ssl_match_hostname, socket_keepalive,
-                       _EventListeners(event_listeners))
+                       _EventListeners(event_listeners),
+                       appname)
 
 
 class ClientOptions(object):
@@ -122,7 +145,11 @@ class ClientOptions(object):
         self.__read_preference = _parse_read_preference(options)
         self.__replica_set_name = options.get('replicaset')
         self.__write_concern = _parse_write_concern(options)
+        self.__read_concern = _parse_read_concern(options)
         self.__connect = options.get('connect')
+        self.__heartbeat_frequency = options.get(
+            'heartbeatfrequencyms', common.HEARTBEAT_FREQUENCY)
+        self.__retry_writes = options.get('retrywrites', common.RETRY_WRITES)
 
     @property
     def _options(self):
@@ -154,6 +181,11 @@ class ClientOptions(object):
         """The server selection timeout for this instance in seconds."""
         return self.__server_selection_timeout
 
+    @property
+    def heartbeat_frequency(self):
+        """The monitoring frequency in seconds."""
+        return self.__heartbeat_frequency
+
     @property
     def pool_options(self):
         """A :class:`~pymongo.pool.PoolOptions` instance."""
@@ -173,3 +205,13 @@ class ClientOptions(object):
     def write_concern(self):
         """A :class:`~pymongo.write_concern.WriteConcern` instance."""
         return self.__write_concern
+
+    @property
+    def read_concern(self):
+        """A :class:`~pymongo.read_concern.ReadConcern` instance."""
+        return self.__read_concern
+
+    @property
+    def retry_writes(self):
+        """If this instance should retry supported write operations."""
+        return self.__retry_writes
