@@ -1,4 +1,4 @@
-# Copyright 2011-2015 MongoDB, Inc.
+# Copyright 2011-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you
 # may not use this file except in compliance with the License.  You
@@ -16,22 +16,30 @@
 """Functions and classes common to multiple pymongo modules."""
 
 import collections
+import datetime
 import warnings
 
+from bson import SON
 from bson.binary import (STANDARD, PYTHON_LEGACY,
                          JAVA_LEGACY, CSHARP_LEGACY)
 from bson.codec_options import CodecOptions
-from bson.py3compat import string_type, integer_types, iteritems
+from bson.py3compat import abc, integer_types, iteritems, string_type
 from bson.raw_bson import RawBSONDocument
 from pymongo.auth import MECHANISMS
 from pymongo.errors import ConfigurationError
 from pymongo.monitoring import _validate_event_listeners
 from pymongo.read_concern import ReadConcern
-from pymongo.read_preferences import (read_pref_mode_from_name,
-                                      _ServerMode)
+from pymongo.read_preferences import _MONGOS_MODES, _ServerMode
 from pymongo.ssl_support import validate_cert_reqs
 from pymongo.write_concern import WriteConcern
 
+try:
+    from collections import OrderedDict
+    ORDERED_TYPES = (SON, OrderedDict)
+except ImportError:
+    ORDERED_TYPES = (SON,)
+
+
 # Defaults until we connect to a server and get updated limits.
 MAX_BSON_SIZE = 16 * (1024 ** 2)
 MAX_MESSAGE_SIZE = 2 * MAX_BSON_SIZE
@@ -40,8 +48,9 @@ MAX_WIRE_VERSION = 0
 MAX_WRITE_BATCH_SIZE = 1000
 
 # What this version of PyMongo supports.
-MIN_SUPPORTED_WIRE_VERSION = 0
-MAX_SUPPORTED_WIRE_VERSION = 3
+MIN_SUPPORTED_SERVER_VERSION = "2.6"
+MIN_SUPPORTED_WIRE_VERSION = 2
+MAX_SUPPORTED_WIRE_VERSION = 5
 
 # Frequency to call ismaster on servers, in seconds.
 HEARTBEAT_FREQUENCY = 10
@@ -49,6 +58,9 @@ HEARTBEAT_FREQUENCY = 10
 # Frequency to process kill-cursors, in seconds. See MongoClient.close_cursor.
 KILL_CURSOR_FREQUENCY = 1
 
+# Frequency to process events queue, in seconds.
+EVENTS_QUEUE_FREQUENCY = 1
+
 # How long to wait, in seconds, for a suitable server to be found before
 # aborting an operation. For example, if the client attempts an insert
 # during a replica set election, SERVER_SELECTION_TIMEOUT governs the
@@ -64,20 +76,28 @@ CONNECT_TIMEOUT = 20.0
 # Default value for maxPoolSize.
 MAX_POOL_SIZE = 100
 
+# Default value for minPoolSize.
+MIN_POOL_SIZE = 0
+
+# Default value for maxIdleTimeMS.
+MAX_IDLE_TIME_MS = None
+
 # Default value for localThresholdMS.
 LOCAL_THRESHOLD_MS = 15
 
-# mongod/s 2.6 and above return code 59 when a
-# command doesn't exist. mongod versions previous
-# to 2.6 and mongos 2.4.x return no error code
-# when a command does exist. mongos versions previous
-# to 2.4.0 return code 13390 when a command does not
-# exist.
-COMMAND_NOT_FOUND_CODES = (59, 13390, None)
+# Default value for retryWrites.
+RETRY_WRITES = False
+
+# mongod/s 2.6 and above return code 59 when a command doesn't exist.
+COMMAND_NOT_FOUND_CODES = (59,)
 
 # Error codes to ignore if GridFS calls createIndex on a secondary
 UNAUTHORIZED_CODES = (13, 16547, 16548)
 
+# Maximum number of sessions to send in a single endSessions command.
+# From the driver sessions spec.
+_MAX_END_SESSIONS = 10000
+
 
 def partition_node(node):
     """Split a host:port string into (host, int(port)) pair."""
@@ -276,6 +296,14 @@ def validate_timeout_or_zero(option, value):
     return validate_positive_float(option, value) / 1000.0
 
 
+def validate_max_staleness(option, value):
+    """Validates maxStalenessSeconds according to the Max Staleness Spec."""
+    if value == -1 or value == "-1":
+        # Default: No maximum staleness.
+        return -1
+    return validate_positive_integer(option, value)
+
+
 def validate_read_preference(dummy, value):
     """Validate a read preference.
     """
@@ -284,13 +312,16 @@ def validate_read_preference(dummy, value):
     return value
 
 
-def validate_read_preference_mode(dummy, name):
+def validate_read_preference_mode(dummy, value):
     """Validate read preference mode for a MongoReplicaSetClient.
+
+    .. versionchanged:: 3.5
+       Returns the original ``value`` instead of the validated read preference
+       mode.
     """
-    try:
-        return read_pref_mode_from_name(name)
-    except ValueError:
-        raise ValueError("%s is not a valid read preference" % (name,))
+    if value not in _MONGOS_MODES:
+        raise ValueError("%s is not a valid read preference" % (value,))
+    return value
 
 
 def validate_auth_mechanism(option, value):
@@ -335,7 +366,9 @@ def validate_read_preference_tags(name, value):
     return tag_sets
 
 
-_MECHANISM_PROPS = frozenset(['SERVICE_NAME'])
+_MECHANISM_PROPS = frozenset(['SERVICE_NAME',
+                              'CANONICALIZE_HOST_NAME',
+                              'SERVICE_REALM'])
 
 
 def validate_auth_mechanism_properties(option, value):
@@ -353,23 +386,40 @@ def validate_auth_mechanism_properties(option, value):
             raise ValueError("%s is not a supported auth "
                              "mechanism property. Must be one of "
                              "%s." % (key, tuple(_MECHANISM_PROPS)))
-        props[key] = val
+        if key == 'CANONICALIZE_HOST_NAME':
+            props[key] = validate_boolean_or_string(key, val)
+        else:
+            props[key] = val
 
     return props
 
 
 def validate_document_class(option, value):
     """Validate the document_class option."""
-    if not issubclass(value, (collections.MutableMapping, RawBSONDocument)):
+    if not issubclass(value, (abc.MutableMapping, RawBSONDocument)):
         raise TypeError("%s must be dict, bson.son.SON, "
                         "bson.raw_bson.RawBSONDocument, or a "
                         "sublass of collections.MutableMapping" % (option,))
     return value
 
 
+def validate_list(option, value):
+    """Validates that 'value' is a list."""
+    if not isinstance(value, list):
+        raise TypeError("%s must be a list" % (option,))
+    return value
+
+
+def validate_list_or_none(option, value):
+    """Validates that 'value' is a list or None."""
+    if value is None:
+        return value
+    return validate_list(option, value)
+
+
 def validate_is_mapping(option, value):
     """Validate the type of method arguments that expect a document."""
-    if not isinstance(value, collections.Mapping):
+    if not isinstance(value, abc.Mapping):
         raise TypeError("%s must be an instance of dict, bson.son.SON, or "
                         "other type that inherits from "
                         "collections.Mapping" % (option,))
@@ -377,13 +427,24 @@ def validate_is_mapping(option, value):
 
 def validate_is_document_type(option, value):
     """Validate the type of method arguments that expect a MongoDB document."""
-    if not isinstance(value, (collections.MutableMapping, RawBSONDocument)):
+    if not isinstance(value, (abc.MutableMapping, RawBSONDocument)):
         raise TypeError("%s must be an instance of dict, bson.son.SON, "
                         "bson.raw_bson.RawBSONDocument, or "
                         "a type that inherits from "
                         "collections.MutableMapping" % (option,))
 
 
+def validate_appname_or_none(option, value):
+    """Validate the appname option."""
+    if value is None:
+        return value
+    validate_string(option, value)
+    # We need length in bytes, so encode utf8 first.
+    if len(value.encode('utf-8')) > 128:
+        raise ValueError("%s must be <= 128 bytes" % (option,))
+    return value
+
+
 def validate_ok_for_replace(replacement):
     """Validate a replacement document."""
     validate_is_mapping("replacement", replacement)
@@ -405,9 +466,30 @@ def validate_ok_for_update(update):
         raise ValueError('update only works with $ operators')
 
 
+_UNICODE_DECODE_ERROR_HANDLERS = frozenset(['strict', 'replace', 'ignore'])
+
+
+def validate_unicode_decode_error_handler(dummy, value):
+    """Validate the Unicode decode error handler option of CodecOptions.
+    """
+    if value not in _UNICODE_DECODE_ERROR_HANDLERS:
+        raise ValueError("%s is an invalid Unicode decode error handler. "
+                         "Must be one of "
+                         "%s" % (value, tuple(_UNICODE_DECODE_ERROR_HANDLERS)))
+    return value
+
+
+def validate_tzinfo(dummy, value):
+    """Validate the tzinfo option
+    """
+    if value is not None and not isinstance(value, datetime.tzinfo):
+        raise TypeError("%s must be an instance of datetime.tzinfo" % value)
+    return value
+
+
 # journal is an alias for j,
 # wtimeoutms is an alias for wtimeout,
-VALIDATORS = {
+URI_VALIDATORS = {
     'replicaset': validate_string_or_none,
     'w': validate_int_or_basestring,
     'wtimeout': validate_integer,
@@ -415,34 +497,56 @@ VALIDATORS = {
     'fsync': validate_boolean_or_string,
     'j': validate_boolean_or_string,
     'journal': validate_boolean_or_string,
-    'connecttimeoutms': validate_timeout_or_none,
     'maxpoolsize': validate_positive_integer_or_none,
     'socketkeepalive': validate_boolean_or_string,
-    'sockettimeoutms': validate_timeout_or_none,
-    'waitqueuetimeoutms': validate_timeout_or_none,
     'waitqueuemultiple': validate_non_negative_integer_or_none,
     'ssl': validate_boolean_or_string,
     'ssl_keyfile': validate_readable,
     'ssl_certfile': validate_readable,
+    'ssl_pem_passphrase': validate_string_or_none,
     'ssl_cert_reqs': validate_cert_reqs,
     'ssl_ca_certs': validate_readable,
     'ssl_match_hostname': validate_boolean_or_string,
+    'ssl_crlfile': validate_readable,
     'readconcernlevel': validate_string_or_none,
-    'read_preference': validate_read_preference,
     'readpreference': validate_read_preference_mode,
     'readpreferencetags': validate_read_preference_tags,
     'localthresholdms': validate_positive_float_or_zero,
-    'serverselectiontimeoutms': validate_timeout_or_zero,
     'authmechanism': validate_auth_mechanism,
     'authsource': validate_string,
     'authmechanismproperties': validate_auth_mechanism_properties,
-    'document_class': validate_document_class,
     'tz_aware': validate_boolean_or_string,
     'uuidrepresentation': validate_uuid_representation,
-    'connect': validate_boolean,
-    'event_listeners': _validate_event_listeners
+    'connect': validate_boolean_or_string,
+    'minpoolsize': validate_non_negative_integer,
+    'appname': validate_appname_or_none,
+    'unicode_decode_error_handler': validate_unicode_decode_error_handler,
+    'retrywrites': validate_boolean_or_string,
 }
 
+TIMEOUT_VALIDATORS = {
+    'connecttimeoutms': validate_timeout_or_none,
+    'sockettimeoutms': validate_timeout_or_none,
+    'waitqueuetimeoutms': validate_timeout_or_none,
+    'serverselectiontimeoutms': validate_timeout_or_zero,
+    'heartbeatfrequencyms': validate_timeout_or_none,
+    'maxidletimems': validate_timeout_or_none,
+    'maxstalenessseconds': validate_max_staleness,
+}
+
+KW_VALIDATORS = {
+    'document_class': validate_document_class,
+    'read_preference': validate_read_preference,
+    'event_listeners': _validate_event_listeners,
+    'tzinfo': validate_tzinfo,
+    'username': validate_string_or_none,
+    'password': validate_string_or_none,
+}
+
+URI_VALIDATORS.update(TIMEOUT_VALIDATORS)
+VALIDATORS = URI_VALIDATORS.copy()
+VALIDATORS.update(KW_VALIDATORS)
+
 
 _AUTH_OPTIONS = frozenset(['authmechanismproperties'])
 
@@ -466,7 +570,7 @@ def validate(option, value):
     return lower, value
 
 
-def get_validated_options(options):
+def get_validated_options(options, warn=True):
     """Validate each entry in options and raise a warning if it is not valid.
     Returns a copy of options with invalid entries removed
     """
@@ -474,10 +578,13 @@ def get_validated_options(options):
     for opt, value in iteritems(options):
         lower = opt.lower()
         try:
-            validator = VALIDATORS.get(lower, raise_config_error)
+            validator = URI_VALIDATORS.get(lower, raise_config_error)
             value = validator(opt, value)
         except (ValueError, ConfigurationError) as exc:
-            warnings.warn(str(exc))
+            if warn:
+                warnings.warn(str(exc))
+            else:
+                raise
         else:
             validated_options[lower] = value
     return validated_options
@@ -552,7 +659,8 @@ class BaseObject(object):
 
     @property
     def read_concern(self):
-        """Read only access to the read concern of this instance.
+        """Read only access to the :class:`~pymongo.read_concern.ReadConcern`
+        of this instance.
 
         .. versionadded:: 3.2
         """
