@@ -1,4 +1,4 @@
-# Copyright 2009-2015 MongoDB, Inc.
+# Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,6 +26,10 @@ class PyMongoError(Exception):
     """Base class for all PyMongo exceptions."""
 
 
+class ProtocolError(PyMongoError):
+    """Raised for failures related to the wire protocol."""
+
+
 class ConnectionFailure(PyMongoError):
     """Raised when a connection to the database cannot be made or is lost."""
 
@@ -43,7 +47,7 @@ class AutoReconnect(ConnectionFailure):
     Subclass of :exc:`~pymongo.errors.ConnectionFailure`.
     """
     def __init__(self, message='', errors=None):
-        self.errors = errors or []
+        self.errors = self.details = errors or []
         ConnectionFailure.__init__(self, message)
 
 
