@@ -1,4 +1,4 @@
-# Copyright 2010-2015 MongoDB, Inc.
+# Copyright 2010-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,14 @@
 import os
 import socket
 import sys
-from pymongo.common import partition_node
+import time
+import warnings
+
+try:
+    import ipaddress
+    HAVE_IPADDRESS = True
+except ImportError:
+    HAVE_IPADDRESS = False
 
 if sys.version_info[:2] == (2, 6):
     import unittest2 as unittest
@@ -26,15 +33,16 @@ if sys.version_info[:2] == (2, 6):
 else:
     import unittest
     from unittest import SkipTest
-import warnings
 
 from functools import wraps
 
 import pymongo
 import pymongo.errors
 
+from bson.son import SON
 from bson.py3compat import _unicode
 from pymongo import common
+from pymongo.common import partition_node
 from pymongo.ssl_support import HAVE_SSL, validate_cert_reqs
 from test.version import Version
 
@@ -82,14 +90,29 @@ def is_server_resolvable():
 
 
 def _connect(host, port, **kwargs):
-    try:
-        client = pymongo.MongoClient(
-            host, port, serverSelectionTimeoutMS=100, **kwargs)
-        client.admin.command('ismaster')  # Can we connect?
-        # If connected, then return client with default timeout
-        return pymongo.MongoClient(host, port, **kwargs)
-    except pymongo.errors.ConnectionFailure:
-        return None
+    client = pymongo.MongoClient(host, port, **kwargs)
+    start = time.time()
+    # Jython takes a long time to connect.
+    if sys.platform.startswith('java'):
+        time_limit = 10
+    else:
+        time_limit = .5
+    while not client.nodes:
+        time.sleep(0.05)
+        if time.time() - start > time_limit:
+            return None
+
+    return client
+
+
+def _create_user(authdb, user, pwd=None, roles=None, **kwargs):
+    cmd = SON([('createUser', user)])
+    # X509 doesn't use a password
+    if pwd:
+        cmd['pwd'] = pwd
+    cmd['roles'] = roles or ['root']
+    cmd.update(**kwargs)
+    return authdb.command(cmd)
 
 
 class client_knobs(object):
@@ -140,10 +163,15 @@ class client_knobs(object):
         self.disable()
 
 
+def _all_users(db):
+    return set(u['user'] for u in db.command('usersInfo').get('users', []))
+
+
 class ClientContext(object):
 
     def __init__(self):
         """Create a client and grab essential information from the server."""
+        self.connection_attempts = []
         self.connected = False
         self.ismaster = {}
         self.w = None
@@ -161,11 +189,12 @@ class ClientContext(object):
         self.ssl_certfile = False
         self.server_is_resolvable = is_server_resolvable()
         self.ssl_client_options = {}
-        self.client = _connect(host, port)
+        self.sessions_enabled = False
+        self.client = self._connect(host, port)
 
         if HAVE_SSL and not self.client:
             # Is MongoDB configured for SSL?
-            self.client = _connect(host, port, **_SSL_OPTIONS)
+            self.client = self._connect(host, port, **_SSL_OPTIONS)
             if self.client:
                 self.ssl = True
                 self.ssl_client_options = _SSL_OPTIONS
@@ -175,30 +204,6 @@ class ClientContext(object):
 
         if self.client:
             self.connected = True
-            ismaster = self.client.admin.command('ismaster')
-            if 'setName' in ismaster:
-                self.replica_set_name = ismaster['setName']
-                self.is_rs = True
-                # It doesn't matter which member we use as the seed here.
-                self.client = pymongo.MongoClient(
-                    host,
-                    port,
-                    replicaSet=self.replica_set_name,
-                    **self.ssl_client_options)
-                # Get the authoritative ismaster result from the primary.
-                self.ismaster = self.client.admin.command('ismaster')
-                nodes = [partition_node(node.lower())
-                         for node in self.ismaster.get('hosts', [])]
-                nodes.extend([partition_node(node.lower())
-                              for node in self.ismaster.get('passives', [])])
-                nodes.extend([partition_node(node.lower())
-                              for node in self.ismaster.get('arbiters', [])])
-                self.nodes = set(nodes)
-            else:
-                self.ismaster = ismaster
-                self.nodes = set([(host, port)])
-            self.w = len(self.ismaster.get("hosts", [])) or 1
-            self.version = Version.from_client(self.client)
 
             try:
                 self.cmd_line = self.client.admin.command('getCmdLineOpts')
@@ -214,17 +219,54 @@ class ClientContext(object):
 
             if self.auth_enabled:
                 # See if db_user already exists.
-                self.user_provided = self._check_user_provided()
-                if not self.user_provided:
-                    roles = {}
-                    if self.version.at_least(2, 5, 3, -1):
-                        roles = {'roles': ['root']}
-                    self.client.admin.add_user(db_user, db_pwd, **roles)
-                    self.client.admin.authenticate(db_user, db_pwd)
+                if not self._check_user_provided():
+                    _create_user(self.client.admin, db_user, db_pwd)
+
+                self.client = self._connect(
+                    host, port, username=db_user, password=db_pwd,
+                    replicaSet=self.replica_set_name,
+                    **self.ssl_client_options)
 
                 # May not have this if OperationFailure was raised earlier.
                 self.cmd_line = self.client.admin.command('getCmdLineOpts')
 
+            self.ismaster = ismaster = self.client.admin.command('isMaster')
+            self.sessions_enabled = 'logicalSessionTimeoutMinutes' in ismaster
+
+            if 'setName' in ismaster:
+                self.replica_set_name = ismaster['setName']
+                self.is_rs = True
+                if self.auth_enabled:
+                    # It doesn't matter which member we use as the seed here.
+                    self.client = pymongo.MongoClient(
+                        host,
+                        port,
+                        username=db_user,
+                        password=db_pwd,
+                        replicaSet=self.replica_set_name,
+                        **self.ssl_client_options)
+                else:
+                    self.client = pymongo.MongoClient(
+                        host,
+                        port,
+                        replicaSet=self.replica_set_name,
+                        **self.ssl_client_options)
+
+                # Get the authoritative ismaster result from the primary.
+                self.ismaster = self.client.admin.command('ismaster')
+                nodes = [partition_node(node.lower())
+                         for node in self.ismaster.get('hosts', [])]
+                nodes.extend([partition_node(node.lower())
+                              for node in self.ismaster.get('passives', [])])
+                nodes.extend([partition_node(node.lower())
+                              for node in self.ismaster.get('arbiters', [])])
+                self.nodes = set(nodes)
+            else:
+                self.ismaster = ismaster
+                self.nodes = set([(host, port)])
+            self.w = len(self.ismaster.get("hosts", [])) or 1
+            self.version = Version.from_client(self.client)
+
             if 'enableTestCommands=1' in self.cmd_line['argv']:
                 self.test_commands_enabled = True
             elif 'parsed' in self.cmd_line:
@@ -239,6 +281,35 @@ class ClientContext(object):
             self.is_mongos = (self.ismaster.get('msg') == 'isdbgrid')
             self.has_ipv6 = self._server_started_with_ipv6()
 
+    def _connect(self, host, port, **kwargs):
+        # Jython takes a long time to connect.
+        if sys.platform.startswith('java'):
+            timeout_ms = 10000
+        else:
+            timeout_ms = 500
+        client = pymongo.MongoClient(
+            host, port, serverSelectionTimeoutMS=timeout_ms, **kwargs)
+        try:
+            try:
+                client.admin.command('isMaster')  # Can we connect?
+            except pymongo.errors.OperationFailure as exc:
+                # SERVER-32063
+                self.connection_attempts.append(
+                    'connected client %r, but isMaster failed: %s' % (
+                        client, exc))
+            else:
+                self.connection_attempts.append(
+                    'successfully connected client %r' % (client,))
+            # If connected, then return client with default timeout
+            return pymongo.MongoClient(host, port, **kwargs)
+        except pymongo.errors.ConnectionFailure as exc:
+            self.connection_attempts.append(
+                'failed to connect client %r: %s' % (client, exc))
+            return None
+
+    def connection_attempt_info(self):
+        return '\n'.join(self.connection_attempts)
+
     @property
     def host(self):
         if self.is_rs:
@@ -259,12 +330,21 @@ class ClientContext(object):
 
     @property
     def has_secondaries(self):
+        if not self.client:
+            return False
         return bool(len(self.client.secondaries))
 
     def _check_user_provided(self):
+        """Return True if db_user/db_password is already an admin user."""
+        client = pymongo.MongoClient(
+            host, port,
+            username=db_user,
+            password=db_pwd,
+            serverSelectionTimeoutMS=100,
+            **self.ssl_client_options)
+
         try:
-            self.client.admin.authenticate(db_user, db_pwd)
-            return True
+            return db_user in _all_users(client.admin)
         except pymongo.errors.OperationFailure as e:
             msg = e.details.get('errmsg', '')
             if e.code == 18 or 'auth fails' in msg:
@@ -331,6 +411,14 @@ class ClientContext(object):
             return decorate
         return make_wrapper(func)
 
+    def create_user(self, dbname, user, pwd=None, roles=None, **kwargs):
+        kwargs['writeConcern'] = {'w': self.w}
+        return _create_user(self.client[dbname], user, pwd, roles, **kwargs)
+
+    def drop_user(self, dbname, user):
+        self.client[dbname].command(
+            'dropUser', user, writeConcern={'w': self.w})
+
     def require_connection(self, func):
         """Run a test only if we can connect to MongoDB."""
         return self._require(
@@ -375,7 +463,7 @@ class ClientContext(object):
         """Run a test only if the client is connected to a replica set that has
         `count` secondaries.
         """
-        sec_count = len(self.client.secondaries)
+        sec_count = 0 if not self.client else len(self.client.secondaries)
         return self._require(sec_count >= count,
                              "Need %d secondaries, %d available"
                              % (count, sec_count))
@@ -405,6 +493,18 @@ class ClientContext(object):
                              "Must be connected to a mongos",
                              func=func)
 
+    def require_standalone(self, func):
+        """Run a test only if the client is connected to a standalone."""
+        return self._require(not (self.is_mongos or self.is_rs),
+                             "Must be connected to a standalone",
+                             func=func)
+
+    def require_no_standalone(self, func):
+        """Run a test only if the client is not connected to a standalone."""
+        return self._require(self.is_mongos or self.is_rs,
+                             "Must be connected to a replica set or mongos",
+                             func=func)
+
     def check_auth_with_sharding(self, func):
         """Skip a test when connected to mongos < 2.0 and running with auth."""
         condition = not (self.auth_enabled and
@@ -450,11 +550,40 @@ class ClientContext(object):
                              "hostname in the certificate",
                              func=func)
 
+    def require_sessions(self, func):
+        """Run a test only if the deployment supports sessions."""
+        return self._require(self.sessions_enabled,
+                             "Sessions not supported",
+                             func=func)
+
+
 # Reusable client context
 client_context = ClientContext()
 
 
-class IntegrationTest(unittest.TestCase):
+def sanitize_cmd(cmd):
+    cp = cmd.copy()
+    cp.pop('$clusterTime', None)
+    cp.pop('lsid', None)
+    return cp
+
+
+def sanitize_reply(reply):
+    cp = reply.copy()
+    cp.pop('$clusterTime', None)
+    cp.pop('operationTime', None)
+    return cp
+
+
+class PyMongoTestCase(unittest.TestCase):
+    def assertEqualCommand(self, expected, actual, msg=None):
+        self.assertEqual(expected, sanitize_cmd(actual), msg)
+
+    def assertEqualReply(self, expected, actual, msg=None):
+        self.assertEqual(expected, sanitize_reply(actual), msg)
+
+
+class IntegrationTest(PyMongoTestCase):
     """Base class for TestCases that need a connection to MongoDB to pass."""
 
     @classmethod
@@ -462,6 +591,16 @@ class IntegrationTest(unittest.TestCase):
     def setUpClass(cls):
         cls.client = client_context.client
         cls.db = cls.client.pymongo_test
+        if client_context.auth_enabled:
+            cls.credentials = {'username': db_user, 'password': db_pwd}
+        else:
+            cls.credentials = {}
+
+
+# Use assertRaisesRegex if available, otherwise use Python 2.7's
+# deprecated assertRaisesRegexp, with a 'p'.
+if not hasattr(unittest.TestCase, 'assertRaisesRegex'):
+    IntegrationTest.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
 
 
 class MockClientTest(unittest.TestCase):
@@ -500,8 +639,6 @@ def teardown():
     c.drop_database("pymongo_test2")
     c.drop_database("pymongo_test_mike")
     c.drop_database("pymongo_test_bernie")
-    if client_context.auth_enabled and not client_context.user_provided:
-        c.admin.remove_user(db_user)
 
 
 class PymongoTestRunner(unittest.TextTestRunner):
