@@ -1,4 +1,4 @@
-# Copyright 2009-2012 10gen, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,24 +28,25 @@ from stat import S_IRUSR
 import pymongo
 import pymongo.errors
 from pymongo.read_preferences import ReadPreference
+from test.utils import connected
 
 home = os.environ.get('HOME')
 default_dbpath = os.path.join(home, 'data', 'pymongo_high_availability')
 dbpath = os.environ.get('DBPATH', default_dbpath)
 default_logpath = os.path.join(home, 'log', 'pymongo_high_availability')
-logpath = os.environ.get('LOGPATH', default_logpath)
-hostname = os.environ.get('HOSTNAME', socket.gethostname())
+logpath = os.path.expanduser(os.environ.get('LOGPATH', default_logpath))
+hostname = os.environ.get('HOSTNAME', 'localhost')
 port = int(os.environ.get('DBPORT', 27017))
-mongod = os.environ.get('MONGOD', 'mongod')
-mongos = os.environ.get('MONGOS', 'mongos')
-set_name = os.environ.get('SETNAME', 'repl0')
-use_greenlets = bool(os.environ.get('GREENLETS'))
+mongod = os.path.expanduser(os.environ.get('MONGOD', 'mongod'))
+mongos = os.path.expanduser(os.environ.get('MONGOS', 'mongos'))
+replica_set_name = os.environ.get('SETNAME', 'repl0')
 ha_tools_debug = bool(os.environ.get('HA_TOOLS_DEBUG'))
 
 
 nodes = {}
 routers = {}
 cur_port = port
+key_file = None
 
 try:
     from subprocess import DEVNULL  # Python 3.
@@ -57,10 +58,13 @@ def kill_members(members, sig, hosts=nodes):
     for member in sorted(members):
         try:
             if ha_tools_debug:
-                print('killing %s' % (member,)),
+                print('killing %s' % (member,))
             proc = hosts[member]['proc']
+            if 'java' in sys.platform:
+                # _process is a wrapped java.lang.UNIXProcess.
+                proc._process.destroy()
             # Not sure if cygwin makes sense here...
-            if sys.platform in ('win32', 'cygwin'):
+            elif sys.platform in ('win32', 'cygwin'):
                 os.kill(proc.pid, signal.CTRL_C_EVENT)
             else:
                 os.kill(proc.pid, sig)
@@ -99,6 +103,7 @@ def start_subprocess(cmd):
 
 def start_replica_set(members, auth=False, fresh=True):
     global cur_port
+    global key_file
 
     if fresh:
         if os.path.exists(dbpath):
@@ -109,19 +114,15 @@ def start_replica_set(members, auth=False, fresh=True):
 
         try:
             os.makedirs(dbpath)
-        except OSError:
-            exc = sys.exc_info()[1]
+        except OSError as exc:
             print(exc)
             print("\tWhile creating %s" % (dbpath,))
 
     if auth:
         key_file = os.path.join(dbpath, 'key.txt')
         if not os.path.exists(key_file):
-            f = open(key_file, 'w')
-            try:
+            with open(key_file, 'w') as f:
                 f.write("my super secret system password")
-            finally:
-                f.close()
             os.chmod(key_file, S_IRUSR)
 
     for i in range(len(members)):
@@ -136,7 +137,7 @@ def start_replica_set(members, auth=False, fresh=True):
         cmd = [mongod,
                '--dbpath', path,
                '--port', str(cur_port),
-               '--replSet', set_name,
+               '--replSet', replica_set_name,
                '--nojournal', '--oplogSize', '64',
                '--logappend', '--logpath', member_logpath]
         if auth:
@@ -146,7 +147,7 @@ def start_replica_set(members, auth=False, fresh=True):
             print('starting %s' % (' '.join(cmd),))
 
         proc = start_subprocess(cmd)
-        nodes[host] = {'proc': proc, 'cmd': cmd}
+        nodes[host] = {'proc': proc, 'cmd': cmd, 'dbpath': path}
         res = wait_for(proc, cur_port)
 
         cur_port += 1
@@ -154,18 +155,17 @@ def start_replica_set(members, auth=False, fresh=True):
         if not res:
             return None
 
-    config = {'_id': set_name, 'members': members}
+    config = {'_id': replica_set_name, 'members': members}
     primary = members[0]['host']
-    c = pymongo.MongoClient(primary, use_greenlets=use_greenlets)
+    c = pymongo.MongoClient(primary)
     try:
         if ha_tools_debug:
             print('rs.initiate(%s)' % (config,))
 
         c.admin.command('replSetInitiate', config)
-    except pymongo.errors.OperationFailure:
+    except pymongo.errors.OperationFailure as exc:
         # Already initialized from a previous run?
         if ha_tools_debug:
-            exc = sys.exc_info()[1]
             print(exc)
 
     expected_arbiters = 0
@@ -174,8 +174,8 @@ def start_replica_set(members, auth=False, fresh=True):
             expected_arbiters += 1
     expected_secondaries = len(members) - expected_arbiters - 1
 
-    # Wait for 8 minutes for replica set to come up
-    patience = 8
+    # Wait a minute for replica set to come up.
+    patience = 1
     for i in range(int(patience * 60 / 2)):
         time.sleep(2)
         try:
@@ -193,12 +193,15 @@ def start_replica_set(members, auth=False, fresh=True):
         kill_all_members()
         raise Exception(
             "Replica set still not initalized after %s minutes" % patience)
-    return primary, set_name
+    return primary, replica_set_name
 
 
 def create_sharded_cluster(num_routers=3):
     global cur_port
 
+    if not os.path.exists(logpath):
+        os.makedirs(logpath)
+
     # Start a config server
     configdb_host = '%s:%d' % (hostname, cur_port)
     path = os.path.join(dbpath, 'configdb')
@@ -211,7 +214,7 @@ def create_sharded_cluster(num_routers=3):
            '--nojournal', '--logappend',
            '--logpath', configdb_logpath]
     proc = start_subprocess(cmd)
-    nodes[configdb_host] = {'proc': proc, 'cmd': cmd}
+    nodes[configdb_host] = {'proc': proc, 'cmd': cmd, 'dbpath': path}
     res = wait_for(proc, cur_port)
     if not res:
         return None
@@ -229,7 +232,7 @@ def create_sharded_cluster(num_routers=3):
            '--nojournal', '--logappend',
            '--logpath', db_logpath]
     proc = start_subprocess(cmd)
-    nodes[shard_host] = {'proc': proc, 'cmd': cmd}
+    nodes[shard_host] = {'proc': proc, 'cmd': cmd, 'dbpath': path}
     res = wait_for(proc, cur_port)
     if not res:
         return None
@@ -264,10 +267,18 @@ def create_sharded_cluster(num_routers=3):
 
 # Connect to a random member
 def get_client():
-    return pymongo.MongoClient(
-        nodes.keys(),
-        read_preference=ReadPreference.PRIMARY_PREFERRED,
-        use_greenlets=use_greenlets)
+    # Attempt a direct connection to each node until one succeeds. Using a
+    # non-PRIMARY read preference allows us to use the node even if it's a
+    # secondary.
+    for i, node in enumerate(nodes.keys()):
+        try:
+            return connected(
+                pymongo.MongoClient(
+                    node,
+                    read_preference=ReadPreference.PRIMARY_PREFERRED))
+        except pymongo.errors.ConnectionFailure:
+            if i == len(nodes) - 1:
+                raise
 
 
 def get_mongos_seed_list():
@@ -296,12 +307,21 @@ def get_primary():
         assert len(primaries) <= 1
         if primaries:
             return primaries[0]
-    except pymongo.errors.ConnectionFailure:
+    except (pymongo.errors.ConnectionFailure, pymongo.errors.OperationFailure):
         pass
 
     return None
 
 
+def wait_for_primary():
+    for _ in range(30):
+        time.sleep(1)
+        if get_primary():
+            break
+    else:
+        raise AssertionError("Primary didn't come back up")
+
+
 def get_random_secondary():
     secondaries = get_members_in_state(2)
     if len(secondaries):
@@ -369,19 +389,85 @@ def kill_all_secondaries(sig=2):
     return secondaries
 
 
+# TODO: refactor w/ start_replica_set
+def add_member(auth=False):
+    global cur_port
+    host = '%s:%d' % (hostname, cur_port)
+    primary = get_primary()
+    assert primary
+    c = pymongo.MongoClient(primary)
+    config = c.local.system.replset.find_one()
+    _id = max([member['_id'] for member in config['members']]) + 1
+    member = {'_id': _id, 'host': host}
+    path = os.path.join(dbpath, 'db' + str(_id))
+    if os.path.exists(path):
+        shutil.rmtree(path)
+
+    os.makedirs(path)
+    member_logpath = os.path.join(logpath, 'db' + str(_id) + '.log')
+    if not os.path.exists(os.path.dirname(member_logpath)):
+        os.makedirs(os.path.dirname(member_logpath))
+    cmd = [mongod,
+           '--dbpath', path,
+           '--port', str(cur_port),
+           '--replSet', replica_set_name,
+           '--nojournal', '--oplogSize', '64',
+           '--logappend', '--logpath', member_logpath]
+    if auth:
+        cmd += ['--keyFile', key_file]
+
+    if ha_tools_debug:
+        print('starting %s' % ' '.join(cmd))
+
+    proc = subprocess.Popen(cmd,
+                            stdout=subprocess.PIPE,
+                            stderr=subprocess.STDOUT)
+    nodes[host] = {'proc': proc, 'cmd': cmd, 'dbpath': path}
+    res = wait_for(proc, cur_port)
+
+    cur_port += 1
+
+    config['members'].append(member)
+    config['version'] += 1
+
+    if ha_tools_debug:
+        print({'replSetReconfig': config})
+
+    response = c.admin.command({'replSetReconfig': config})
+    if ha_tools_debug:
+        print(response)
+
+    if not res:
+        return None
+    return host
+
+
 def stepdown_primary():
     primary = get_primary()
     if primary:
         if ha_tools_debug:
             print('stepping down primary: %s' % (primary,))
-        c = pymongo.MongoClient(primary, use_greenlets=use_greenlets)
-        # replSetStepDown causes mongod to close all connections
-        try:
-            c.admin.command('replSetStepDown', 20)
-        except Exception:
-            if ha_tools_debug:
-                exc = sys.exc_info()[1]
-                print('Exception from replSetStepDown: %s' % (exc.message,))
+        c = pymongo.MongoClient(primary)
+
+        for _ in range(10):
+            try:
+                c.admin.command('replSetStepDown', 20)
+            except pymongo.errors.OperationFailure as exc:
+                if ha_tools_debug:
+                    print('Code %s from replSetStepDown: %s' % (exc.code, exc))
+                    print('Trying again in one second....')
+
+                time.sleep(1)
+            except pymongo.errors.ConnectionFailure as exc:
+                # replSetStepDown causes mongod to close all connections.
+                if ha_tools_debug:
+                    print('Exception from replSetStepDown: %s' % exc)
+
+                # Seems to have succeeded.
+                break
+        else:
+            raise AssertionError("Couldn't complete replSetStepDown")
+
         if ha_tools_debug:
             print('\tcalled replSetStepDown')
     elif ha_tools_debug:
@@ -391,7 +477,7 @@ def stepdown_primary():
 def set_maintenance(member, value):
     """Put a member into RECOVERING state if value is True, else normal state.
     """
-    c = pymongo.MongoClient(member, use_greenlets=use_greenlets)
+    c = pymongo.MongoClient(member)
     c.admin.command('replSetMaintenance', value)
     start = time.time()
     while value != (member in get_recovering()):
@@ -408,6 +494,10 @@ def restart_members(members, router=False):
             cmd = routers[member]['cmd']
         else:
             cmd = nodes[member]['cmd']
+            lockfile_path = os.path.join(nodes[member]['dbpath'], 'mongod.lock')
+            if os.path.exists(lockfile_path):
+                os.remove(lockfile_path)
+
         proc = start_subprocess(cmd)
         if router:
             routers[member]['proc'] = proc
