@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,7 +38,7 @@ from bson.codec_options import CodecOptions
 from bson.int64 import Int64
 from bson.objectid import ObjectId
 from bson.dbref import DBRef
-from bson.py3compat import PY3, text_type, iteritems, StringIO
+from bson.py3compat import abc, iteritems, PY3, StringIO, text_type
 from bson.son import SON
 from bson.timestamp import Timestamp
 from bson.tz_util import FixedOffset
@@ -56,7 +56,7 @@ if PY3:
     long = int
 
 
-class NotADict(collections.MutableMapping):
+class NotADict(abc.MutableMapping):
     """Non-dict type that implements the mapping protocol."""
 
     def __init__(self, initial=None):
@@ -81,7 +81,7 @@ class NotADict(collections.MutableMapping):
         return len(self._dict)
 
     def __eq__(self, other):
-        if isinstance(other, collections.Mapping):
+        if isinstance(other, abc.Mapping):
             return all(self.get(k) == other.get(k) for k in self)
         return NotImplemented
 
@@ -788,6 +788,17 @@ class TestBSON(unittest.TestCase):
     def test_regex_hash(self):
         self.assertRaises(TypeError, hash, Regex('hello'))
 
+    def test_regex_comparison(self):
+        re1 = Regex('a')
+        re2 = Regex('b')
+        self.assertNotEqual(re1, re2)
+        re1 = Regex('a', re.I)
+        re2 = Regex('a', re.M)
+        self.assertNotEqual(re1, re2)
+        re1 = Regex('a', re.I)
+        re2 = Regex('a', re.I)
+        self.assertEqual(re1, re2)
+
     def test_exception_wrapping(self):
         # No matter what exception is raised while trying to decode BSON,
         # the final exception always matches InvalidBSON.
@@ -814,7 +825,7 @@ class TestBSON(unittest.TestCase):
         self.assertTrue(MinKey() != 1)
         self.assertFalse(MinKey() == 1)
         self.assertTrue(MinKey() == MinKey())
-        
+
         # MinKey compared to MaxKey.
         self.assertTrue(MinKey() < MaxKey())
         self.assertTrue(MinKey() <= MaxKey())
@@ -822,7 +833,7 @@ class TestBSON(unittest.TestCase):
         self.assertFalse(MinKey() >= MaxKey())
         self.assertTrue(MinKey() != MaxKey())
         self.assertFalse(MinKey() == MaxKey())
-        
+
         # MaxKey's <, <=, >, >=, !=, and ==.
         self.assertFalse(MaxKey() < None)
         self.assertFalse(MaxKey() < 1)
@@ -872,6 +883,14 @@ class TestBSON(unittest.TestCase):
         self.assertTrue(Timestamp(1, 0) <= Timestamp(1, 0))
         self.assertFalse(Timestamp(1, 0) > Timestamp(1, 0))
 
+    def test_timestamp_highorder_bits(self):
+        doc = {'a': Timestamp(0xFFFFFFFF, 0xFFFFFFFF)}
+        doc_bson = (b'\x10\x00\x00\x00'
+                    b'\x11a\x00\xff\xff\xff\xff\xff\xff\xff\xff'
+                    b'\x00')
+        self.assertEqual(doc_bson, BSON.encode(doc))
+        self.assertEqual(doc, BSON(doc_bson).decode())
+
     def test_bad_id_keys(self):
         self.assertRaises(InvalidDocument, BSON.encode,
                           {"_id": {"$bad": 123}}, True)
@@ -927,7 +946,8 @@ class TestCodecOptions(unittest.TestCase):
 
         # Test handling of bad key value.
         invalid_key = BSON(enc[:7] + b'\xe9' + enc[8:])
-        replaced_key = b'ke\xef\xbf\xbdstr'.decode('utf-8')
+        replaced_key = b'ke\xe9str'.decode('utf-8', 'replace')
+        ignored_key = b'ke\xe9str'.decode('utf-8', 'ignore')
 
         dec = BSON.decode(invalid_key, CodecOptions(
             unicode_decode_error_handler="replace"))
@@ -935,7 +955,7 @@ class TestCodecOptions(unittest.TestCase):
 
         dec = BSON.decode(invalid_key, CodecOptions(
             unicode_decode_error_handler="ignore"))
-        self.assertEqual(dec, {"kestr": "foobar"})
+        self.assertEqual(dec, {ignored_key: u"foobar"})
 
         self.assertRaises(InvalidBSON, BSON.decode, invalid_key, CodecOptions(
             unicode_decode_error_handler="strict"))
@@ -945,15 +965,16 @@ class TestCodecOptions(unittest.TestCase):
 
         # Test handing of bad string value.
         invalid_val = BSON(enc[:18] + b'\xe9' + enc[19:])
-        replaced_val = b'fo\xef\xbf\xbdbar'.decode('utf-8')
+        replaced_val = b'fo\xe9bar'.decode('utf-8', 'replace')
+        ignored_val = b'fo\xe9bar'.decode('utf-8', 'ignore')
 
         dec = BSON.decode(invalid_val, CodecOptions(
             unicode_decode_error_handler="replace"))
-        self.assertEqual(dec, {"keystr": replaced_val})
+        self.assertEqual(dec, {u"keystr": replaced_val})
 
         dec = BSON.decode(invalid_val, CodecOptions(
             unicode_decode_error_handler="ignore"))
-        self.assertEqual(dec, {"keystr": "fobar"})
+        self.assertEqual(dec, {u"keystr": ignored_val})
 
         self.assertRaises(InvalidBSON, BSON.decode, invalid_val, CodecOptions(
             unicode_decode_error_handler="strict"))
@@ -971,7 +992,7 @@ class TestCodecOptions(unittest.TestCase):
 
         dec = BSON.decode(invalid_both, CodecOptions(
             unicode_decode_error_handler="ignore"))
-        self.assertEqual(dec, {"kestr": "fobar"})
+        self.assertEqual(dec, {ignored_key: ignored_val})
 
         self.assertRaises(InvalidBSON, BSON.decode, invalid_both, CodecOptions(
             unicode_decode_error_handler="strict"))
