@@ -1,4 +1,4 @@
-# Copyright 2013-2015 MongoDB, Inc.
+# Copyright 2013-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,12 +16,14 @@
 
 import contextlib
 import datetime
+import gc
 import os
+import signal
 import socket
 import struct
 import sys
 import time
-import traceback
+import warnings
 
 sys.path[0:0] = [""]
 
@@ -32,6 +34,7 @@ from bson.son import SON
 from bson.tz_util import utc
 from pymongo import auth, message
 from pymongo.common import _UUID_REPRESENTATIONS
+from pymongo.command_cursor import CommandCursor
 from pymongo.cursor import CursorType
 from pymongo.database import Database
 from pymongo.errors import (AutoReconnect,
@@ -58,22 +61,26 @@ from test import (client_context,
                   IntegrationTest,
                   db_pwd,
                   db_user,
-                  MockClientTest)
+                  MockClientTest,
+                  HAVE_IPADDRESS)
 from test.pymongo_mocks import MockClient
 from test.utils import (assertRaisesExactly,
-                        delay,
-                        remove_all_users,
-                        server_is_master_with_slave,
-                        get_pool,
-                        one,
                         connected,
-                        wait_until,
+                        delay,
+                        get_pool,
+                        gevent_monkey_patched,
+                        ignore_deprecations,
+                        is_greenthread_patched,
+                        lazy_client_trial,
+                        NTHREADS,
+                        one,
+                        remove_all_users,
                         rs_client,
                         rs_or_single_client,
                         rs_or_single_client_noauth,
+                        server_is_master_with_slave,
                         single_client,
-                        lazy_client_trial,
-                        NTHREADS)
+                        wait_until)
 
 
 class ClientUnitTest(unittest.TestCase):
@@ -90,7 +97,6 @@ class ClientUnitTest(unittest.TestCase):
                              connectTimeoutMS=20000,
                              waitQueueTimeoutMS=None,
                              waitQueueMultiple=None,
-                             socketKeepAlive=False,
                              replicaSet=None,
                              read_preference=ReadPreference.PRIMARY,
                              ssl=False,
@@ -108,7 +114,7 @@ class ClientUnitTest(unittest.TestCase):
         self.assertEqual(20.0, pool_opts.connect_timeout)
         self.assertEqual(None, pool_opts.wait_queue_timeout)
         self.assertEqual(None, pool_opts.wait_queue_multiple)
-        self.assertFalse(pool_opts.socket_keepalive)
+        self.assertTrue(pool_opts.socket_keepalive)
         self.assertEqual(None, pool_opts.ssl_context)
         self.assertEqual(None, options.replica_set_name)
         self.assertEqual(ReadPreference.PRIMARY, client.read_preference)
@@ -170,25 +176,25 @@ class ClientUnitTest(unittest.TestCase):
 
         self.assertRaises(TypeError, iterate)
 
-    def test_get_default_database(self):
+    def test_get_database_default(self):
         c = rs_or_single_client("mongodb://%s:%d/foo" % (client_context.host,
                                                          client_context.port),
                                 connect=False)
-        self.assertEqual(Database(c, 'foo'), c.get_default_database())
+        self.assertEqual(Database(c, 'foo'), c.get_database())
 
-    def test_get_default_database_error(self):
+    def test_get_database_default_error(self):
         # URI with no database.
         c = rs_or_single_client("mongodb://%s:%d/" % (client_context.host,
                                                       client_context.port),
                                 connect=False)
-        self.assertRaises(ConfigurationError, c.get_default_database)
+        self.assertRaises(ConfigurationError, c.get_database)
 
-    def test_get_default_database_with_authsource(self):
+    def test_get_database_default_with_authsource(self):
         # Ensure we distinguish database name from authSource.
         uri = "mongodb://%s:%d/foo?authSource=src" % (
             client_context.host, client_context.port)
         c = rs_or_single_client(uri, connect=False)
-        self.assertEqual(Database(c, 'foo'), c.get_default_database())
+        self.assertEqual(Database(c, 'foo'), c.get_database())
 
     def test_primary_read_pref_with_tags(self):
         # No tags allowed with "primary".
@@ -199,6 +205,12 @@ class ClientUnitTest(unittest.TestCase):
             MongoClient('mongodb://host/?'
                         'readpreference=primary&readpreferencetags=dc:east')
 
+    def test_read_preference(self):
+        c = rs_or_single_client(
+            "mongodb://host", connect=False,
+            readpreference=ReadPreference.NEAREST.mongos_mode)
+        self.assertEqual(c.read_preference, ReadPreference.NEAREST)
+
     def test_metadata(self):
         metadata = _METADATA.copy()
         metadata['application'] = {'name': 'foobar'}
@@ -271,27 +283,53 @@ class TestClient(IntegrationTest):
                 pass
             self.assertEqual(1, len(server._pool.sockets))
             self.assertTrue(sock_info in server._pool.sockets)
+            client.close()
 
             # Assert reaper removes idle socket and replaces it with a new one
-            client = rs_or_single_client(maxIdleTimeMS=.5,
+            client = rs_or_single_client(maxIdleTimeMS=500,
                                          minPoolSize=1)
             server = client._get_topology().select_server(any_server_selector)
             with server._pool.get_socket({}) as sock_info:
                 pass
+            # When the reaper runs at the same time as the get_socket, two
+            # sockets could be created and checked into the pool.
+            self.assertGreaterEqual(len(server._pool.sockets), 1)
+            wait_until(lambda: sock_info not in server._pool.sockets,
+                       "remove stale socket")
+            wait_until(lambda: 1 <= len(server._pool.sockets),
+                       "replace stale socket")
+            client.close()
+
+            # Assert reaper respects maxPoolSize when adding new sockets.
+            client = rs_or_single_client(maxIdleTimeMS=500,
+                                         minPoolSize=1,
+                                         maxPoolSize=1)
+            server = client._get_topology().select_server(any_server_selector)
+            with server._pool.get_socket({}) as sock_info:
+                pass
+            # When the reaper runs at the same time as the get_socket,
+            # maxPoolSize=1 should prevent two sockets from being created.
             self.assertEqual(1, len(server._pool.sockets))
             wait_until(lambda: sock_info not in server._pool.sockets,
-                       "reaper removes stale socket eventually")
+                       "remove stale socket")
             wait_until(lambda: 1 == len(server._pool.sockets),
-                       "reaper replaces stale socket with new one")
+                       "replace stale socket")
+            client.close()
 
             # Assert reaper has removed idle socket and NOT replaced it
-            client = rs_or_single_client(maxIdleTimeMS=.5)
+            client = rs_or_single_client(maxIdleTimeMS=500)
             server = client._get_topology().select_server(any_server_selector)
-            with server._pool.get_socket({}):
+            with server._pool.get_socket({}) as sock_info_one:
                 pass
+            # Assert that the pool does not close sockets prematurely.
+            time.sleep(.300)
+            with server._pool.get_socket({}) as sock_info_two:
+                pass
+            self.assertIs(sock_info_one, sock_info_two)
             wait_until(
                 lambda: 0 == len(server._pool.sockets),
                 "stale socket reaped and new one NOT added to the pool")
+            client.close()
 
     def test_min_pool_size(self):
         with client_knobs(kill_cursor_frequency=.1):
@@ -315,7 +353,7 @@ class TestClient(IntegrationTest):
     def test_max_idle_time_checkout(self):
         # Use high frequency to test _get_socket_no_auth.
         with client_knobs(kill_cursor_frequency=99999999):
-            client = rs_or_single_client(maxIdleTimeMS=.5)
+            client = rs_or_single_client(maxIdleTimeMS=500)
             server = client._get_topology().select_server(any_server_selector)
             with server._pool.get_socket({}) as sock_info:
                 pass
@@ -344,23 +382,27 @@ class TestClient(IntegrationTest):
         port are not overloaded.
         """
         host, port = client_context.host, client_context.port
+        kwargs = client_context.ssl_client_options.copy()
+        if client_context.auth_enabled:
+            kwargs['username'] = db_user
+            kwargs['password'] = db_pwd
+
         # Set bad defaults.
         MongoClient.HOST = "somedomainthatdoesntexist.org"
         MongoClient.PORT = 123456789
         with self.assertRaises(AutoReconnect):
             connected(MongoClient(serverSelectionTimeoutMS=10,
-                                  **client_context.ssl_client_options))
+                                  **kwargs))
 
         # Override the defaults. No error.
-        connected(MongoClient(host, port,
-                              **client_context.ssl_client_options))
+        connected(MongoClient(host, port, **kwargs))
 
         # Set good defaults.
         MongoClient.HOST = host
         MongoClient.PORT = port
 
         # No error.
-        connected(MongoClient(**client_context.ssl_client_options))
+        connected(MongoClient(**kwargs))
 
     def test_init_disconnected(self):
         host, port = client_context.host, client_context.port
@@ -459,16 +501,53 @@ class TestClient(IntegrationTest):
         self.assertEqual(eval(the_repr), client)
 
     def test_getters(self):
+        if (client_context.version >= (3, 7, 2)
+                and client_context.auth_enabled and client_context.is_rs):
+            raise SkipTest("Disabled due to SERVER-32845")
         wait_until(lambda: client_context.nodes == self.client.nodes,
                    "find all nodes")
 
-    def test_database_names(self):
+    def test_list_databases(self):
+        cmd_docs = self.client.admin.command('listDatabases')['databases']
+        cursor = self.client.list_databases()
+        self.assertIsInstance(cursor, CommandCursor)
+        helper_docs = list(cursor)
+        self.assertTrue(len(helper_docs) > 0)
+        self.assertEqual(helper_docs, cmd_docs)
+        for doc in helper_docs:
+            self.assertIs(type(doc), dict)
+        client = rs_or_single_client(document_class=SON)
+        for doc in client.list_databases():
+            self.assertIs(type(doc), dict)
+
+        if client_context.version.at_least(3, 4, 2):
+            self.client.pymongo_test.test.insert_one({})
+            cursor = self.client.list_databases(filter={"name": "admin"})
+            docs = list(cursor)
+            self.assertEqual(1, len(docs))
+            self.assertEqual(docs[0]["name"], "admin")
+
+        if client_context.version.at_least(3, 4, 3):
+            cursor = self.client.list_databases(nameOnly=True)
+            for doc in cursor:
+                self.assertEqual(["name"], list(doc))
+
+    def _test_list_names(self, meth):
         self.client.pymongo_test.test.insert_one({"dummy": u"object"})
         self.client.pymongo_test_mike.test.insert_one({"dummy": u"object"})
+        cmd_docs = self.client.admin.command("listDatabases")["databases"]
+        cmd_names = [doc["name"] for doc in cmd_docs]
 
-        dbs = self.client.database_names()
-        self.assertTrue("pymongo_test" in dbs)
-        self.assertTrue("pymongo_test_mike" in dbs)
+        db_names = meth()
+        self.assertTrue("pymongo_test" in db_names)
+        self.assertTrue("pymongo_test_mike" in db_names)
+        self.assertEqual(db_names, cmd_names)
+
+    def test_list_database_names(self):
+        self._test_list_names(self.client.list_database_names)
+
+    def test_database_names(self):
+        self._test_list_names(self.client.database_names)
 
     def test_drop_database(self):
         self.assertRaises(TypeError, self.client.drop_database, 5)
@@ -507,6 +586,41 @@ class TestClient(IntegrationTest):
 
         coll.count()
 
+    def test_close_kills_cursors(self):
+        if sys.platform.startswith('java'):
+            # We can't figure out how to make this test reliable with Jython.
+            raise SkipTest("Can't test with Jython")
+        # Kill any cursors possibly queued up by previous tests.
+        gc.collect()
+        self.client._process_periodic_tasks()
+
+        # Add some test data.
+        coll = self.client.pymongo_test.test_close_kills_cursors
+        docs_inserted = 1000
+        coll.insert_many([{"i": i} for i in range(docs_inserted)])
+
+        # Open a cursor and leave it open on the server.
+        cursor = coll.find().batch_size(10)
+        self.assertTrue(bool(next(cursor)))
+        self.assertLess(cursor.retrieved, docs_inserted)
+
+        # Open a command cursor and leave it open on the server.
+        cursor = coll.aggregate([], batchSize=10)
+        self.assertTrue(bool(next(cursor)))
+        del cursor
+        # Required for PyPy, Jython and other Python implementations that
+        # don't use reference counting garbage collection.
+        gc.collect()
+
+        # Close the client and ensure the topology is closed.
+        self.assertTrue(self.client._topology._opened)
+        self.client.close()
+        self.assertFalse(self.client._topology._opened)
+
+        # The killCursors task should not need to re-open the topology.
+        self.client._process_periodic_tasks()
+        self.assertFalse(self.client._topology._opened)
+
     def test_bad_uri(self):
         with self.assertRaises(InvalidURI):
             MongoClient("http://localhost")
@@ -514,12 +628,12 @@ class TestClient(IntegrationTest):
     @client_context.require_auth
     def test_auth_from_uri(self):
         host, port = client_context.host, client_context.port
-        self.client.admin.add_user("admin", "pass", roles=["root"])
-        self.addCleanup(self.client.admin.remove_user, 'admin')
+        client_context.create_user("admin", "admin", "pass")
+        self.addCleanup(client_context.drop_user, "admin", "admin")
         self.addCleanup(remove_all_users, self.client.pymongo_test)
 
-        self.client.pymongo_test.add_user(
-            "user", "pass", roles=['userAdmin', 'readWrite'])
+        client_context.create_user(
+            "pymongo_test", "user", "pass", roles=['userAdmin', 'readWrite'])
 
         with self.assertRaises(OperationFailure):
             connected(rs_or_single_client(
@@ -552,9 +666,31 @@ class TestClient(IntegrationTest):
                           bad_client.pymongo_test.test.find_one)
 
     @client_context.require_auth
+    def test_username_and_password(self):
+        client_context.create_user("admin", "ad min", "pa/ss")
+        self.addCleanup(client_context.drop_user, "admin", "ad min")
+
+        c = rs_or_single_client(username="ad min", password="pa/ss")
+
+        # Username and password aren't in strings that will likely be logged.
+        self.assertNotIn("ad min", repr(c))
+        self.assertNotIn("ad min", str(c))
+        self.assertNotIn("pa/ss", repr(c))
+        self.assertNotIn("pa/ss", str(c))
+
+        # Auth succeeds.
+        c.server_info()
+
+        with self.assertRaises(OperationFailure):
+            rs_or_single_client(username="ad min", password="foo").server_info()
+
+    @client_context.require_auth
+    @ignore_deprecations
     def test_multiple_logins(self):
-        self.client.pymongo_test.add_user('user1', 'pass', roles=['readWrite'])
-        self.client.pymongo_test.add_user('user2', 'pass', roles=['readWrite'])
+        client_context.create_user(
+            'pymongo_test', 'user1', 'pass', roles=['readWrite'])
+        client_context.create_user(
+            'pymongo_test', 'user2', 'pass', roles=['readWrite'])
         self.addCleanup(remove_all_users, self.client.pymongo_test)
 
         client = rs_or_single_client_noauth(
@@ -588,6 +724,7 @@ class TestClient(IntegrationTest):
         assertRaisesExactly(
             OperationFailure, lazy_client.test.collection.find_one)
 
+    @client_context.require_no_ssl
     def test_unix_socket(self):
         if not hasattr(socket, "AF_UNIX"):
             raise SkipTest("UNIX-sockets are not supported on this system")
@@ -633,10 +770,16 @@ class TestClient(IntegrationTest):
         self.assertTrue(isinstance(db.test.find_one(), SON))
 
     def test_timeouts(self):
-        client = rs_or_single_client(connectTimeoutMS=10500)
+        client = rs_or_single_client(
+            connectTimeoutMS=10500,
+            socketTimeoutMS=10500,
+            maxIdleTimeMS=10500,
+            serverSelectionTimeoutMS=10500)
         self.assertEqual(10.5, get_pool(client).opts.connect_timeout)
-        client = rs_or_single_client(socketTimeoutMS=10500)
         self.assertEqual(10.5, get_pool(client).opts.socket_timeout)
+        self.assertEqual(10.5, get_pool(client).opts.max_idle_time_seconds)
+        self.assertEqual(10500, client.max_idle_time_ms)
+        self.assertEqual(10.5, client.server_selection_timeout)
 
     def test_socket_timeout_ms_validation(self):
         c = rs_or_single_client(socketTimeoutMS=10 * 1000)
@@ -716,8 +859,19 @@ class TestClient(IntegrationTest):
         self.assertEqual(pool._socket_semaphore.waiter_semaphore.counter, 6)
 
     def test_socketKeepAlive(self):
-        client = rs_or_single_client(socketKeepAlive=True)
-        self.assertTrue(get_pool(client).opts.socket_keepalive)
+        for socketKeepAlive in [True, False]:
+            with warnings.catch_warnings(record=True) as ctx:
+                warnings.simplefilter("always")
+                client = rs_or_single_client(socketKeepAlive=socketKeepAlive)
+                self.assertIn("The socketKeepAlive option is deprecated",
+                              str(ctx[0]))
+                pool = get_pool(client)
+                self.assertEqual(socketKeepAlive,
+                                 pool.opts.socket_keepalive)
+                with pool.get_socket({}) as sock_info:
+                    keepalive = sock_info.sock.getsockopt(socket.SOL_SOCKET,
+                                                          socket.SO_KEEPALIVE)
+                    self.assertEqual(socketKeepAlive, bool(keepalive))
 
     def test_tz_aware(self):
         self.assertRaises(ValueError, MongoClient, tz_aware='foo')
@@ -737,6 +891,13 @@ class TestClient(IntegrationTest):
 
     @client_context.require_ipv6
     def test_ipv6(self):
+        if client_context.ssl:
+            # http://bugs.python.org/issue13034
+            if sys.version_info[:2] == (2, 6):
+                raise SkipTest("Python 2.6 can't parse SANs")
+            if not HAVE_IPADDRESS:
+                raise SkipTest("Need the ipaddress module to test with SSL")
+
         if client_context.auth_enabled:
             auth_str = "%s:%s@" % (db_user, db_pwd)
         else:
@@ -756,14 +917,15 @@ class TestClient(IntegrationTest):
 
     @client_context.require_no_mongos
     def test_fsync_lock_unlock(self):
-        if (server_is_master_with_slave(client_context.client) and
-                client_context.version.at_least(2, 3, 0)):
+        if server_is_master_with_slave(client_context.client):
             raise SkipTest('SERVER-7714')
 
         self.assertFalse(self.client.is_locked)
         # async flushing not supported on windows...
         if sys.platform not in ('cygwin', 'win32'):
-            self.client.fsync(async=True)
+            # Work around async becoming a reserved keyword in Python 3.7
+            opts = {'async': True}
+            self.client.fsync(**opts)
             self.assertFalse(self.client.is_locked)
         self.client.fsync(lock=True)
         self.assertTrue(self.client.is_locked)
@@ -801,6 +963,8 @@ class TestClient(IntegrationTest):
             # without simply killing the whole thread in Jython. This suggests
             # PYTHON-294 can't actually occur in Jython.
             raise SkipTest("Can't test interrupts in Jython")
+        if is_greenthread_patched():
+            raise SkipTest("Can't reliably test interrupts with green threads")
 
         # Test fix for PYTHON-294 -- make sure MongoClient closes its
         # socket if it gets an interrupt while waiting to recv() from it.
@@ -813,44 +977,69 @@ class TestClient(IntegrationTest):
         db.drop_collection('foo')
         db.foo.insert_one({'_id': 1})
 
-        def interrupter():
-            # Raises KeyboardInterrupt in the main thread
-            time.sleep(0.25)
-            thread.interrupt_main()
-
-        thread.start_new_thread(interrupter, ())
-
-        raised = False
+        old_signal_handler = None
         try:
-            # Will be interrupted by a KeyboardInterrupt.
-            next(db.foo.find({'$where': where}))
-        except KeyboardInterrupt:
-            raised = True
+            # Platform-specific hacks for raising a KeyboardInterrupt on the
+            # main thread while find() is in-progress: On Windows, SIGALRM is
+            # unavailable so we use a second thread. In our Evergreen setup on
+            # Linux, the thread technique causes an error in the test at
+            # sock.recv(): TypeError: 'int' object is not callable
+            # We don't know what causes this, so we hack around it.
 
-        # Can't use self.assertRaises() because it doesn't catch system
-        # exceptions
-        self.assertTrue(raised, "Didn't raise expected KeyboardInterrupt")
+            if sys.platform == 'win32':
+                def interrupter():
+                    # Raises KeyboardInterrupt in the main thread
+                    time.sleep(0.25)
+                    thread.interrupt_main()
 
-        # Raises AssertionError due to PYTHON-294 -- Mongo's response to the
-        # previous find() is still waiting to be read on the socket, so the
-        # request id's don't match.
-        self.assertEqual(
-            {'_id': 1},
-            next(db.foo.find())
-        )
+                thread.start_new_thread(interrupter, ())
+            else:
+                # Convert SIGALRM to SIGINT -- it's hard to schedule a SIGINT
+                # for one second in the future, but easy to schedule SIGALRM.
+                def sigalarm(num, frame):
+                    raise KeyboardInterrupt
+
+                old_signal_handler = signal.signal(signal.SIGALRM, sigalarm)
+                signal.alarm(1)
+
+            raised = False
+            try:
+                # Will be interrupted by a KeyboardInterrupt.
+                next(db.foo.find({'$where': where}))
+            except KeyboardInterrupt:
+                raised = True
+
+            # Can't use self.assertRaises() because it doesn't catch system
+            # exceptions
+            self.assertTrue(raised, "Didn't raise expected KeyboardInterrupt")
+
+            # Raises AssertionError due to PYTHON-294 -- Mongo's response to
+            # the previous find() is still waiting to be read on the socket,
+            # so the request id's don't match.
+            self.assertEqual(
+                {'_id': 1},
+                next(db.foo.find())
+            )
+        finally:
+            if old_signal_handler:
+                signal.signal(signal.SIGALRM, old_signal_handler)
 
     def test_operation_failure(self):
         # Ensure MongoClient doesn't close socket after it gets an error
-        # response to getLastError. PYTHON-395.
-        pool = get_pool(self.client)
+        # response to getLastError. PYTHON-395. We need a new client here
+        # to avoid race conditions caused by replica set failover or idle
+        # socket reaping.
+        client = single_client()
+        client.pymongo_test.test.find_one()
+        pool = get_pool(client)
         socket_count = len(pool.sockets)
         self.assertGreaterEqual(socket_count, 1)
         old_sock_info = next(iter(pool.sockets))
-        self.client.pymongo_test.test.drop()
-        self.client.pymongo_test.test.insert_one({'_id': 'foo'})
+        client.pymongo_test.test.drop()
+        client.pymongo_test.test.insert_one({'_id': 'foo'})
         self.assertRaises(
             OperationFailure,
-            self.client.pymongo_test.test.insert_one, {'_id': 'foo'})
+            client.pymongo_test.test.insert_one, {'_id': 'foo'})
 
         self.assertEqual(socket_count, len(pool.sockets))
         new_sock_info = next(iter(pool.sockets))
@@ -940,7 +1129,8 @@ class TestClient(IntegrationTest):
                                serverSelectionTimeoutMS=100)
             client._send_message_with_response(
                 operation=message._GetMore('pymongo_test', 'collection',
-                                           101, 1234, client.codec_options),
+                                           101, 1234, client.codec_options,
+                                           None, client),
                 address=('not-a-member', 27017))
 
     def test_heartbeat_frequency_ms(self):
@@ -958,10 +1148,11 @@ class TestClient(IntegrationTest):
                 pass
 
         old_init = ServerHeartbeatStartedEvent.__init__
+        heartbeat_times = []
 
         def init(self, *args):
             old_init(self, *args)
-            self.time = time.time()
+            heartbeat_times.append(time.time())
 
         try:
             ServerHeartbeatStartedEvent.__init__ = init
@@ -972,12 +1163,10 @@ class TestClient(IntegrationTest):
             wait_until(lambda: len(listener.results) >= 2,
                        "record two ServerHeartbeatStartedEvents")
 
-            events = listener.results
-
             # Default heartbeatFrequencyMS is 10 sec. Check the interval was
             # closer to 0.5 sec with heartbeatFrequencyMS configured.
             self.assertAlmostEqual(
-                events[1].time - events[0].time, 0.5, delta=2)
+                heartbeat_times[1] - heartbeat_times[0], 0.5, delta=2)
 
             client.close()
         finally:
@@ -999,10 +1188,6 @@ class TestExhaustCursor(IntegrationTest):
         if client_context.is_mongos:
             raise SkipTest("mongos doesn't support exhaust, SERVER-2627")
 
-    # mongod < 2.2.0 closes exhaust socket on error, so it behaves like
-    # test_exhaust_query_network_error. Here we test that on query error
-    # the client correctly keeps the socket *open* and checks it in.
-    @client_context.require_version_min(2, 2, 0)
     def test_exhaust_query_server_error(self):
         # When doing an exhaust query, the socket stays checked out on success
         # but must be checked in on error to avoid semaphore leaks.
@@ -1045,14 +1230,14 @@ class TestExhaustCursor(IntegrationTest):
         cursor.next()
 
         # Cause a server error on getmore.
-        def receive_message(operation, request_id):
+        def receive_message(request_id):
             # Discard the actual server response.
-            SocketInfo.receive_message(sock_info, operation, request_id)
+            SocketInfo.receive_message(sock_info, request_id)
 
             # responseFlags bit 1 is QueryFailure.
             msg = struct.pack('<iiiii', 1 << 1, 0, 0, 0, 0)
             msg += BSON.encode({'$err': 'mock err', 'code': 0})
-            return msg
+            return message._OpReply.unpack(msg)
 
         saved = sock_info.receive_message
         sock_info.receive_message = receive_message
@@ -1254,8 +1439,8 @@ class TestMongoClientFailover(MockClientTest):
                 connect=False)
 
             # Set host-specific information so we can test whether it is reset.
-            c.set_wire_version_range('a:1', 0, 1)
-            c.set_wire_version_range('b:2', 0, 2)
+            c.set_wire_version_range('a:1', 2, 6)
+            c.set_wire_version_range('b:2', 2, 7)
             c._get_topology().select_servers(writable_server_selector)
             wait_until(lambda: len(c.nodes) == 2, 'connect')
 
@@ -1275,8 +1460,8 @@ class TestMongoClientFailover(MockClientTest):
             server_b = c._get_topology().get_server_by_address(('b', 2))
             sd_b = server_b.description
             self.assertEqual(SERVER_TYPE.RSSecondary, sd_b.server_type)
-            self.assertEqual(0, sd_b.min_wire_version)
-            self.assertEqual(2, sd_b.max_wire_version)
+            self.assertEqual(2, sd_b.min_wire_version)
+            self.assertEqual(7, sd_b.max_wire_version)
 
     def test_network_error_on_query(self):
         callback = lambda client: client.db.collection.find_one()
@@ -1299,6 +1484,18 @@ class TestMongoClientFailover(MockClientTest):
         callback = lambda client: client.db.collection.delete_many({})
         self._test_network_error(callback)
 
+    def test_gevent_task(self):
+        if not gevent_monkey_patched():
+            raise SkipTest("Must be running monkey patched by gevent")
+        from gevent import spawn
+        def poller():
+            while True:
+                client_context.client.pymongo_test.test.insert_one({})
+
+        task = spawn(poller)
+        task.kill()
+        self.assertTrue(task.dead)
+
 
 if __name__ == "__main__":
     unittest.main()
