@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2009-2015 MongoDB, Inc.
+# Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ import re
 import sys
 import threading
 
+from codecs import utf_8_decode
 from collections import defaultdict
 
 sys.path[0:0] = [""]
@@ -41,7 +42,8 @@ from pymongo.bulk import BulkWriteError
 from pymongo.collection import Collection, ReturnDocument
 from pymongo.command_cursor import CommandCursor
 from pymongo.cursor import CursorType
-from pymongo.errors import (DocumentTooLarge,
+from pymongo.errors import (ConfigurationError,
+                            DocumentTooLarge,
                             DuplicateKeyError,
                             ExecutionTimeout,
                             InvalidDocument,
@@ -52,6 +54,7 @@ from pymongo.errors import (DocumentTooLarge,
 from pymongo.message import _COMMAND_OVERHEAD, _gen_find_command
 from pymongo.mongo_client import MongoClient
 from pymongo.operations import *
+from pymongo.read_concern import DEFAULT_READ_CONCERN
 from pymongo.read_preferences import ReadPreference
 from pymongo.results import (InsertOneResult,
                              InsertManyResult,
@@ -59,7 +62,7 @@ from pymongo.results import (InsertOneResult,
                              DeleteResult)
 from pymongo.write_concern import WriteConcern
 from test.test_client import IntegrationTest
-from test.utils import (is_mongos, enable_text_search, get_pool,
+from test.utils import (is_mongos, get_pool,
                         rs_or_single_client, single_client,
                         wait_until, EventListener,
                         IMPOSSIBLE_WRITE_CONCERN)
@@ -144,12 +147,19 @@ class TestCollection(IntegrationTest):
         # No Exception.
         db = client_context.client.pymongo_test
         db.create_test_no_wc.drop()
+        wait_until(lambda: 'create_test_no_wc' not in db.collection_names(),
+                 'drop create_test_no_wc collection')
         Collection(db, name='create_test_no_wc', create=True)
-        with self.assertRaises(OperationFailure):
-            Collection(
-                db, name='create-test-wc',
-                write_concern=IMPOSSIBLE_WRITE_CONCERN,
-                create=True)
+        wait_until(lambda: 'create_test_no_wc' in db.collection_names(),
+                 'create create_test_no_wc collection')
+        # SERVER-33317
+        if (not client_context.is_mongos or not
+                client_context.version.at_least(3, 7, 0)):
+            with self.assertRaises(OperationFailure):
+                Collection(
+                    db, name='create-test-wc',
+                    write_concern=IMPOSSIBLE_WRITE_CONCERN,
+                    create=True)
 
     def test_drop_nonexistent_collection(self):
         self.db.drop_collection('test')
@@ -158,7 +168,6 @@ class TestCollection(IntegrationTest):
         # No exception
         self.db.drop_collection('test')
 
-    @client_context.require_version_min(2, 6)
     def test_create_indexes(self):
         db = self.db
 
@@ -283,6 +292,35 @@ class TestCollection(IntegrationTest):
         with self.write_concern_collection() as coll:
             coll.drop_index('hello_1')
 
+    @client_context.require_no_mongos
+    @client_context.require_test_commands
+    def test_index_management_max_time_ms(self):
+        if (client_context.version[:2] == (3, 4) and
+                client_context.version[2] < 4):
+            raise unittest.SkipTest("SERVER-27711")
+        coll = self.db.test
+        self.client.admin.command("configureFailPoint",
+                                  "maxTimeAlwaysTimeOut",
+                                  mode="alwaysOn")
+        try:
+            self.assertRaises(
+                ExecutionTimeout, coll.create_index, "foo", maxTimeMS=1)
+            self.assertRaises(
+                ExecutionTimeout,
+                coll.create_indexes,
+                [IndexModel("foo")],
+                maxTimeMS=1)
+            self.assertRaises(
+                ExecutionTimeout, coll.drop_index, "foo", maxTimeMS=1)
+            self.assertRaises(
+                ExecutionTimeout, coll.drop_indexes, maxTimeMS=1)
+            self.assertRaises(
+                ExecutionTimeout, coll.reindex, maxTimeMS=1)
+        finally:
+            self.client.admin.command("configureFailPoint",
+                                      "maxTimeAlwaysTimeOut",
+                                      mode="off")
+
     def test_reindex(self):
         db = self.db
         db.drop_collection("test")
@@ -309,8 +347,12 @@ class TestCollection(IntegrationTest):
         else:
             check_result(reindexed)
 
-        with self.write_concern_collection() as coll:
-            coll.reindex()
+        coll = Collection(
+            self.db,
+            'test',
+            write_concern=WriteConcern(w=100))
+        # No error since writeConcern is not sent.
+        coll.reindex()
 
     def test_list_indexes(self):
         db = self.db
@@ -339,6 +381,14 @@ class TestCollection(IntegrationTest):
                          SON([("hello", DESCENDING), ("world", ASCENDING)]))
         self.assertEqual(True, index_map["hello_-1_world_1"]["unique"])
 
+        # List indexes on a collection that does not exist.
+        indexes = list(db.does_not_exist.list_indexes())
+        self.assertEqual(len(indexes), 0)
+
+        # List indexes on a database that does not exist.
+        indexes = list(self.client.db_does_not_exist.coll.list_indexes())
+        self.assertEqual(len(indexes), 0)
+
     def test_index_info(self):
         db = self.db
         db.test.drop()
@@ -403,36 +453,31 @@ class TestCollection(IntegrationTest):
             "type": "restaurant"
         }, results[0])
 
-    @client_context.require_version_min(2, 3, 2)
     @client_context.require_no_mongos
     def test_index_text(self):
-        enable_text_search(self.client)
-
         db = self.db
         db.test.drop_indexes()
         self.assertEqual("t_text", db.test.create_index([("t", TEXT)]))
         index_info = db.test.index_information()["t_text"]
         self.assertTrue("weights" in index_info)
 
-        if client_context.version.at_least(2, 5, 5):
-            db.test.insert_many([
-                {'t': 'spam eggs and spam'},
-                {'t': 'spam'},
-                {'t': 'egg sausage and bacon'}])
+        db.test.insert_many([
+            {'t': 'spam eggs and spam'},
+            {'t': 'spam'},
+            {'t': 'egg sausage and bacon'}])
 
-            # MongoDB 2.6 text search. Create 'score' field in projection.
-            cursor = db.test.find(
-                {'$text': {'$search': 'spam'}},
-                {'score': {'$meta': 'textScore'}})
+        # MongoDB 2.6 text search. Create 'score' field in projection.
+        cursor = db.test.find(
+            {'$text': {'$search': 'spam'}},
+            {'score': {'$meta': 'textScore'}})
 
-            # Sort by 'score' field.
-            cursor.sort([('score', {'$meta': 'textScore'})])
-            results = list(cursor)
-            self.assertTrue(results[0]['score'] >= results[1]['score'])
+        # Sort by 'score' field.
+        cursor.sort([('score', {'$meta': 'textScore'})])
+        results = list(cursor)
+        self.assertTrue(results[0]['score'] >= results[1]['score'])
 
         db.test.drop_indexes()
 
-    @client_context.require_version_min(2, 3, 2)
     def test_index_2dsphere(self):
         db = self.db
         db.test.drop_indexes()
@@ -454,7 +499,6 @@ class TestCollection(IntegrationTest):
         db.test.find(query)
         db.test.drop_indexes()
 
-    @client_context.require_version_min(2, 3, 2)
     def test_index_hashed(self):
         db = self.db
         db.test.drop_indexes()
@@ -1189,7 +1233,7 @@ class TestCollection(IntegrationTest):
                 b'\xe2\x98\x83\xe2\x98\x83\xe2\x98\x83\xe2\x98\x83'
                 b'\xe2\x98\x83\xe2\x98\x83\xe2\x98\x83\xe2\x98\x83')
 
-        text = data.decode("utf8")
+        text = utf_8_decode(data, None, True)
         db.test.insert_one({"text": text})
 
         # Should raise DuplicateKeyError, not InvalidBSON
@@ -1396,26 +1440,28 @@ class TestCollection(IntegrationTest):
         self.assertRaises(InvalidOperation, lambda: result.upserted_id)
         self.assertFalse(result.acknowledged)
 
+    # MongoDB >= 3.5.8 allows dotted fields in updates
+    @client_context.require_version_max(3, 5, 7)
     def test_update_with_invalid_keys(self):
         self.db.drop_collection("test")
         self.assertTrue(self.db.test.insert_one({"hello": "world"}))
         doc = self.db.test.find_one()
         doc['a.b'] = 'c'
 
-        expected = InvalidDocument
-        if client_context.version.at_least(2, 5, 4, -1):
-            expected = OperationFailure
-
         # Replace
-        self.assertRaises(expected, self.db.test.replace_one,
+        self.assertRaises(OperationFailure, self.db.test.replace_one,
                           {"hello": "world"}, doc)
         # Upsert
-        self.assertRaises(expected, self.db.test.replace_one,
+        self.assertRaises(OperationFailure, self.db.test.replace_one,
                           {"foo": "bar"}, doc, upsert=True)
 
         # Check that the last two ops didn't actually modify anything
         self.assertTrue('a.b' not in self.db.test.find_one())
 
+    def test_update_check_keys(self):
+        self.db.drop_collection("test")
+        self.assertTrue(self.db.test.insert_one({"hello": "world"}))
+
         # Modify shouldn't check keys...
         self.assertTrue(self.db.test.update_one({"hello": "world"},
                                                 {"$set": {"foo.bar": "baz"}},
@@ -1432,7 +1478,7 @@ class TestCollection(IntegrationTest):
         # doesn't change. If the behavior changes checking the first key for
         # '$' in update won't be good enough anymore.
         doc = SON([("hello", "world"), ("$set", {"foo.bar": "bim"})])
-        self.assertRaises(expected, self.db.test.replace_one,
+        self.assertRaises(OperationFailure, self.db.test.replace_one,
                           {"hello": "world"}, doc, upsert=True)
 
         # Replace with empty document
@@ -1483,16 +1529,19 @@ class TestCollection(IntegrationTest):
         self.assertRaises(TypeError, db.test.aggregate, "wow")
 
         pipeline = {"$project": {"_id": False, "foo": True}}
-        result = db.test.aggregate([pipeline], useCursor=False)
+        # MongoDB 3.5.1+ requires either the 'cursor' or 'explain' options.
+        if client_context.version.at_least(3, 5, 1):
+            result = db.test.aggregate([pipeline])
+        else:
+            result = db.test.aggregate([pipeline], useCursor=False)
 
         self.assertTrue(isinstance(result, CommandCursor))
         self.assertEqual([{'foo': [1, 2]}], list(result))
 
         # Test write concern.
-        if client_context.version.at_least(2, 6):
-            out_pipeline = [pipeline, {'$out': 'output-collection'}]
-            with self.write_concern_collection() as coll:
-                coll.aggregate(out_pipeline)
+        out_pipeline = [pipeline, {'$out': 'output-collection'}]
+        with self.write_concern_collection() as coll:
+            coll.aggregate(out_pipeline)
 
     def test_aggregate_raw_bson(self):
         db = self.db
@@ -1502,16 +1551,19 @@ class TestCollection(IntegrationTest):
         self.assertRaises(TypeError, db.test.aggregate, "wow")
 
         pipeline = {"$project": {"_id": False, "foo": True}}
-        result = db.get_collection(
+        coll = db.get_collection(
             'test',
-            codec_options=CodecOptions(document_class=RawBSONDocument)
-        ).aggregate([pipeline], useCursor=False)
+            codec_options=CodecOptions(document_class=RawBSONDocument))
+        # MongoDB 3.5.1+ requires either the 'cursor' or 'explain' options.
+        if client_context.version.at_least(3, 5, 1):
+            result = coll.aggregate([pipeline])
+        else:
+            result = coll.aggregate([pipeline], useCursor=False)
         self.assertTrue(isinstance(result, CommandCursor))
         first_result = next(result)
         self.assertIsInstance(first_result, RawBSONDocument)
         self.assertEqual([1, 2], list(first_result['foo']))
 
-    @client_context.require_version_min(2, 5, 1)
     def test_aggregation_cursor_validation(self):
         db = self.db
         projection = {'$project': {'_id': '$_id'}}
@@ -1521,7 +1573,6 @@ class TestCollection(IntegrationTest):
         cursor = db.test.aggregate([projection], useCursor=True)
         self.assertTrue(isinstance(cursor, CommandCursor))
 
-    @client_context.require_version_min(2, 5, 1)
     def test_aggregation_cursor(self):
         db = self.db
         if client_context.has_secondaries:
@@ -1550,16 +1601,12 @@ class TestCollection(IntegrationTest):
         # Force a getMore
         cursor._CommandCursor__data.clear()
         next(cursor)
-        # startingFrom for a command cursor doesn't include the initial batch
-        # returned by the command.
-        self.assertEqual(5, cursor._CommandCursor__retrieved)
         # batchSize - 1
         self.assertEqual(4, len(cursor._CommandCursor__data))
         # Exhaust the cursor. There shouldn't be any errors.
         for doc in cursor:
             pass
 
-    @client_context.require_version_min(2, 5, 1)
     def test_aggregation_cursor_alive(self):
         self.db.test.delete_many({})
         self.db.test.insert_many([{} for _ in range(3)])
@@ -1575,7 +1622,6 @@ class TestCollection(IntegrationTest):
 
             self.assertTrue(cursor.alive)
 
-    @client_context.require_version_min(2, 5, 5)
     @client_context.require_no_mongos
     def test_parallel_scan(self):
         db = self.db
@@ -1618,101 +1664,6 @@ class TestCollection(IntegrationTest):
                                           "maxTimeAlwaysTimeOut",
                                           mode="off")
 
-    def test_group(self):
-        db = self.db
-        db.drop_collection("test")
-
-        self.assertEqual([],
-                         db.test.group([], {}, {"count": 0},
-                                       "function (obj, prev) { prev.count++; }"
-                                      ))
-
-        db.test.insert_many([{"a": 2}, {"b": 5}, {"a": 1}])
-
-        self.assertEqual([{"count": 3}],
-                         db.test.group([], {}, {"count": 0},
-                                       "function (obj, prev) { prev.count++; }"
-                                      ))
-
-        self.assertEqual([{"count": 1}],
-                         db.test.group([], {"a": {"$gt": 1}}, {"count": 0},
-                                       "function (obj, prev) { prev.count++; }"
-                                      ))
-
-        db.test.insert_one({"a": 2, "b": 3})
-
-        self.assertEqual([{"a": 2, "count": 2},
-                          {"a": None, "count": 1},
-                          {"a": 1, "count": 1}],
-                         db.test.group(["a"], {}, {"count": 0},
-                                       "function (obj, prev) { prev.count++; }"
-                                      ))
-
-        # modifying finalize
-        self.assertEqual([{"a": 2, "count": 3},
-                          {"a": None, "count": 2},
-                          {"a": 1, "count": 2}],
-                         db.test.group(["a"], {}, {"count": 0},
-                                       "function (obj, prev) "
-                                       "{ prev.count++; }",
-                                       "function (obj) { obj.count++; }"))
-
-        # returning finalize
-        self.assertEqual([2, 1, 1],
-                         db.test.group(["a"], {}, {"count": 0},
-                                       "function (obj, prev) "
-                                       "{ prev.count++; }",
-                                       "function (obj) { return obj.count; }"))
-
-        # keyf
-        self.assertEqual([2, 2],
-                         db.test.group("function (obj) { if (obj.a == 2) "
-                                       "{ return {a: true} }; "
-                                       "return {b: true}; }", {}, {"count": 0},
-                                       "function (obj, prev) "
-                                       "{ prev.count++; }",
-                                       "function (obj) { return obj.count; }"))
-
-        # no key
-        self.assertEqual([{"count": 4}],
-                         db.test.group(None, {}, {"count": 0},
-                                       "function (obj, prev) { prev.count++; }"
-                                      ))
-
-        self.assertRaises(OperationFailure, db.test.group,
-                          [], {}, {}, "5 ++ 5")
-
-    def test_group_with_scope(self):
-        db = self.db
-        db.drop_collection("test")
-        db.test.insert_many([{"a": 1}, {"b": 1}])
-
-        reduce_function = "function (obj, prev) { prev.count += inc_value; }"
-
-        self.assertEqual(2, db.test.group([], {}, {"count": 0},
-                                          Code(reduce_function,
-                                               {"inc_value": 1}))[0]['count'])
-        self.assertEqual(4, db.test.group([], {}, {"count": 0},
-                                          Code(reduce_function,
-                                               {"inc_value": 2}))[0]['count'])
-
-        self.assertEqual(1,
-                         db.test.group([], {}, {"count": 0},
-                                       Code(reduce_function,
-                                            {"inc_value": 0.5}))[0]['count'])
-
-        self.assertEqual(2, db.test.group(
-            [], {}, {"count": 0},
-            Code(reduce_function, {"inc_value": 1}))[0]['count'])
-
-        self.assertEqual(4, db.test.group(
-            [], {}, {"count": 0},
-            Code(reduce_function, {"inc_value": 2}))[0]['count'])
-
-        self.assertEqual(1, db.test.group(
-            [], {}, {"count": 0},
-            Code(reduce_function, {"inc_value": 0.5}))[0]['count'])
-
     def test_large_limit(self):
         db = self.db
         db.drop_collection("test_large_limit")
@@ -1799,6 +1750,16 @@ class TestCollection(IntegrationTest):
 
         self.assertTrue("hello" in db.test.find_one(projection=["hello"]))
         self.assertTrue("hello" not in db.test.find_one(projection=["foo"]))
+
+        self.assertTrue("hello" in db.test.find_one(projection=("hello",)))
+        self.assertTrue("hello" not in db.test.find_one(projection=("foo",)))
+
+        self.assertTrue("hello" in db.test.find_one(projection=set(["hello"])))
+        self.assertTrue("hello" not in db.test.find_one(projection=set(["foo"])))
+
+        self.assertTrue("hello" in db.test.find_one(projection=frozenset(["hello"])))
+        self.assertTrue("hello" not in db.test.find_one(projection=frozenset(["foo"])))
+
         self.assertEqual(["_id"], list(db.test.find_one(projection=[])))
 
         self.assertEqual(None, db.test.find_one({"hello": "foo"}))
@@ -2079,6 +2040,41 @@ class TestCollection(IntegrationTest):
         c.insert_one({'bad': bad})
         self.assertEqual('bar', c.find_one()['bad']['foo'])
 
+    @client_context.require_version_max(3, 5, 5)
+    def test_array_filters_unsupported(self):
+        c = self.db.test
+        with self.assertRaises(ConfigurationError):
+            c.update_one(
+                {}, {'$set': {'y.$[i].b': 5}}, array_filters=[{'i.b': 1}])
+        with self.assertRaises(ConfigurationError):
+            c.update_many(
+                {}, {'$set': {'y.$[i].b': 5}}, array_filters=[{'i.b': 1}])
+        with self.assertRaises(ConfigurationError):
+            c.find_one_and_update(
+                {}, {'$set': {'y.$[i].b': 5}}, array_filters=[{'i.b': 1}])
+
+    def test_array_filters_validation(self):
+        # array_filters must be a list.
+        c = self.db.test
+        with self.assertRaises(TypeError):
+            c.update_one({}, {'$set': {'a': 1}}, array_filters={})
+        with self.assertRaises(TypeError):
+            c.update_many({}, {'$set': {'a': 1}}, array_filters={})
+        with self.assertRaises(TypeError):
+            c.find_one_and_update({}, {'$set': {'a': 1}}, array_filters={})
+
+    def test_array_filters_unacknowledged(self):
+        c_w0 = self.db.test.with_options(write_concern=WriteConcern(w=0))
+        with self.assertRaises(ConfigurationError):
+            c_w0.update_one({}, {'$set': {'y.$[i].b': 5}},
+                            array_filters=[{'i.b': 1}])
+        with self.assertRaises(ConfigurationError):
+            c_w0.update_many({}, {'$set': {'y.$[i].b': 5}},
+                             array_filters=[{'i.b': 1}])
+        with self.assertRaises(ConfigurationError):
+            c_w0.find_one_and_update({}, {'$set': {'y.$[i].b': 5}},
+                                     array_filters=[{'i.b': 1}])
+
     def test_find_one_and(self):
         c = self.db.test
         c.drop()
@@ -2283,8 +2279,9 @@ class TestCollection(IntegrationTest):
             self.assertTrue(isinstance(doc['r'], Regex))
 
     def test_find_command_generation(self):
-        cmd = _gen_find_command(
-            'coll', {'$query': {'foo': 1}, '$dumb': 2}, None, 0, 0, 0, None)
+        cmd = _gen_find_command('coll', {'$query': {'foo': 1}, '$dumb': 2},
+                                None, 0, 0, 0, None, DEFAULT_READ_CONCERN,
+                                None)
         self.assertEqual(
             cmd.to_dict(),
             SON([('find', 'coll'),
