@@ -1,4 +1,4 @@
-# Copyright 2011-2015 MongoDB, Inc.
+# Copyright 2011-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ from bson.codec_options import CodecOptions
 from bson.objectid import ObjectId
 from pymongo.errors import OperationFailure
 from pymongo.write_concern import WriteConcern
-from test import client_context, unittest, IntegrationTest
+from test import client_context, IntegrationTest
 from test.utils import connected, rs_or_single_client, single_client
 
 
@@ -147,22 +147,6 @@ class TestCommon(IntegrationTest):
         self.db.drop_collection("result")
         coll.drop()
 
-        # Test group
-        coll.insert_one({"_id": uu, "a": 2})
-        coll.insert_one({"_id": uuid.uuid4(), "a": 1})
-
-        reduce = "function (obj, prev) { prev.count++; }"
-        coll = self.db.get_collection(
-            "uuid", CodecOptions(uuid_representation=STANDARD))
-        self.assertEqual([],
-                         coll.group([], {"_id": uu},
-                                    {"count": 0}, reduce))
-        coll = self.db.get_collection(
-            "uuid", CodecOptions(uuid_representation=PYTHON_LEGACY))
-        self.assertEqual([{"count": 1}],
-                         coll.group([], {"_id": uu},
-                                    {"count": 0}, reduce))
-
     def test_write_concern(self):
         c = rs_or_single_client(connect=False)
         self.assertEqual(WriteConcern(), c.write_concern)
@@ -211,7 +195,8 @@ class TestCommon(IntegrationTest):
 
         # Equality tests
         direct = connected(single_client(w=0))
-        direct2 = connected(single_client("mongodb://%s/?w=0" % (pair,)))
+        direct2 = connected(single_client("mongodb://%s/?w=0" % (pair,),
+                                          **self.credentials))
         self.assertEqual(direct, direct2)
         self.assertFalse(direct != direct2)
 
