@@ -1,4 +1,4 @@
-# Copyright 2015 MongoDB, Inc.
+# Copyright 2015-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,10 +22,16 @@ import sys
 sys.path[0:0] = [""]
 
 from bson.py3compat import iteritems
+from pymongo import operations
 from pymongo.command_cursor import CommandCursor
 from pymongo.cursor import Cursor
-from pymongo.results import _WriteResult
-from pymongo.operations import UpdateOne, UpdateMany, DeleteOne, DeleteMany, InsertOne, ReplaceOne
+from pymongo.results import _WriteResult, BulkWriteResult
+from pymongo.operations import (InsertOne,
+                                DeleteOne,
+                                DeleteMany,
+                                ReplaceOne,
+                                UpdateOne,
+                                UpdateMany)
 
 from test import unittest, client_context, IntegrationTest
 
@@ -44,6 +50,10 @@ def camel_to_snake(camel):
     return re.sub('([a-z0-9])([A-Z])', r'\1_\2', snake).lower()
 
 
+def camel_to_upper_camel(camel):
+    return camel[0].upper() + camel[1:]
+
+
 def check_result(expected_result, result):
     if isinstance(result, Cursor) or isinstance(result, CommandCursor):
         return list(result) == expected_result
@@ -51,7 +61,39 @@ def check_result(expected_result, result):
     elif isinstance(result, _WriteResult):
         for res in expected_result:
             prop = camel_to_snake(res)
-            return getattr(result, prop) == expected_result[res]
+            # SPEC-869: Only BulkWriteResult has upserted_count.
+            if (prop == "upserted_count" and
+                    not isinstance(result, BulkWriteResult)):
+                if result.upserted_id is not None:
+                    upserted_count = 1
+                else:
+                    upserted_count = 0
+                if upserted_count != expected_result[res]:
+                    return False
+            elif prop == "inserted_ids":
+                # BulkWriteResult does not have inserted_ids.
+                if isinstance(result, BulkWriteResult):
+                    if len(expected_result[res]) != result.inserted_count:
+                        return False
+                else:
+                    # InsertManyResult may be compared to [id1] from the
+                    # crud spec or {"0": id1} from the retryable write spec.
+                    ids = expected_result[res]
+                    if isinstance(ids, dict):
+                        ids = [ids[str(i)] for i in range(len(ids))]
+                    if ids != result.inserted_ids:
+                        return False
+            elif prop == "upserted_ids":
+                # Convert indexes from strings to integers.
+                ids = expected_result[res]
+                expected_ids = {}
+                for str_index in ids:
+                    expected_ids[int(str_index)] = ids[str_index]
+                if expected_ids != result.upserted_ids:
+                    return False
+            elif getattr(result, prop) != expected_result[res]:
+                return False
+        return True
     else:
         if not expected_result:
             return result is None
@@ -59,19 +101,33 @@ def check_result(expected_result, result):
             return result == expected_result
 
 
-def create_test(scenario_def, test):
-    def run_scenario(self):
-        # Load data.
-        assert scenario_def['data'], "tests must have non-empty data"
-        self.db.test.drop()
-        self.db.test.insert_many(scenario_def['data'])
+def camel_to_snake_args(arguments):
+    for arg_name in list(arguments):
+        c2s = camel_to_snake(arg_name)
+        arguments[c2s] = arguments.pop(arg_name)
+    return arguments
 
-        # Convert command from CamelCase to pymongo.collection method.
-        operation = camel_to_snake(test['operation']['name'])
-        cmd = getattr(self.db.test, operation)
 
-        # Convert arguments to snake_case and handle special cases.
-        arguments = test['operation']['arguments']
+def run_operation(collection, test):
+    # Convert command from CamelCase to pymongo.collection method.
+    operation = camel_to_snake(test['operation']['name'])
+    cmd = getattr(collection, operation)
+
+    # Convert arguments to snake_case and handle special cases.
+    arguments = test['operation']['arguments']
+    options = arguments.pop("options", {})
+    for option_name in options:
+        arguments[camel_to_snake(option_name)] = options[option_name]
+    if operation == "bulk_write":
+        # Parse each request into a bulk write model.
+        requests = []
+        for request in arguments["requests"]:
+            bulk_model = camel_to_upper_camel(request["name"])
+            bulk_class = getattr(operations, bulk_model)
+            bulk_arguments = camel_to_snake_args(request["arguments"])
+            requests.append(bulk_class(**bulk_arguments))
+        arguments["requests"] = requests
+    else:
         for arg_name in list(arguments):
             c2s = camel_to_snake(arg_name)
             # PyMongo accepts sort as list of tuples. Asserting len=1
@@ -92,7 +148,24 @@ def create_test(scenario_def, test):
             else:
                 arguments[c2s] = arguments.pop(arg_name)
 
-        result = cmd(**arguments)
+    result = cmd(**arguments)
+
+    if operation == "aggregate":
+        if arguments["pipeline"] and "$out" in arguments["pipeline"][-1]:
+            out = collection.database[arguments["pipeline"][-1]["$out"]]
+            return out.find()
+    return result
+
+
+def create_test(scenario_def, test):
+    def run_scenario(self):
+        # Load data.
+        assert scenario_def['data'], "tests must have non-empty data"
+        for name in self.db.collection_names(False):
+            self.db.drop_collection(name)
+        self.db.test.insert_many(scenario_def['data'])
+
+        result = run_operation(self.db.test, test)
 
         # Assert final state is expected.
         expected_c = test['outcome'].get('collection')
@@ -103,8 +176,8 @@ def create_test(scenario_def, test):
             else:
                 db_coll = self.db.test
             self.assertEqual(list(db_coll.find()), expected_c['data'])
-        else:
-            check_result(test['outcome'].get('result'), result)
+        expected_result = test['outcome'].get('result')
+        self.assertTrue(check_result(expected_result, result))
 
     return run_scenario
 
