@@ -1,4 +1,4 @@
-# Copyright 2009-2015 MongoDB, Inc.
+# Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,43 +14,46 @@
 
 """Test the cursor module."""
 import copy
+import gc
 import itertools
 import random
 import re
 import sys
+import time
+import threading
 
 sys.path[0:0] = [""]
 
+from bson import decode_all
 from bson.code import Code
 from bson.py3compat import PY3
 from bson.son import SON
 from pymongo import (monitoring,
-                     MongoClient,
                      ASCENDING,
                      DESCENDING,
                      ALL,
                      OFF)
+from pymongo.collation import Collation
 from pymongo.cursor import CursorType
-from pymongo.errors import (InvalidOperation,
-                            OperationFailure,
-                            ExecutionTimeout)
+from pymongo.errors import (ConfigurationError,
+                            ExecutionTimeout,
+                            InvalidOperation,
+                            OperationFailure)
+from pymongo.read_concern import ReadConcern
 from test import (client_context,
                   SkipTest,
                   unittest,
-                  IntegrationTest)
-from test.utils import (rs_or_single_client,
-                        EventListener)
+                  IntegrationTest, Version)
+from test.utils import (EventListener,
+                        ignore_deprecations,
+                        rs_or_single_client,
+                        WhiteListEventListener)
 
 if PY3:
     long = int
 
 
-class TestCursorNoConnect(unittest.TestCase):
-
-    @classmethod
-    def setUpClass(cls):
-        cls.db = MongoClient(connect=False).test
-
+class TestCursor(IntegrationTest):
     def test_deepcopy_cursor_littered_with_regexes(self):
         cursor = self.db.test.find({
             "x": re.compile("^hmmm.*"),
@@ -127,9 +130,6 @@ class TestCursorNoConnect(unittest.TestCase):
         cursor.remove_option(128)
         self.assertEqual(0, cursor._Cursor__query_flags)
 
-
-class TestCursor(IntegrationTest):
-
     def test_add_remove_option_exhaust(self):
         # Exhaust - which mongos doesn't support
         if client_context.is_mongos:
@@ -146,7 +146,6 @@ class TestCursor(IntegrationTest):
             self.assertEqual(0, cursor._Cursor__query_flags)
             self.assertFalse(cursor._Cursor__exhaust)
 
-    @client_context.require_version_min(2, 5, 3, -1)
     def test_max_time_ms(self):
         db = self.db
         db.pymongo_test.drop()
@@ -172,7 +171,8 @@ class TestCursor(IntegrationTest):
         self.assertTrue(coll.find_one(max_time_ms=1000))
 
         client = self.client
-        if "enableTestCommands=1" in client_context.cmd_line['argv']:
+        if (not client_context.is_mongos
+                and client_context.test_commands_enabled):
             # Cursor parses server timeout error in response to initial query.
             client.admin.command("configureFailPoint",
                                  "maxTimeAlwaysTimeOut",
@@ -225,8 +225,7 @@ class TestCursor(IntegrationTest):
                 10).max_await_time_ms(90)
         self.assertEqual(90, cursor._Cursor__max_await_time_ms)
 
-        listener = EventListener()
-        listener.add_command_filter('killCursors')
+        listener = WhiteListEventListener('find', 'getMore')
         saved_listeners = monitoring._LISTENERS
         monitoring._LISTENERS = monitoring._Listeners([], [], [], [])
         coll = rs_or_single_client(
@@ -256,11 +255,11 @@ class TestCursor(IntegrationTest):
 
             # Tailable_await with max_time_ms
             list(coll.find(
-                cursor_type=CursorType.TAILABLE_AWAIT).max_time_ms(1))
+                cursor_type=CursorType.TAILABLE_AWAIT).max_time_ms(99))
             # find
             self.assertEqual('find', results['started'][0].command_name)
             self.assertTrue('maxTimeMS' in results['started'][0].command)
-            self.assertEqual(1, results['started'][0].command['maxTimeMS'])
+            self.assertEqual(99, results['started'][0].command['maxTimeMS'])
             # getMore
             self.assertEqual('getMore', results['started'][1].command_name)
             self.assertFalse('maxTimeMS' in results['started'][1].command)
@@ -269,11 +268,11 @@ class TestCursor(IntegrationTest):
             # Tailable_await with both max_time_ms and max_await_time_ms
             list(coll.find(
                 cursor_type=CursorType.TAILABLE_AWAIT).max_time_ms(
-                    1).max_await_time_ms(99))
+                    99).max_await_time_ms(99))
             # find
             self.assertEqual('find', results['started'][0].command_name)
             self.assertTrue('maxTimeMS' in results['started'][0].command)
-            self.assertEqual(1, results['started'][0].command['maxTimeMS'])
+            self.assertEqual(99, results['started'][0].command['maxTimeMS'])
             # getMore
             self.assertEqual('getMore', results['started'][1].command_name)
             self.assertTrue('maxTimeMS' in results['started'][1].command)
@@ -313,7 +312,6 @@ class TestCursor(IntegrationTest):
         finally:
             monitoring._LISTENERS = saved_listeners
 
-    @client_context.require_version_min(2, 5, 3, -1)
     @client_context.require_test_commands
     @client_context.require_no_mongos
     def test_max_time_ms_getmore(self):
@@ -349,6 +347,18 @@ class TestCursor(IntegrationTest):
         # "cursor" pre MongoDB 2.7.6, "executionStats" post
         self.assertTrue("cursor" in b or "executionStats" in b)
 
+    def test_explain_with_read_concern(self):
+        # Do not add readConcern level to explain.
+        listener = WhiteListEventListener("explain")
+        client = rs_or_single_client(event_listeners=[listener])
+        self.addCleanup(client.close)
+        coll = client.pymongo_test.test.with_options(
+            read_concern=ReadConcern(level="local"))
+        self.assertTrue(coll.find().explain())
+        started = listener.results['started']
+        self.assertEqual(len(started), 1)
+        self.assertNotIn("readConern", started[0].command)
+
     def test_hint(self):
         db = self.db
         self.assertRaises(TypeError, db.test.find().hint, 5.5)
@@ -758,28 +768,15 @@ class TestCursor(IntegrationTest):
         self.assertEqual(1, collection.find({'i': 1}).hint("_id_").count())
         self.assertEqual(2, collection.find().hint("_id_").count())
 
-        if client_context.version.at_least(2, 6, 0):
-            # Count supports hint
-            self.assertRaises(OperationFailure,
-                              collection.find({'i': 1}).hint("BAD HINT").count)
-        else:
-            # Hint is ignored
-            self.assertEqual(
-                1, collection.find({'i': 1}).hint("BAD HINT").count())
+        self.assertRaises(OperationFailure,
+                          collection.find({'i': 1}).hint("BAD HINT").count)
 
         # Create a sparse index which should have no entries.
         collection.create_index([('x', 1)], sparse=True)
 
-        if client_context.version.at_least(2, 6, 0):
-            # Count supports hint
-            self.assertEqual(0, collection.find({'i': 1}).hint("x_1").count())
-            self.assertEqual(
-                0, collection.find({'i': 1}).hint([("x", 1)]).count())
-        else:
-            # Hint is ignored
-            self.assertEqual(1, collection.find({'i': 1}).hint("x_1").count())
-            self.assertEqual(
-                1, collection.find({'i': 1}).hint([("x", 1)]).count())
+        self.assertEqual(0, collection.find({'i': 1}).hint("x_1").count())
+        self.assertEqual(
+            0, collection.find({'i': 1}).hint([("x", 1)]).count())
 
         if client_context.version.at_least(3, 3, 2):
             self.assertEqual(0, collection.find().hint("x_1").count())
@@ -1127,6 +1124,46 @@ class TestCursor(IntegrationTest):
 
         self.assertEqual(3, db.test.count())
 
+        # __getitem__(index)
+        for cursor in (db.test.find(cursor_type=CursorType.TAILABLE),
+                       db.test.find(cursor_type=CursorType.TAILABLE_AWAIT)):
+            self.assertEqual(4, cursor[0]["x"])
+            self.assertEqual(5, cursor[1]["x"])
+            self.assertEqual(6, cursor[2]["x"])
+
+            cursor.rewind()
+            self.assertEqual([4], [doc["x"] for doc in cursor[0:1]])
+            cursor.rewind()
+            self.assertEqual([5], [doc["x"] for doc in cursor[1:2]])
+            cursor.rewind()
+            self.assertEqual([6], [doc["x"] for doc in cursor[2:3]])
+            cursor.rewind()
+            self.assertEqual([4, 5], [doc["x"] for doc in cursor[0:2]])
+            cursor.rewind()
+            self.assertEqual([5, 6], [doc["x"] for doc in cursor[1:3]])
+            cursor.rewind()
+            self.assertEqual([4, 5, 6], [doc["x"] for doc in cursor[0:3]])
+
+    def test_concurrent_close(self):
+        """Ensure a tailable can be closed from another thread."""
+        db = self.db
+        db.drop_collection("test")
+        db.create_collection("test", capped=True, size=1000, max=3)
+        self.addCleanup(db.drop_collection, "test")
+        cursor = db.test.find(cursor_type=CursorType.TAILABLE)
+
+        def iterate_cursor():
+            while cursor.alive:
+                for doc in cursor:
+                    pass
+        t = threading.Thread(target=iterate_cursor)
+        t.start()
+        time.sleep(1)
+        cursor.close()
+        self.assertFalse(cursor.alive)
+        t.join(3)
+        self.assertFalse(t.is_alive())
+
 
     def test_distinct(self):
         self.db.drop_collection("test")
@@ -1182,7 +1219,9 @@ class TestCursor(IntegrationTest):
         # MongoDB 3.1.5 changed the ns for commands.
         regex = {'$regex': 'pymongo_test.(\$cmd|test)'}
 
-        if client_context.version.at_least(3, 1, 8, -1):
+        if client_context.version.at_least(3, 5, 8, -1):
+            query_key = "command.comment"
+        elif client_context.version.at_least(3, 1, 8, -1):
             query_key = "query.comment"
         else:
             query_key = "query.$comment"
@@ -1219,17 +1258,55 @@ class TestCursor(IntegrationTest):
         self.assertRaises(InvalidOperation, cursor.comment, 'hello')
 
     def test_modifiers(self):
-        cur = self.db.test.find()
-        self.assertTrue('$query' not in cur._Cursor__query_spec())
-        cur = self.db.test.find().comment("testing").max_time_ms(500)
-        self.assertTrue('$query' in cur._Cursor__query_spec())
-        self.assertEqual(cur._Cursor__query_spec()["$comment"], "testing")
-        self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 500)
-        cur = self.db.test.find(
-            modifiers={"$maxTimeMS": 500, "$comment": "testing"})
-        self.assertTrue('$query' in cur._Cursor__query_spec())
-        self.assertEqual(cur._Cursor__query_spec()["$comment"], "testing")
-        self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 500)
+        c = self.db.test
+
+        # "modifiers" is deprecated.
+        with ignore_deprecations():
+            cur = c.find()
+            self.assertTrue('$query' not in cur._Cursor__query_spec())
+            cur = c.find().comment("testing").max_time_ms(500)
+            self.assertTrue('$query' in cur._Cursor__query_spec())
+            self.assertEqual(cur._Cursor__query_spec()["$comment"], "testing")
+            self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 500)
+            cur = c.find(
+                modifiers={"$maxTimeMS": 500, "$comment": "testing"})
+            self.assertTrue('$query' in cur._Cursor__query_spec())
+            self.assertEqual(cur._Cursor__query_spec()["$comment"], "testing")
+            self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 500)
+
+            # Keyword arg overwrites modifier.
+            # If we remove the "modifiers" arg, delete this test after checking
+            # that TestCommandMonitoring.test_find_options covers all cases.
+            cur = c.find(comment="hi", modifiers={"$comment": "bye"})
+            self.assertEqual(cur._Cursor__query_spec()["$comment"], "hi")
+
+            cur = c.find(max_scan=1, modifiers={"$maxScan": 2})
+            self.assertEqual(cur._Cursor__query_spec()["$maxScan"], 1)
+
+            cur = c.find(max_time_ms=1, modifiers={"$maxTimeMS": 2})
+            self.assertEqual(cur._Cursor__query_spec()["$maxTimeMS"], 1)
+
+            cur = c.find(min=1, modifiers={"$min": 2})
+            self.assertEqual(cur._Cursor__query_spec()["$min"], 1)
+
+            cur = c.find(max=1, modifiers={"$max": 2})
+            self.assertEqual(cur._Cursor__query_spec()["$max"], 1)
+
+            cur = c.find(return_key=True, modifiers={"$returnKey": False})
+            self.assertEqual(cur._Cursor__query_spec()["$returnKey"], True)
+
+            cur = c.find(hint=[("a", 1)], modifiers={"$hint": {"b": "1"}})
+            self.assertEqual(cur._Cursor__query_spec()["$hint"], {"a": 1})
+
+            cur = c.find(snapshot=True, modifiers={"$snapshot": False})
+            self.assertEqual(cur._Cursor__query_spec()["$snapshot"], True)
+
+            # The arg is named show_record_id after the "find" command arg, the
+            # modifier is named $showDiskLoc for the OP_QUERY modifier. It's
+            # stored as $showDiskLoc then upgraded to showRecordId if we send a
+            # "find" command.
+            cur = c.find(show_record_id=True, modifiers={"$showDiskLoc": False})
+            self.assertEqual(cur._Cursor__query_spec()["$showDiskLoc"], True)
 
     def test_alive(self):
         self.db.test.delete_many({})
@@ -1246,6 +1323,254 @@ class TestCursor(IntegrationTest):
 
             self.assertTrue(cursor.alive)
 
+    def test_close_kills_cursor_synchronously(self):
+        # Kill any cursors possibly queued up by previous tests.
+        gc.collect()
+        self.client._process_periodic_tasks()
+
+        listener = WhiteListEventListener("killCursors")
+        results = listener.results
+        client = rs_or_single_client(event_listeners=[listener])
+        self.addCleanup(client.close)
+        coll = client[self.db.name].test_close_kills_cursors
+
+        # Add some test data.
+        docs_inserted = 1000
+        coll.insert_many([{"i": i} for i in range(docs_inserted)])
+
+        results.clear()
+
+        # Close a cursor while it's still open on the server.
+        cursor = coll.find().batch_size(10)
+        self.assertTrue(bool(next(cursor)))
+        self.assertLess(cursor.retrieved, docs_inserted)
+        cursor.close()
+
+        def assertCursorKilled():
+            self.assertEqual(1, len(results["started"]))
+            self.assertEqual("killCursors", results["started"][0].command_name)
+            self.assertEqual(1, len(results["succeeded"]))
+            self.assertEqual("killCursors",
+                             results["succeeded"][0].command_name)
+
+        assertCursorKilled()
+        results.clear()
+
+        # Close a command cursor while it's still open on the server.
+        cursor = coll.aggregate([], batchSize=10)
+        self.assertTrue(bool(next(cursor)))
+        cursor.close()
+
+        # The cursor should be killed if it had a non-zero id.
+        if cursor.cursor_id:
+            assertCursorKilled()
+        else:
+            self.assertEqual(0, len(results["started"]))
+
+
+class TestRawBatchCursor(IntegrationTest):
+    def test_find_raw(self):
+        c = self.db.test
+        c.drop()
+        docs = [{'_id': i, 'x': 3.0 * i} for i in range(10)]
+        c.insert_many(docs)
+        batches = list(c.find_raw_batches().sort('_id'))
+        self.assertEqual(1, len(batches))
+        self.assertEqual(docs, decode_all(batches[0]))
+
+    def test_manipulate(self):
+        c = self.db.test
+        with self.assertRaises(InvalidOperation):
+            c.find_raw_batches(manipulate=True)
+
+    def test_explain(self):
+        c = self.db.test
+        c.insert_one({})
+        explanation = c.find_raw_batches().explain()
+        self.assertIsInstance(explanation, dict)
+
+    def test_clone(self):
+        cursor = self.db.test.find_raw_batches()
+        # Copy of a RawBatchCursor is also a RawBatchCursor, not a Cursor.
+        self.assertIsInstance(next(cursor.clone()), bytes)
+        self.assertIsInstance(next(copy.copy(cursor)), bytes)
+
+    @client_context.require_no_mongos
+    def test_exhaust(self):
+        c = self.db.test
+        c.drop()
+        c.insert_many({'_id': i} for i in range(200))
+        result = b''.join(c.find_raw_batches(cursor_type=CursorType.EXHAUST))
+        self.assertEqual([{'_id': i} for i in range(200)], decode_all(result))
+
+    def test_server_error(self):
+        with self.assertRaises(OperationFailure) as exc:
+            next(self.db.test.find_raw_batches({'x': {'$bad': 1}}))
+
+        # The server response was decoded, not left raw.
+        self.assertIsInstance(exc.exception.details, dict)
+
+    def test_get_item(self):
+        with self.assertRaises(InvalidOperation):
+            self.db.test.find_raw_batches()[0]
+
+    @client_context.require_version_min(3, 4)
+    def test_collation(self):
+        next(self.db.test.find_raw_batches(collation=Collation('en_US')))
+
+    @client_context.require_version_max(3, 2)
+    def test_collation_error(self):
+        with self.assertRaises(ConfigurationError):
+            next(self.db.test.find_raw_batches(collation=Collation('en_US')))
+
+    @client_context.require_version_min(3, 2)
+    def test_read_concern(self):
+        c = self.db.get_collection("test", read_concern=ReadConcern("majority"))
+        next(c.find_raw_batches())
+
+    @client_context.require_version_max(3, 1)
+    def test_read_concern_error(self):
+        c = self.db.get_collection("test", read_concern=ReadConcern("majority"))
+        with self.assertRaises(ConfigurationError):
+            next(c.find_raw_batches())
+
+    def test_monitoring(self):
+        listener = EventListener()
+        client = rs_or_single_client(event_listeners=[listener])
+        c = client.pymongo_test.test
+        c.drop()
+        c.insert_many([{'_id': i} for i in range(10)])
+
+        listener.results.clear()
+        cursor = c.find_raw_batches(batch_size=4)
+
+        # First raw batch of 4 documents.
+        next(cursor)
+
+        started = listener.results['started'][0]
+        succeeded = listener.results['succeeded'][0]
+        self.assertEqual(0, len(listener.results['failed']))
+        self.assertEqual('find', started.command_name)
+        self.assertEqual('pymongo_test', started.database_name)
+        self.assertEqual('find', succeeded.command_name)
+        csr = succeeded.reply["cursor"]
+        self.assertEqual(csr["ns"], "pymongo_test.test")
+
+        # The batch is a list of one raw bytes object.
+        self.assertEqual(len(csr["firstBatch"]), 1)
+        self.assertEqual(decode_all(csr["firstBatch"][0]),
+                         [{'_id': i} for i in range(0, 4)])
+
+        listener.results.clear()
+
+        # Next raw batch of 4 documents.
+        next(cursor)
+        try:
+            results = listener.results
+            started = results['started'][0]
+            succeeded = results['succeeded'][0]
+            self.assertEqual(0, len(results['failed']))
+            self.assertEqual('getMore', started.command_name)
+            self.assertEqual('pymongo_test', started.database_name)
+            self.assertEqual('getMore', succeeded.command_name)
+            csr = succeeded.reply["cursor"]
+            self.assertEqual(csr["ns"], "pymongo_test.test")
+            self.assertEqual(len(csr["nextBatch"]), 1)
+            self.assertEqual(decode_all(csr["nextBatch"][0]),
+                             [{'_id': i} for i in range(4, 8)])
+        finally:
+            # Finish the cursor.
+            tuple(cursor)
+
+
+class TestRawBatchCommandCursor(IntegrationTest):
+    @classmethod
+    def setUpClass(cls):
+        super(TestRawBatchCommandCursor, cls).setUpClass()
+
+    def test_aggregate_raw(self):
+        c = self.db.test
+        c.drop()
+        docs = [{'_id': i, 'x': 3.0 * i} for i in range(10)]
+        c.insert_many(docs)
+        batches = list(c.aggregate_raw_batches([{'$sort': {'_id': 1}}]))
+        self.assertEqual(1, len(batches))
+        self.assertEqual(docs, decode_all(batches[0]))
+
+    def test_server_error(self):
+        c = self.db.test
+        c.drop()
+        docs = [{'_id': i, 'x': 3.0 * i} for i in range(10)]
+        c.insert_many(docs)
+        c.insert_one({'_id': 10, 'x': 'not a number'})
+
+        with self.assertRaises(OperationFailure) as exc:
+            list(self.db.test.aggregate_raw_batches([{
+                '$sort': {'_id': 1},
+            }, {
+                '$project': {'x': {'$multiply': [2, '$x']}}
+            }], batchSize=4))
+
+        # The server response was decoded, not left raw.
+        self.assertIsInstance(exc.exception.details, dict)
+
+    def test_get_item(self):
+        with self.assertRaises(InvalidOperation):
+            self.db.test.aggregate_raw_batches([])[0]
+
+    @client_context.require_version_min(3, 4)
+    def test_collation(self):
+        next(self.db.test.aggregate_raw_batches([], collation=Collation('en_US')))
+
+    @client_context.require_version_max(3, 2)
+    def test_collation_error(self):
+        with self.assertRaises(ConfigurationError):
+            next(self.db.test.aggregate_raw_batches([], collation=Collation('en_US')))
+
+    def test_monitoring(self):
+        listener = EventListener()
+        client = rs_or_single_client(event_listeners=[listener])
+        c = client.pymongo_test.test
+        c.drop()
+        c.insert_many([{'_id': i} for i in range(10)])
+
+        listener.results.clear()
+        cursor = c.aggregate_raw_batches([{'$sort': {'_id': 1}}], batchSize=4)
+
+        # Start cursor, no initial batch.
+        started = listener.results['started'][0]
+        succeeded = listener.results['succeeded'][0]
+        self.assertEqual(0, len(listener.results['failed']))
+        self.assertEqual('aggregate', started.command_name)
+        self.assertEqual('pymongo_test', started.database_name)
+        self.assertEqual('aggregate', succeeded.command_name)
+        csr = succeeded.reply["cursor"]
+        self.assertEqual(csr["ns"], "pymongo_test.test")
+
+        # First batch is empty.
+        self.assertEqual(len(csr["firstBatch"]), 0)
+        listener.results.clear()
+
+        # Batches of 4 documents.
+        n = 0
+        for batch in cursor:
+            results = listener.results
+            started = results['started'][0]
+            succeeded = results['succeeded'][0]
+            self.assertEqual(0, len(results['failed']))
+            self.assertEqual('getMore', started.command_name)
+            self.assertEqual('pymongo_test', started.database_name)
+            self.assertEqual('getMore', succeeded.command_name)
+            csr = succeeded.reply["cursor"]
+            self.assertEqual(csr["ns"], "pymongo_test.test")
+            self.assertEqual(len(csr["nextBatch"]), 1)
+            self.assertEqual(csr["nextBatch"][0], batch)
+            self.assertEqual(decode_all(batch),
+                             [{'_id': i} for i in range(n, min(n + 4, 10))])
+
+            n += 4
+            listener.results.clear()
+
 
 if __name__ == "__main__":
     unittest.main()
