@@ -1,4 +1,4 @@
-# Copyright 2014-2015 MongoDB, Inc.
+# Copyright 2014-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,6 @@ import warnings
 
 sys.path[0:0] = [""]
 
-from pymongo.command_cursor import CommandCursor
 from pymongo.cursor_manager import CursorManager
 from pymongo.errors import CursorNotFound
 from pymongo.message import _CursorAddress
@@ -49,7 +48,7 @@ class TestCursorManager(IntegrationTest):
     @classmethod
     def tearDownClass(cls):
         cls.warn_context.__exit__()
-        cls.warn_context = None 
+        cls.warn_context = None
         cls.collection.drop()
 
     def test_cursor_manager_validation(self):
@@ -57,12 +56,6 @@ class TestCursorManager(IntegrationTest):
             client_context.client.set_cursor_manager(1)
 
     def test_cursor_manager(self):
-        if (client_context.is_mongos
-                and not client_context.version.at_least(2, 4, 7)):
-            # Old mongos sends incorrectly formatted error response when
-            # cursor isn't found, see SERVER-9738.
-            raise SkipTest("Can't test kill_cursors against old mongos")
-
         self.close_was_called = False
 
         test_case = self
@@ -97,37 +90,6 @@ class TestCursorManager(IntegrationTest):
             wait_until(raises_cursor_not_found, 'close cursor')
             self.assertTrue(self.close_was_called)
 
-    def test_cursor_transfer(self):
-
-        # This is just a test, don't try this at home...
-
-        client = rs_or_single_client()
-        db = client.pymongo_test
-
-        db.test.delete_many({})
-        db.test.insert_many([{'_id': i} for i in range(200)])
-
-        class CManager(CursorManager):
-            def __init__(self, client):
-                super(CManager, self).__init__(client)
-
-            def close(self, dummy, dummy2):
-                # Do absolutely nothing...
-                pass
-
-        client.set_cursor_manager(CManager)
-        docs = []
-        cursor = db.test.find().batch_size(10)
-        docs.append(next(cursor))
-        cursor.close()
-        docs.extend(cursor)
-        self.assertEqual(len(docs), 10)
-        cmd_cursor = {'id': cursor.cursor_id, 'firstBatch': []}
-        ccursor = CommandCursor(cursor.collection, cmd_cursor,
-                                cursor.address, retrieved=cursor.retrieved)
-        docs.extend(ccursor)
-        self.assertEqual(len(docs), 200)
-
 
 if __name__ == "__main__":
     unittest.main()
