@@ -1,4 +1,4 @@
-# Copyright 2009-2015 MongoDB, Inc.
+# Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -155,20 +155,24 @@ class TestDatabase(IntegrationTest):
         self.assertTrue(u"test.foo" in db.collection_names())
         self.assertRaises(CollectionInvalid, db.create_collection, "test.foo")
 
-    def test_collection_names(self):
+    def _test_collection_names(self, meth, test_no_system):
         db = Database(self.client, "pymongo_test")
         db.test.insert_one({"dummy": u"object"})
         db.test.mike.insert_one({"dummy": u"object"})
 
-        colls = db.collection_names()
+        colls = getattr(db, meth)()
         self.assertTrue("test" in colls)
         self.assertTrue("test.mike" in colls)
         for coll in colls:
             self.assertTrue("$" not in coll)
 
-        colls_without_systems = db.collection_names(False)
-        for coll in colls_without_systems:
-            self.assertTrue(not coll.startswith("system."))
+        if test_no_system:
+            db.systemcoll.test.insert_one({})
+            no_system_collections = getattr(
+                db, meth)(include_system_collections=False)
+            for coll in no_system_collections:
+                self.assertTrue(not coll.startswith("system."))
+            self.assertIn("systemcoll.test", no_system_collections)
 
         # Force more than one batch.
         db = self.client.many_collections
@@ -176,10 +180,91 @@ class TestDatabase(IntegrationTest):
             db["coll" + str(i)].insert_one({})
         # No Error
         try:
-            db.collection_names()
+            getattr(db, meth)()
         finally:
             self.client.drop_database("many_collections")
 
+    def test_collection_names(self):
+        self._test_collection_names('collection_names', True)
+
+    def test_list_collection_names(self):
+        self._test_collection_names('list_collection_names', False)
+
+    def test_list_collections(self):
+        self.client.drop_database("pymongo_test")
+        db = Database(self.client, "pymongo_test")
+        db.test.insert_one({"dummy": u"object"})
+        db.test.mike.insert_one({"dummy": u"object"})
+
+        results = db.list_collections()
+        colls = [result["name"] for result in results]
+
+        # All the collections present.
+        self.assertTrue("test" in colls)
+        self.assertTrue("test.mike" in colls)
+
+        # No collection containing a '$'.
+        for coll in colls:
+            self.assertTrue("$" not in coll)
+
+        # Duplicate check.
+        coll_cnt = {}
+        for coll in colls:
+            try:
+                # Found duplicate.
+                coll_cnt[coll] += 1
+                self.assertTrue(False)
+            except KeyError:
+                coll_cnt[coll] = 1
+        coll_cnt = {}
+
+        # Checking if is there any collection which don't exists.
+        if (len(set(colls) - set(["test","test.mike"])) == 0 or
+            len(set(colls) - set(["test","test.mike","system.indexes"])) == 0):
+            self.assertTrue(True)
+        else:
+            self.assertTrue(False)
+
+        colls = db.list_collections(filter={"name": {"$regex": "^test$"}})
+        self.assertEqual(1, len(list(colls)))
+
+        colls = db.list_collections(filter={"name": {"$regex": "^test.mike$"}})
+        self.assertEqual(1, len(list(colls)))
+
+        db.drop_collection("test")
+
+        db.create_collection("test", capped=True, size=4096)
+        results = db.list_collections(filter={'options.capped': True})
+        colls = [result["name"] for result in results]
+
+        # Checking only capped collections are present
+        self.assertTrue("test" in colls)
+        self.assertFalse("test.mike" in colls)
+
+        # No collection containing a '$'.
+        for coll in colls:
+            self.assertTrue("$" not in coll)
+
+        # Duplicate check.
+        coll_cnt = {}
+        for coll in colls:
+            try:
+                # Found duplicate.
+                coll_cnt[coll] += 1
+                self.assertTrue(False)
+            except KeyError:
+                coll_cnt[coll] = 1
+        coll_cnt = {}
+
+        # Checking if is there any collection which don't exists.
+        if (len(set(colls) - set(["test"])) == 0 or
+            len(set(colls) - set(["test","system.indexes"])) == 0):
+            self.assertTrue(True)
+        else:
+            self.assertTrue(False)
+
+        self.client.drop_database("pymongo_test")
+
     def test_collection_names_single_socket(self):
         # Test that Database.collection_names only requires one socket.
         client = rs_or_single_client(maxPoolSize=1)
@@ -313,11 +398,14 @@ class TestDatabase(IntegrationTest):
             self.assertEqual(None, db.error())
             self.assertEqual(None, db.previous_error())
 
-            db.command("forceerror", check=False)
+            db.test.insert_one({"_id": 1})
+            unacked = db.test.with_options(write_concern=WriteConcern(w=0))
+
+            unacked.insert_one({"_id": 1})
             self.assertTrue(db.error())
             self.assertTrue(db.previous_error())
 
-            db.command("forceerror", check=False)
+            unacked.insert_one({"_id": 1})
             self.assertTrue(db.error())
             prev_error = db.previous_error()
             self.assertEqual(prev_error["nPrev"], 1)
@@ -340,14 +428,25 @@ class TestDatabase(IntegrationTest):
             self.assertEqual(None, db.previous_error())
 
     def test_command(self):
+        self.maxDiff = None
         db = self.client.admin
-
-        self.assertEqual(db.command("buildinfo"), db.command({"buildinfo": 1}))
+        first = db.command("buildinfo")
+        second = db.command({"buildinfo": 1})
+        third = db.command("buildinfo", 1)
+        # The logicalTime and operationTime fields were introduced in MongoDB
+        # 3.5. Their value can change from one command call to the next.
+        for doc in (first, second, third):
+            doc.pop("logicalTime", None)
+            doc.pop("operationTime", None)
+        self.assertEqual(first, second)
+        self.assertEqual(second, third)
 
     # We use 'aggregate' as our example command, since it's an easy way to
     # retrieve a BSON regex from a collection using a command. But until
     # MongoDB 2.3.2, aggregation turned regexes into strings: SERVER-6470.
-    @client_context.require_version_min(2, 3, 2)
+    # Note: MongoDB 3.5.2 requires the 'cursor' or 'explain' option for
+    # aggregate.
+    @client_context.require_version_max(3, 5, 0)
     def test_command_with_regex(self):
         db = self.client.pymongo_test
         db.test.drop()
@@ -376,7 +475,14 @@ class TestDatabase(IntegrationTest):
     def test_authenticate_add_remove_user(self):
         # "self.client" is logged in as root.
         auth_db = self.client.pymongo_test
-        db = rs_or_single_client_noauth().pymongo_test
+
+        def check_auth(username, password):
+            c = rs_or_single_client_noauth(
+                username=username,
+                password=password,
+                authSource="pymongo_test")
+
+            c.pymongo_test.collection.find_one()
 
         # Configuration errors
         self.assertRaises(ValueError, auth_db.add_user, "user", '')
@@ -386,126 +492,133 @@ class TestDatabase(IntegrationTest):
         self.assertRaises(ConfigurationError, auth_db.add_user,
                           "user", 'password', True, roles=['read'])
 
-        if client_context.version.at_least(2, 5, 3, -1):
-            with warnings.catch_warnings():
-                warnings.simplefilter("error", DeprecationWarning)
-                self.assertRaises(DeprecationWarning, auth_db.add_user,
-                                  "user", "password")
-                self.assertRaises(DeprecationWarning, auth_db.add_user,
-                                  "user", "password", True)
+        with warnings.catch_warnings():
+            warnings.simplefilter("error", DeprecationWarning)
+            self.assertRaises(DeprecationWarning, auth_db.add_user,
+                              "user", "password")
+            self.assertRaises(DeprecationWarning, auth_db.add_user,
+                              "user", "password", True)
 
-            with ignore_deprecations():
-                self.assertRaises(ConfigurationError, auth_db.add_user,
-                                  "user", "password", digestPassword=True)
+        with ignore_deprecations():
+            self.assertRaises(ConfigurationError, auth_db.add_user,
+                              "user", "password", digestPassword=True)
+
+        extra = {}
+        if client_context.version.at_least(3, 7, 2):
+            extra['mechanisms'] = ['SCRAM-SHA-1']
 
         # Add / authenticate / remove
-        auth_db.add_user("mike", "password", roles=["dbOwner"])
+        auth_db.add_user("mike", "password", roles=["read"], **extra)
         self.addCleanup(remove_all_users, auth_db)
-        self.assertRaises(TypeError, db.authenticate, 5, "password")
-        self.assertRaises(TypeError, db.authenticate, "mike", 5)
+        self.assertRaises(TypeError, check_auth, 5, "password")
+        self.assertRaises(TypeError, check_auth, "mike", 5)
         self.assertRaises(OperationFailure,
-                          db.authenticate, "mike", "not a real password")
-        self.assertRaises(OperationFailure,
-                          db.authenticate, "faker", "password")
-        db.authenticate("mike", "password")
-        db.logout()
+                          check_auth, "mike", "not a real password")
+        self.assertRaises(OperationFailure, check_auth, "faker", "password")
+        check_auth("mike", "password")
 
         # Unicode name and password.
-        db.authenticate(u"mike", u"password")
-        db.logout()
+        check_auth(u"mike", u"password")
 
         auth_db.remove_user("mike")
         self.assertRaises(OperationFailure,
-                          db.authenticate, "mike", "password")
+                          check_auth, "mike", "password")
 
         # Add / authenticate / change password
-        self.assertRaises(OperationFailure,
-                          db.authenticate, "Gustave", u"Dor\xe9")
-        auth_db.add_user("Gustave", u"Dor\xe9", roles=["dbOwner"])
-        db.authenticate("Gustave", u"Dor\xe9")
+        self.assertRaises(OperationFailure, check_auth, "Gustave", u"Dor\xe9")
+        auth_db.add_user("Gustave", u"Dor\xe9", roles=["read"], **extra)
+        check_auth("Gustave", u"Dor\xe9")
 
         # Change password.
-        auth_db.add_user("Gustave", "password", roles=["dbOwner"])
-        db.logout()
-        self.assertRaises(OperationFailure,
-                          db.authenticate, "Gustave", u"Dor\xe9")
-        self.assertTrue(db.authenticate("Gustave", u"password"))
-
-        if not client_context.version.at_least(2, 5, 3, -1):
-            # Add a readOnly user
-            with ignore_deprecations():
-                auth_db.add_user("Ross", "password", read_only=True)
-
-            db.logout()
-            db.authenticate("Ross", u"password")
-            self.assertTrue(
-                auth_db.system.users.find({"readOnly": True}).count())
+        auth_db.add_user("Gustave", "password", roles=["read"], **extra)
+        self.assertRaises(OperationFailure, check_auth, "Gustave", u"Dor\xe9")
+        check_auth("Gustave", u"password")
 
     @client_context.require_auth
+    @ignore_deprecations
     def test_make_user_readonly(self):
+        extra = {}
+        if client_context.version.at_least(3, 7, 2):
+            extra['mechanisms'] = ['SCRAM-SHA-1']
+
         # "self.client" is logged in as root.
         auth_db = self.client.pymongo_test
-        db = rs_or_single_client_noauth().pymongo_test
 
         # Make a read-write user.
-        auth_db.add_user('jesse', 'pw')
+        auth_db.add_user('jesse', 'pw', **extra)
         self.addCleanup(remove_all_users, auth_db)
 
         # Check that we're read-write by default.
-        db.authenticate('jesse', 'pw')
-        db.collection.insert_one({})
-        db.logout()
+        c = rs_or_single_client_noauth(username='jesse',
+                                       password='pw',
+                                       authSource='pymongo_test')
+
+        c.pymongo_test.collection.insert_one({})
 
         # Make the user read-only.
-        auth_db.add_user('jesse', 'pw', read_only=True)
+        auth_db.add_user('jesse', 'pw', read_only=True, **extra)
 
-        db.authenticate('jesse', 'pw')
-        self.assertRaises(OperationFailure, db.collection.insert_one, {})
+        c = rs_or_single_client_noauth(username='jesse',
+                                       password='pw',
+                                       authSource='pymongo_test')
+
+        self.assertRaises(OperationFailure,
+                          c.pymongo_test.collection.insert_one,
+                          {})
 
-    @client_context.require_version_min(2, 5, 3, -1)
     @client_context.require_auth
+    @ignore_deprecations
     def test_default_roles(self):
+        extra = {}
+        if client_context.version.at_least(3, 7, 2):
+            extra['mechanisms'] = ['SCRAM-SHA-1']
+
         # "self.client" is logged in as root.
         auth_admin = self.client.admin
-        auth_admin.add_user('test_default_roles', 'pass')
-        self.addCleanup(auth_admin.remove_user, 'test_default_roles')
+        auth_admin.add_user('test_default_roles', 'pass', **extra)
+        self.addCleanup(client_context.drop_user, 'admin', 'test_default_roles')
         info = auth_admin.command(
             'usersInfo', 'test_default_roles')['users'][0]
 
         self.assertEqual("root", info['roles'][0]['role'])
 
         # Read only "admin" user
-        auth_admin.add_user('ro-admin', 'pass', read_only=True)
-        self.addCleanup(auth_admin.remove_user, 'ro-admin')
+        auth_admin.add_user('ro-admin', 'pass', read_only=True, **extra)
+        self.addCleanup(client_context.drop_user, 'admin', 'ro-admin')
         info = auth_admin.command('usersInfo', 'ro-admin')['users'][0]
         self.assertEqual("readAnyDatabase", info['roles'][0]['role'])
 
         # "Non-admin" user
         auth_db = self.client.pymongo_test
-        auth_db.add_user('user', 'pass')
+        auth_db.add_user('user', 'pass', **extra)
         self.addCleanup(remove_all_users, auth_db)
         info = auth_db.command('usersInfo', 'user')['users'][0]
         self.assertEqual("dbOwner", info['roles'][0]['role'])
 
         # Read only "Non-admin" user
-        auth_db.add_user('ro-user', 'pass', read_only=True)
+        auth_db.add_user('ro-user', 'pass', read_only=True, **extra)
         info = auth_db.command('usersInfo', 'ro-user')['users'][0]
         self.assertEqual("read", info['roles'][0]['role'])
 
-    @client_context.require_version_min(2, 5, 3, -1)
     @client_context.require_auth
+    @ignore_deprecations
     def test_new_user_cmds(self):
+        extra = {}
+        if client_context.version.at_least(3, 7, 2):
+            extra['mechanisms'] = ['SCRAM-SHA-1']
+
         # "self.client" is logged in as root.
         auth_db = self.client.pymongo_test
-        auth_db.add_user("amalia", "password", roles=["userAdmin"])
-        self.addCleanup(auth_db.remove_user, "amalia")
+        auth_db.add_user("amalia", "password", roles=["userAdmin"], **extra)
+        self.addCleanup(client_context.drop_user, "pymongo_test", "amalia")
 
-        db = rs_or_single_client_noauth().pymongo_test
-        db.authenticate("amalia", "password")
+        db = rs_or_single_client_noauth(username="amalia",
+                                        password="password",
+                                        authSource="pymongo_test").pymongo_test
 
         # This tests the ability to update user attributes.
         db.add_user("amalia", "new_password",
-                    customData={"secret": "koalas"})
+                    customData={"secret": "koalas"}, **extra)
 
         user_info = db.command("usersInfo", "amalia")
         self.assertTrue(user_info["users"])
@@ -514,7 +627,12 @@ class TestDatabase(IntegrationTest):
         self.assertEqual(amalia_user["customData"], {"secret": "koalas"})
 
     @client_context.require_auth
+    @ignore_deprecations
     def test_authenticate_multiple(self):
+        extra = {}
+        if client_context.version.at_least(3, 7, 2):
+            extra['mechanisms'] = ['SCRAM-SHA-1']
+
         # "self.client" is logged in as root.
         self.client.drop_database("pymongo_test")
         self.client.drop_database("pymongo_test1")
@@ -529,15 +647,15 @@ class TestDatabase(IntegrationTest):
 
         self.assertRaises(OperationFailure, users_db.test.find_one)
 
-        if client_context.version.at_least(2, 5, 3, -1):
-            admin_db_auth.add_user('ro-admin', 'pass',
-                                   roles=["userAdmin", "readAnyDatabase"])
-        else:
-            admin_db_auth.add_user('ro-admin', 'pass', read_only=True)
+        admin_db_auth.add_user(
+            'ro-admin',
+            'pass',
+            roles=["userAdmin", "readAnyDatabase"],
+            **extra)
 
-        self.addCleanup(admin_db_auth.remove_user, 'ro-admin')
-        users_db_auth.add_user('user', 'pass',
-                               roles=["userAdmin", "readWrite"])
+        self.addCleanup(client_context.drop_user, 'admin', 'ro-admin')
+        users_db_auth.add_user(
+            'user', 'pass', roles=["userAdmin", "readWrite"], **extra)
         self.addCleanup(remove_all_users, users_db_auth)
 
         # Regular user should be able to query its own db, but
@@ -624,27 +742,28 @@ class TestDatabase(IntegrationTest):
         db = self.client.pymongo_test
         db.test.drop()
 
-        self.assertRaises(TypeError, db.eval, None)
-        self.assertRaises(TypeError, db.eval, 5)
-        self.assertRaises(TypeError, db.eval, [])
+        with ignore_deprecations():
+            self.assertRaises(TypeError, db.eval, None)
+            self.assertRaises(TypeError, db.eval, 5)
+            self.assertRaises(TypeError, db.eval, [])
 
-        self.assertEqual(3, db.eval("function (x) {return x;}", 3))
-        self.assertEqual(3, db.eval(u"function (x) {return x;}", 3))
+            self.assertEqual(3, db.eval("function (x) {return x;}", 3))
+            self.assertEqual(3, db.eval(u"function (x) {return x;}", 3))
 
-        self.assertEqual(None,
-                         db.eval("function (x) {db.test.save({y:x});}", 5))
-        self.assertEqual(db.test.find_one()["y"], 5)
+            self.assertEqual(None,
+                             db.eval("function (x) {db.test.save({y:x});}", 5))
+            self.assertEqual(db.test.find_one()["y"], 5)
 
-        self.assertEqual(5, db.eval("function (x, y) {return x + y;}", 2, 3))
-        self.assertEqual(5, db.eval("function () {return 5;}"))
-        self.assertEqual(5, db.eval("2 + 3;"))
+            self.assertEqual(5, db.eval("function (x, y) {return x + y;}", 2, 3))
+            self.assertEqual(5, db.eval("function () {return 5;}"))
+            self.assertEqual(5, db.eval("2 + 3;"))
 
-        self.assertEqual(5, db.eval(Code("2 + 3;")))
-        self.assertRaises(OperationFailure, db.eval, Code("return i;"))
-        self.assertEqual(2, db.eval(Code("return i;", {"i": 2})))
-        self.assertEqual(5, db.eval(Code("i + 3;", {"i": 2})))
+            self.assertEqual(5, db.eval(Code("2 + 3;")))
+            self.assertRaises(OperationFailure, db.eval, Code("return i;"))
+            self.assertEqual(2, db.eval(Code("return i;", {"i": 2})))
+            self.assertEqual(5, db.eval(Code("i + 3;", {"i": 2})))
 
-        self.assertRaises(OperationFailure, db.eval, "5 ++ 5;")
+            self.assertRaises(OperationFailure, db.eval, "5 ++ 5;")
 
     # TODO some of these tests belong in the collection level testing.
     def test_insert_find_one(self):
@@ -748,11 +867,6 @@ class TestDatabase(IntegrationTest):
 
         self.assertRaises(OperationFailure, db.system_js.non_existant)
 
-        # XXX: Broken in V8, works in SpiderMonkey
-        if not client_context.version.at_least(2, 3, 0):
-            db.system_js.no_param = Code("return 5;")
-            self.assertEqual(5, db.system_js.no_param())
-
     def test_system_js_list(self):
         db = self.client.pymongo_test
         db.system.js.delete_many({})
@@ -816,7 +930,6 @@ class TestDatabase(IntegrationTest):
 
         self.assertEqual('outer', str(context.exception))
 
-    @client_context.require_version_min(2, 5, 3, -1)
     @client_context.require_test_commands
     @client_context.require_no_mongos
     def test_command_max_time_ms(self):
@@ -830,10 +943,10 @@ class TestDatabase(IntegrationTest):
                               'count', 'test', maxTimeMS=1)
             pipeline = [{'$project': {'name': 1, 'count': 1}}]
             # Database command helper.
-            db.command('aggregate', 'test', pipeline=pipeline)
+            db.command('aggregate', 'test', pipeline=pipeline, cursor={})
             self.assertRaises(ExecutionTimeout, db.command,
                               'aggregate', 'test',
-                              pipeline=pipeline, maxTimeMS=1)
+                              pipeline=pipeline, cursor={}, maxTimeMS=1)
             # Collection helper.
             db.test.aggregate(pipeline=pipeline)
             self.assertRaises(ExecutionTimeout,
