@@ -14,6 +14,7 @@
 
 """MongoDB documentation examples in Python."""
 
+import threading
 import sys
 
 sys.path[0:0] = [""]
@@ -514,8 +515,6 @@ class TestSampleShellCommands(unittest.TestCase):
             self.assertTrue("instock" in doc)
             self.assertEqual(len(doc["instock"]), 1)
 
-    # The $currentDate operator was new in MongoDB 2.6.
-    @client_context.require_version_min(2, 6)
     def test_update_and_replace(self):
         db = client_context.client.pymongo_test
 
@@ -649,6 +648,78 @@ class TestSampleShellCommands(unittest.TestCase):
 
         self.assertEqual(db.inventory.count(), 0)
 
+    @client_context.require_version_min(3, 5, 11)
+    @client_context.require_replica_set
+    def test_change_streams(self):
+        db = client_context.client.pymongo_test
+        done = False
+
+        def insert_docs():
+            while not done:
+                db.inventory.insert_one({"username": "alice"})
+                db.inventory.delete_one({"username": "alice"})
+
+        t = threading.Thread(target=insert_docs)
+        t.start()
+
+        try:
+            # 1. The database for reactive, real-time applications
+            # Start Changestream Example 1
+            cursor = db.inventory.watch()
+            document = next(cursor)
+            # End Changestream Example 1
+
+            # Start Changestream Example 2
+            cursor = db.inventory.watch(full_document='updateLookup')
+            document = next(cursor)
+            # End Changestream Example 2
+
+            # Start Changestream Example 3
+            resume_token = document.get("_id")
+            cursor = db.inventory.watch(resume_after=resume_token)
+            document = next(cursor)
+            # End Changestream Example 3
+
+            # Start Changestream Example 4
+            pipeline = [
+                {"$match":
+                    {"$or": [
+                        {"fullDocument.username": "alice"},
+                        {"operationType": {"$in": ["delete"]}}]
+                    }
+                }
+            ]
+            cursor = db.inventory.watch(pipeline=pipeline)
+            document = next(cursor)
+            # End Changestream Example 4
+        finally:
+            done = True
+            t.join()
+
+    @client_context.require_version_min(3, 6, 0)
+    @client_context.require_replica_set
+    def test_misc(self):
+        # Marketing examples
+        client = client_context.client
+        self.addCleanup(client.drop_database, "test")
+        self.addCleanup(client.drop_database, "my_database")
+
+        # 2. Tunable consistency controls
+        collection = client.my_database.my_collection
+        with client.start_session() as session:
+            collection.insert_one({'_id': 1}, session=session)
+            collection.update_one(
+                {'_id': 1}, {"$set": {"a": 1}}, session=session)
+            for doc in collection.find({}, session=session):
+                pass
+
+        # 3. Exploiting the power of arrays
+        collection = client.test.array_updates_test
+        collection.update_one(
+            {'_id': 1},
+            {"$set": {"a.$[i].b": 2}},
+            array_filters=[{"i.b": 0}])
+
 
 if __name__ == "__main__":
     unittest.main()
