@@ -1,4 +1,4 @@
-# Copyright 2015 MongoDB, Inc.
+# Copyright 2015-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import copy
 import datetime
 import sys
 import time
@@ -27,14 +28,18 @@ from pymongo.command_cursor import CommandCursor
 from pymongo.errors import NotMasterError, OperationFailure
 from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
-from test import unittest, client_context, client_knobs
+from test import (client_context,
+                  client_knobs,
+                  PyMongoTestCase,
+                  sanitize_cmd,
+                  unittest)
 from test.utils import (EventListener,
                         rs_or_single_client,
                         single_client,
                         wait_until)
 
 
-class TestCommandMonitoring(unittest.TestCase):
+class TestCommandMonitoring(PyMongoTestCase):
 
     @classmethod
     @client_context.require_connection
@@ -62,7 +67,7 @@ class TestCommandMonitoring(unittest.TestCase):
             isinstance(succeeded, monitoring.CommandSucceededEvent))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqual(SON([('ismaster', 1)]), started.command)
+        self.assertEqualCommand(SON([('ismaster', 1)]), started.command)
         self.assertEqual('ismaster', started.command_name)
         self.assertEqual(self.client.address, started.connection_id)
         self.assertEqual('pymongo_test', started.database_name)
@@ -113,7 +118,7 @@ class TestCommandMonitoring(unittest.TestCase):
             isinstance(succeeded, monitoring.CommandSucceededEvent))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqual(
+        self.assertEqualCommand(
             SON([('find', 'test'),
                  ('filter', {}),
                  ('limit', 1),
@@ -140,7 +145,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(0, len(results['failed']))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqual(
+        self.assertEqualCommand(
             SON([('find', 'test'),
                  ('filter', {}),
                  ('projection', {'_id': False}),
@@ -172,7 +177,7 @@ class TestCommandMonitoring(unittest.TestCase):
             self.assertEqual(0, len(results['failed']))
             self.assertTrue(
                 isinstance(started, monitoring.CommandStartedEvent))
-            self.assertEqual(
+            self.assertEqualCommand(
                 SON([('getMore', cursor_id),
                      ('collection', 'test'),
                      ('batchSize', 4)]),
@@ -187,12 +192,10 @@ class TestCommandMonitoring(unittest.TestCase):
             self.assertEqual('getMore', succeeded.command_name)
             self.assertTrue(isinstance(succeeded.request_id, int))
             self.assertEqual(cursor.address, succeeded.connection_id)
-            expected_result = {
-                'cursor': {'id': cursor_id,
-                           'ns': 'pymongo_test.test',
-                           'nextBatch': [{} for _ in range(4)]},
-                'ok': 1}
-            self.assertEqual(expected_result, succeeded.reply)
+            csr = succeeded.reply["cursor"]
+            self.assertEqual(csr["id"], cursor_id)
+            self.assertEqual(csr["ns"], "pymongo_test.test")
+            self.assertEqual(csr["nextBatch"], [{} for _ in range(4)])
         finally:
             # Exhaust the cursor to avoid kill cursors.
             tuple(cursor)
@@ -205,7 +208,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.listener.results.clear()
         coll = self.client.pymongo_test.test
         # Test that we publish the unwrapped command.
-        if self.client.is_mongos and client_context.version.at_least(2, 4, 0):
+        if self.client.is_mongos:
             coll = coll.with_options(
                 read_preference=ReadPreference.PRIMARY_PREFERRED)
         res = coll.find().explain()
@@ -215,7 +218,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(0, len(results['failed']))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqual(cmd, started.command)
+        self.assertEqualCommand(cmd, started.command)
         self.assertEqual('explain', started.command_name)
         self.assertEqual(self.client.address, started.connection_id)
         self.assertEqual('pymongo_test', started.database_name)
@@ -228,33 +231,20 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(self.client.address, succeeded.connection_id)
         self.assertEqual(res, succeeded.reply)
 
-    def test_find_options(self):
-        cmd = SON([('find', 'test'),
-                   ('filter', {}),
-                   ('comment', 'this is a test'),
-                   ('sort', SON([('_id', 1)])),
-                   ('projection', {'x': False}),
-                   ('skip', 1),
-                   ('batchSize', 2),
-                   ('noCursorTimeout', True),
-                   ('allowPartialResults', True)])
-        self.client.pymongo_test.test.drop()
-        self.client.pymongo_test.test.insert_many([{'x': i} for i in range(5)])
-        self.listener.results.clear()
+    def _test_find_options(self, query, expected_cmd):
         coll = self.client.pymongo_test.test
+        coll.drop()
+        coll.create_index('x')
+        coll.insert_many([{'x': i} for i in range(5)])
+
         # Test that we publish the unwrapped command.
-        if self.client.is_mongos and client_context.version.at_least(2, 4, 0):
+        self.listener.results.clear()
+        if self.client.is_mongos:
             coll = coll.with_options(
                 read_preference=ReadPreference.PRIMARY_PREFERRED)
-        cursor = coll.find(
-            filter={},
-            projection={'x': False},
-            skip=1,
-            no_cursor_timeout=True,
-            sort=[('_id', 1)],
-            allow_partial_results=True,
-            modifiers=SON([('$comment', 'this is a test')]),
-            batch_size=2)
+
+        cursor = coll.find(**query)
+
         next(cursor)
         try:
             results = self.listener.results
@@ -263,7 +253,7 @@ class TestCommandMonitoring(unittest.TestCase):
             self.assertEqual(0, len(results['failed']))
             self.assertTrue(
                 isinstance(started, monitoring.CommandStartedEvent))
-            self.assertEqual(cmd, started.command)
+            self.assertEqualCommand(expected_cmd, started.command)
             self.assertEqual('find', started.command_name)
             self.assertEqual(self.client.address, started.connection_id)
             self.assertEqual('pymongo_test', started.database_name)
@@ -278,7 +268,53 @@ class TestCommandMonitoring(unittest.TestCase):
             # Exhaust the cursor to avoid kill cursors.
             tuple(cursor)
 
-    @client_context.require_version_min(2, 6, 0)
+    def test_find_options(self):
+        query = dict(filter={},
+                     hint=[('x', 1)],
+                     max_scan=10,
+                     max_time_ms=10000,
+                     max={'x': 10},
+                     min={'x': -10},
+                     return_key=True,
+                     show_record_id=True,
+                     projection={'x': False},
+                     skip=1,
+                     no_cursor_timeout=True,
+                     sort=[('_id', 1)],
+                     allow_partial_results=True,
+                     comment='this is a test',
+                     batch_size=2)
+
+        cmd = dict(find='test',
+                   filter={},
+                   hint=SON([('x', 1)]),
+                   comment='this is a test',
+                   maxScan=10,
+                   maxTimeMS=10000,
+                   max={'x': 10},
+                   min={'x': -10},
+                   returnKey=True,
+                   showRecordId=True,
+                   sort=SON([('_id', 1)]),
+                   projection={'x': False},
+                   skip=1,
+                   batchSize=2,
+                   noCursorTimeout=True,
+                   allowPartialResults=True)
+
+        self._test_find_options(query, cmd)
+
+    def test_find_snapshot(self):
+        # Test "snapshot" parameter separately, can't combine with "sort".
+        query = dict(filter={},
+                     snapshot=True)
+
+        cmd = dict(find='test',
+                   filter={},
+                   snapshot=True)
+
+        self._test_find_options(query, cmd)
+
     def test_command_and_get_more(self):
         self.client.pymongo_test.test.drop()
         self.client.pymongo_test.test.insert_many(
@@ -286,7 +322,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.listener.results.clear()
         coll = self.client.pymongo_test.test
         # Test that we publish the unwrapped command.
-        if self.client.is_mongos and client_context.version.at_least(2, 4, 0):
+        if self.client.is_mongos:
             coll = coll.with_options(
                 read_preference=ReadPreference.PRIMARY_PREFERRED)
         cursor = coll.aggregate(
@@ -300,7 +336,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(0, len(results['failed']))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqual(
+        self.assertEqualCommand(
             SON([('aggregate', 'test'),
                  ('pipeline', [{'$project': {'_id': False, 'x': 1}}]),
                  ('cursor', {'batchSize': 4})]),
@@ -318,7 +354,7 @@ class TestCommandMonitoring(unittest.TestCase):
         expected_cursor = {'id': cursor_id,
                            'ns': 'pymongo_test.test',
                            'firstBatch': [{'x': 1} for _ in range(4)]}
-        self.assertEqual(expected_cursor, succeeded.reply.get('cursor'))
+        self.assertEqualCommand(expected_cursor, succeeded.reply.get('cursor'))
 
         self.listener.results.clear()
         next(cursor)
@@ -329,7 +365,7 @@ class TestCommandMonitoring(unittest.TestCase):
             self.assertEqual(0, len(results['failed']))
             self.assertTrue(
                 isinstance(started, monitoring.CommandStartedEvent))
-            self.assertEqual(
+            self.assertEqualCommand(
                 SON([('getMore', cursor_id),
                      ('collection', 'test'),
                      ('batchSize', 4)]),
@@ -348,8 +384,8 @@ class TestCommandMonitoring(unittest.TestCase):
                 'cursor': {'id': cursor_id,
                            'ns': 'pymongo_test.test',
                            'nextBatch': [{'x': 1} for _ in range(4)]},
-                'ok': 1}
-            self.assertEqual(expected_result, succeeded.reply)
+                'ok': 1.0}
+            self.assertEqualReply(expected_result, succeeded.reply)
         finally:
             # Exhaust the cursor to avoid kill cursors.
             tuple(cursor)
@@ -369,7 +405,7 @@ class TestCommandMonitoring(unittest.TestCase):
         failed = results['failed'][0]
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqual(
+        self.assertEqualCommand(
             SON([('getMore', 12345),
                  ('collection', 'test')]),
             started.command)
@@ -430,7 +466,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(0, len(results['failed']))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqual(
+        self.assertEqualCommand(
             SON([('find', 'test'),
                  ('filter', {}),
                  ('projection', {'_id': False}),
@@ -451,7 +487,7 @@ class TestCommandMonitoring(unittest.TestCase):
                        'ns': 'pymongo_test.test',
                        'firstBatch': [{} for _ in range(5)]},
             'ok': 1}
-        self.assertEqual(expected_result, succeeded.reply)
+        self.assertEqualReply(expected_result, succeeded.reply)
 
         self.listener.results.clear()
         tuple(cursor)
@@ -461,7 +497,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(0, len(results['failed']))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqual(
+        self.assertEqualCommand(
             SON([('getMore', cursor_id),
                  ('collection', 'test'),
                  ('batchSize', 5)]),
@@ -481,7 +517,7 @@ class TestCommandMonitoring(unittest.TestCase):
                        'ns': 'pymongo_test.test',
                        'nextBatch': [{} for _ in range(5)]},
             'ok': 1}
-        self.assertEqual(expected_result, succeeded.reply)
+        self.assertEqualReply(expected_result, succeeded.reply)
 
     def test_kill_cursors(self):
         with client_knobs(kill_cursor_frequency=0.01):
@@ -503,6 +539,7 @@ class TestCommandMonitoring(unittest.TestCase):
             # when the thread last ran.
             self.assertIn(cursor_id, started.command['cursors'])
             self.assertEqual('killCursors', started.command_name)
+            self.assertIs(type(started.connection_id), tuple)
             self.assertEqual(cursor.address, started.connection_id)
             self.assertEqual('pymongo_test', started.database_name)
             self.assertTrue(isinstance(started.request_id, int))
@@ -511,6 +548,7 @@ class TestCommandMonitoring(unittest.TestCase):
             self.assertTrue(isinstance(succeeded.duration_micros, int))
             self.assertEqual('killCursors', succeeded.command_name)
             self.assertTrue(isinstance(succeeded.request_id, int))
+            self.assertIs(type(succeeded.connection_id), tuple)
             self.assertEqual(cursor.address, succeeded.connection_id)
             # There could be more than one cursor_id here depending on
             # when the thread last ran.
@@ -532,7 +570,7 @@ class TestCommandMonitoring(unittest.TestCase):
         expected = SON([('insert', coll.name),
                         ('ordered', True),
                         ('documents', [{'_id': res.inserted_id, 'x': 1}])])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('insert', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -559,7 +597,7 @@ class TestCommandMonitoring(unittest.TestCase):
                         ('ordered', True),
                         ('documents', [{'_id': res.inserted_id, 'x': 1}]),
                         ('writeConcern', {'w': 0})])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('insert', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -569,7 +607,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(started.command_name, succeeded.command_name)
         self.assertEqual(started.request_id, succeeded.request_id)
         self.assertEqual(started.connection_id, succeeded.connection_id)
-        self.assertEqual(succeeded.reply, {'ok': 1})
+        self.assertEqualReply(succeeded.reply, {'ok': 1})
 
         # Explicit write concern insert_one
         self.listener.results.clear()
@@ -584,7 +622,7 @@ class TestCommandMonitoring(unittest.TestCase):
                         ('ordered', True),
                         ('documents', [{'_id': res.inserted_id, 'x': 1}]),
                         ('writeConcern', {'w': 1})])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('insert', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -611,7 +649,7 @@ class TestCommandMonitoring(unittest.TestCase):
                         ('deletes', [SON([('q', {'x': 1}),
                                           ('limit', 0)])]),
                         ('writeConcern', {'w': 1})])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('delete', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -641,7 +679,7 @@ class TestCommandMonitoring(unittest.TestCase):
                                           ('multi', False),
                                           ('upsert', True)])]),
                         ('writeConcern', {'w': 1})])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('update', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -671,7 +709,7 @@ class TestCommandMonitoring(unittest.TestCase):
                                           ('multi', False),
                                           ('upsert', False)])]),
                         ('writeConcern', {'w': 1})])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('update', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -700,7 +738,7 @@ class TestCommandMonitoring(unittest.TestCase):
                                           ('multi', True),
                                           ('upsert', False)])]),
                         ('writeConcern', {'w': 1})])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('update', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -727,7 +765,7 @@ class TestCommandMonitoring(unittest.TestCase):
                         ('deletes', [SON([('q', {'x': 3}),
                                           ('limit', 1)])]),
                         ('writeConcern', {'w': 1})])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('delete', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -759,7 +797,7 @@ class TestCommandMonitoring(unittest.TestCase):
                         ('ordered', True),
                         ('documents', [{'_id': 1}]),
                         ('writeConcern', {'w': 1})])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('insert', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -797,7 +835,7 @@ class TestCommandMonitoring(unittest.TestCase):
             expected = SON([('insert', coll.name),
                             ('ordered', True),
                             ('documents', [{'_id': _id, 'x': 1}])])
-            self.assertEqual(expected, started.command)
+            self.assertEqualCommand(expected, started.command)
             self.assertEqual('pymongo_test', started.database_name)
             self.assertEqual('insert', started.command_name)
             self.assertIsInstance(started.request_id, int)
@@ -823,7 +861,7 @@ class TestCommandMonitoring(unittest.TestCase):
                             ('ordered', True),
                             ('documents', [{'_id': _id, 'x': 1}]),
                             ('writeConcern', {'w': 0})])
-            self.assertEqual(expected, started.command)
+            self.assertEqualCommand(expected, started.command)
             self.assertEqual('pymongo_test', started.database_name)
             self.assertEqual('insert', started.command_name)
             self.assertIsInstance(started.request_id, int)
@@ -847,7 +885,7 @@ class TestCommandMonitoring(unittest.TestCase):
                             ('ordered', True),
                             ('documents', [{'_id': _id, 'x': 1}]),
                             ('writeConcern', {'w': 1})])
-            self.assertEqual(expected, started.command)
+            self.assertEqualCommand(expected, started.command)
             self.assertEqual('pymongo_test', started.database_name)
             self.assertEqual('insert', started.command_name)
             self.assertIsInstance(started.request_id, int)
@@ -874,7 +912,7 @@ class TestCommandMonitoring(unittest.TestCase):
                             ('deletes', [SON([('q', {'x': 1}),
                                               ('limit', 0)])]),
                             ('writeConcern', {'w': 1})])
-            self.assertEqual(expected, started.command)
+            self.assertEqualCommand(expected, started.command)
             self.assertEqual('pymongo_test', started.database_name)
             self.assertEqual('delete', started.command_name)
             self.assertIsInstance(started.request_id, int)
@@ -904,7 +942,7 @@ class TestCommandMonitoring(unittest.TestCase):
                                               ('multi', False),
                                               ('upsert', True)])]),
                             ('writeConcern', {'w': 1})])
-            self.assertEqual(expected, started.command)
+            self.assertEqualCommand(expected, started.command)
             self.assertEqual('pymongo_test', started.database_name)
             self.assertEqual('update', started.command_name)
             self.assertIsInstance(started.request_id, int)
@@ -933,7 +971,7 @@ class TestCommandMonitoring(unittest.TestCase):
                                               ('u', {'$inc': {'x': 1}}),
                                               ('multi', False),
                                               ('upsert', False)])])])
-            self.assertEqual(expected, started.command)
+            self.assertEqualCommand(expected, started.command)
             self.assertEqual('pymongo_test', started.database_name)
             self.assertEqual('update', started.command_name)
             self.assertIsInstance(started.request_id, int)
@@ -961,7 +999,7 @@ class TestCommandMonitoring(unittest.TestCase):
                                               ('u', {'$inc': {'x': 1}}),
                                               ('multi', True),
                                               ('upsert', False)])])])
-            self.assertEqual(expected, started.command)
+            self.assertEqualCommand(expected, started.command)
             self.assertEqual('pymongo_test', started.database_name)
             self.assertEqual('update', started.command_name)
             self.assertIsInstance(started.request_id, int)
@@ -987,7 +1025,7 @@ class TestCommandMonitoring(unittest.TestCase):
                             ('ordered', True),
                             ('deletes', [SON([('q', {'x': 3}),
                                               ('limit', 1)])])])
-            self.assertEqual(expected, started.command)
+            self.assertEqualCommand(expected, started.command)
             self.assertEqual('pymongo_test', started.database_name)
             self.assertEqual('delete', started.command_name)
             self.assertIsInstance(started.request_id, int)
@@ -1022,7 +1060,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertIsInstance(operation_id, int)
         for start, succeed in zip(started, succeeded):
             self.assertIsInstance(start, monitoring.CommandStartedEvent)
-            cmd = start.command
+            cmd = sanitize_cmd(start.command)
             self.assertEqual(['insert', 'ordered', 'documents'],
                              list(cmd.keys()))
             self.assertEqual(coll.name, cmd['insert'])
@@ -1068,7 +1106,7 @@ class TestCommandMonitoring(unittest.TestCase):
             self.assertIsInstance(operation_id, int)
             for start, succeed in zip(started, succeeded):
                 self.assertIsInstance(start, monitoring.CommandStartedEvent)
-                cmd = start.command
+                cmd = sanitize_cmd(start.command)
                 self.assertEqual(['insert', 'ordered', 'documents'],
                                  list(cmd.keys()))
                 self.assertEqual(coll.name, cmd['insert'])
@@ -1122,19 +1160,19 @@ class TestCommandMonitoring(unittest.TestCase):
         expected = SON([('insert', coll.name),
                         ('ordered', True),
                         ('documents', [{'_id': 1}])])
-        self.assertEqual(expected, started[0].command)
+        self.assertEqualCommand(expected, started[0].command)
         expected = SON([('update', coll.name),
                         ('ordered', True),
                         ('updates', [SON([('q', {'_id': 1}),
                                           ('u', {'$set': {'x': 1}}),
                                           ('multi', False),
                                           ('upsert', False)])])])
-        self.assertEqual(expected, started[1].command)
+        self.assertEqualCommand(expected, started[1].command)
         expected = SON([('delete', coll.name),
                         ('ordered', True),
                         ('deletes', [SON([('q', {'_id': 1}),
                                           ('limit', 1)])])])
-        self.assertEqual(expected, started[2].command)
+        self.assertEqualCommand(expected, started[2].command)
 
     def test_write_errors(self):
         coll = self.client.pymongo_test.test
@@ -1174,38 +1212,20 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(2, len(errors))
         fields = set(['index', 'code', 'errmsg'])
         for error in errors:
-            self.assertEqual(fields, set(error))
+            self.assertTrue(fields.issubset(set(error)))
 
     def test_first_batch_helper(self):
         # Regardless of server version and use of helpers._first_batch
         # this test should still pass.
         self.listener.results.clear()
-        self.client.pymongo_test.collection_names()
+        tuple(self.client.pymongo_test.test.list_indexes())
         results = self.listener.results
         started = results['started'][0]
         succeeded = results['succeeded'][0]
         self.assertEqual(0, len(results['failed']))
         self.assertIsInstance(started, monitoring.CommandStartedEvent)
-        expected = SON([('listCollections', 1), ('cursor', {})])
-        self.assertEqual(expected, started.command)
-        self.assertEqual('pymongo_test', started.database_name)
-        self.assertEqual('listCollections', started.command_name)
-        self.assertIsInstance(started.request_id, int)
-        self.assertEqual(self.client.address, started.connection_id)
-        self.assertIsInstance(succeeded, monitoring.CommandSucceededEvent)
-        self.assertIsInstance(succeeded.duration_micros, int)
-        self.assertEqual(started.command_name, succeeded.command_name)
-        self.assertEqual(started.request_id, succeeded.request_id)
-        self.assertEqual(started.connection_id, succeeded.connection_id)
-
-        self.listener.results.clear()
-        tuple(self.client.pymongo_test.test.list_indexes())
-        started = results['started'][0]
-        succeeded = results['succeeded'][0]
-        self.assertEqual(0, len(results['failed']))
-        self.assertIsInstance(started, monitoring.CommandStartedEvent)
         expected = SON([('listIndexes', 'test'), ('cursor', {})])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('pymongo_test', started.database_name)
         self.assertEqual('listIndexes', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -1215,6 +1235,8 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(started.command_name, succeeded.command_name)
         self.assertEqual(started.request_id, succeeded.request_id)
         self.assertEqual(started.connection_id, succeeded.connection_id)
+        self.assertTrue('cursor' in succeeded.reply)
+        self.assertTrue('ok' in succeeded.reply)
 
         self.listener.results.clear()
         self.client.pymongo_test.current_op(True)
@@ -1223,7 +1245,7 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(0, len(results['failed']))
         self.assertIsInstance(started, monitoring.CommandStartedEvent)
         expected = SON([('currentOp', 1), ('$all', True)])
-        self.assertEqual(expected, started.command)
+        self.assertEqualCommand(expected, started.command)
         self.assertEqual('admin', started.database_name)
         self.assertEqual('currentOp', started.command_name)
         self.assertIsInstance(started.request_id, int)
@@ -1233,6 +1255,8 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual(started.command_name, succeeded.command_name)
         self.assertEqual(started.request_id, succeeded.request_id)
         self.assertEqual(started.connection_id, succeeded.connection_id)
+        self.assertTrue('inprog' in succeeded.reply)
+        self.assertTrue('ok' in succeeded.reply)
 
         if not client_context.is_mongos:
             self.client.fsync(lock=True)
@@ -1246,7 +1270,7 @@ class TestCommandMonitoring(unittest.TestCase):
             self.assertEqual(0, len(results['failed']))
             self.assertIsInstance(started, monitoring.CommandStartedEvent)
             expected = {'fsyncUnlock': 1}
-            self.assertEqual(expected, started.command)
+            self.assertEqualCommand(expected, started.command)
             self.assertEqual('admin', started.database_name)
             self.assertEqual('fsyncUnlock', started.command_name)
             self.assertIsInstance(started.request_id, int)
@@ -1256,6 +1280,8 @@ class TestCommandMonitoring(unittest.TestCase):
             self.assertEqual(started.command_name, succeeded.command_name)
             self.assertEqual(started.request_id, succeeded.request_id)
             self.assertEqual(started.connection_id, succeeded.connection_id)
+            self.assertTrue('info' in succeeded.reply)
+            self.assertTrue('ok' in succeeded.reply)
 
     def test_sensitive_commands(self):
         listeners = self.client._event_listeners
@@ -1286,13 +1312,14 @@ class TestCommandMonitoring(unittest.TestCase):
         self.assertEqual({}, succeeded.reply)
 
 
-class TestGlobalListener(unittest.TestCase):
+class TestGlobalListener(PyMongoTestCase):
 
     @classmethod
     @client_context.require_connection
     def setUpClass(cls):
         cls.listener = EventListener()
-        cls.saved_listeners = monitoring._LISTENERS
+        # We plan to call register(), which internally modifies _LISTENERS.
+        cls.saved_listeners = copy.deepcopy(monitoring._LISTENERS)
         monitoring.register(cls.listener)
         cls.client = single_client()
         # Get one (authenticated) socket in the pool.
@@ -1315,7 +1342,7 @@ class TestGlobalListener(unittest.TestCase):
             isinstance(succeeded, monitoring.CommandSucceededEvent))
         self.assertTrue(
             isinstance(started, monitoring.CommandStartedEvent))
-        self.assertEqual(SON([('ismaster', 1)]), started.command)
+        self.assertEqualCommand(SON([('ismaster', 1)]), started.command)
         self.assertEqual('ismaster', started.command_name)
         self.assertEqual(self.client.address, started.connection_id)
         self.assertEqual('pymongo_test', started.database_name)
