@@ -1,4 +1,4 @@
-# Copyright 2015 MongoDB, Inc.
+# Copyright 2015-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,18 +14,16 @@
 
 """Test the read_concern module."""
 
-import pymongo
-
 from bson.son import SON
 from pymongo import monitoring
 from pymongo.errors import ConfigurationError, OperationFailure
 from pymongo.read_concern import ReadConcern
 
-from test import client_context, pair, unittest
-from test.utils import single_client, EventListener
+from test import client_context, PyMongoTestCase
+from test.utils import single_client, rs_or_single_client, EventListener
 
 
-class TestReadConcern(unittest.TestCase):
+class TestReadConcern(PyMongoTestCase):
 
     @classmethod
     @client_context.require_connection
@@ -33,7 +31,7 @@ class TestReadConcern(unittest.TestCase):
         cls.listener = EventListener()
         cls.saved_listeners = monitoring._LISTENERS
         # Don't use any global subscribers.
-        monitoring._LISTENERS = monitoring._Listeners([])
+        monitoring._LISTENERS = monitoring._Listeners([], [], [], [])
         cls.client = single_client(event_listeners=[cls.listener])
         cls.db = cls.client.pymongo_test
 
@@ -61,8 +59,9 @@ class TestReadConcern(unittest.TestCase):
         self.assertRaises(TypeError, ReadConcern, 42)
 
     def test_read_concern_uri(self):
-        uri = 'mongodb://%s/?readConcernLevel=majority' % (pair,)
-        client = pymongo.MongoClient(uri)
+        uri = 'mongodb://%s/?readConcernLevel=majority' % (
+            client_context.pair,)
+        client = rs_or_single_client(uri, connect=False)
         self.assertEqual(ReadConcern('majority'), client.read_concern)
 
     @client_context.require_version_max(3, 1)
@@ -88,7 +87,7 @@ class TestReadConcern(unittest.TestCase):
         # Explicitly set readConcern to 'local'.
         coll = self.db.get_collection('coll', read_concern=ReadConcern('local'))
         tuple(coll.find({'field': 'value'}))
-        self.assertEqual(
+        self.assertEqualCommand(
             SON([('find', 'coll'),
                  ('filter', {'field': 'value'}),
                  ('readConcern', {'level': 'local'})]),
