@@ -1,4 +1,4 @@
-# Copyright 2011-2016 MongoDB, Inc.
+# Copyright 2011-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,10 @@
 
 import contextlib
 import copy
+import pickle
 import random
 import sys
-import pickle
+import warnings
 
 sys.path[0:0] = [""]
 
@@ -62,6 +63,7 @@ class TestSelections(unittest.TestCase):
 
 class TestReadPreferenceObjects(unittest.TestCase):
     prefs = [Primary(),
+             PrimaryPreferred(),
              Secondary(),
              Nearest(tag_sets=[{'a': 1}, {'b': 2}]),
              SecondaryPreferred(max_staleness=30)]
@@ -74,6 +76,10 @@ class TestReadPreferenceObjects(unittest.TestCase):
         for pref in self.prefs:
             self.assertEqual(pref, copy.copy(pref))
 
+    def test_deepcopy(self):
+        for pref in self.prefs:
+            self.assertEqual(pref, copy.deepcopy(pref))
+
 
 class TestReadPreferencesBase(TestReplicaSetClientBase):
 
@@ -225,6 +231,9 @@ class TestReadPreferences(TestReadPreferencesBase):
                           localthresholdms=-1)
 
     def test_zero_latency(self):
+        if (client_context.version >= (3, 7, 2)
+                and client_context.auth_enabled and client_context.is_rs):
+            raise SkipTest("Disabled due to SERVER-32845")
         ping_times = set()
         # Generate unique ping times.
         while len(ping_times) < len(self.client.nodes):
@@ -416,8 +425,10 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
             lambda: self.c.pymongo_test.some_collection.drop())
 
     def test_group(self):
-        self._test_coll_helper(True, self.c.pymongo_test.test, 'group',
-                               {'a': 1}, {}, {}, 'function() { }')
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            self._test_coll_helper(True, self.c.pymongo_test.test, 'group',
+                                   {'a': 1}, {}, {}, 'function() { }')
 
     def test_map_reduce(self):
         self._test_coll_helper(False, self.c.pymongo_test.test, 'map_reduce',
@@ -436,10 +447,9 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
         self._test_coll_helper(True, self.c.pymongo_test.test, 'distinct', 'a')
 
     def test_aggregate(self):
-        if self.client_version.at_least(2, 1, 0):
-            self._test_coll_helper(True, self.c.pymongo_test.test,
-                                   'aggregate',
-                                   [{'$project': {'_id': 1}}])
+        self._test_coll_helper(True, self.c.pymongo_test.test,
+                               'aggregate',
+                               [{'$project': {'_id': 1}}])
 
 
 class TestMovingAverage(unittest.TestCase):
