@@ -1,4 +1,4 @@
-# Copyright 2009-2015 MongoDB, Inc.
+# Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,8 @@
 """
 
 import sys
+import warnings
+
 sys.path[0:0] = [""]
 
 from bson.son import SON
@@ -31,10 +33,19 @@ class TestSONManipulator(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls):
+        cls.warn_context = warnings.catch_warnings()
+        cls.warn_context.__enter__()
+        warnings.simplefilter("ignore", DeprecationWarning)
+
         client = MongoClient(
             client_context.host, client_context.port, connect=False)
         cls.db = client.pymongo_test
 
+    @classmethod
+    def tearDownClass(cls):
+        cls.warn_context.__exit__()
+        cls.warn_context = None
+
     def test_basic(self):
         manip = SONManipulator()
         collection = self.db.test
