@@ -1,4 +1,4 @@
-# Copyright 2012-2015 MongoDB, Inc.
+# Copyright 2012-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 """
 
 import contextlib
+import functools
 import os
 import struct
 import sys
@@ -34,35 +35,43 @@ from pymongo.write_concern import WriteConcern
 from test import (client_context,
                   db_user,
                   db_pwd)
-from test.version import Version
 
 
-IMPOSSIBLE_WRITE_CONCERN = WriteConcern(w=len(client_context.nodes) + 1)
+IMPOSSIBLE_WRITE_CONCERN = WriteConcern(w=1000)
+
+
+class WhiteListEventListener(monitoring.CommandListener):
+
+    def __init__(self, *commands):
+        self.commands = set(commands)
+        self.results = defaultdict(list)
+
+    def started(self, event):
+        if event.command_name in self.commands:
+            self.results['started'].append(event)
+
+    def succeeded(self, event):
+        if event.command_name in self.commands:
+            self.results['succeeded'].append(event)
+
+    def failed(self, event):
+        if event.command_name in self.commands:
+            self.results['failed'].append(event)
 
 
 class EventListener(monitoring.CommandListener):
 
     def __init__(self):
         self.results = defaultdict(list)
-        self.filters = set()
-
-    def add_command_filter(self, command_name):
-        self.filters.add(command_name)
-
-    def remove_command_filter(self, command_name):
-        self.filters.remove(command_name)
 
     def started(self, event):
-        if event.command_name not in self.filters:
-            self.results['started'].append(event)
+        self.results['started'].append(event)
 
     def succeeded(self, event):
-        if event.command_name not in self.filters:
-            self.results['succeeded'].append(event)
+        self.results['succeeded'].append(event)
 
     def failed(self, event):
-        if event.command_name not in self.filters:
-            self.results['failed'].append(event)
+        self.results['failed'].append(event)
 
 
 class ServerAndTopologyEventListener(monitoring.ServerListener,
@@ -255,13 +264,8 @@ def drop_collections(db):
 
 
 def remove_all_users(db):
-    if Version.from_client(db.client).at_least(2, 5, 3, -1):
-        db.command("dropAllUsersFromDatabase", 1,
-                   writeConcern={"w": client_context.w})
-    else:
-        db = db.client.get_database(
-            db.name, write_concern=WriteConcern(w=client_context.w))
-        db.system.users.delete_many({})
+    db.command("dropAllUsersFromDatabase", 1,
+               writeConcern={"w": client_context.w})
 
 
 def joinall(threads):
@@ -312,17 +316,6 @@ def is_mongos(client):
     return res.get('msg', '') == 'isdbgrid'
 
 
-def enable_text_search(client):
-    sinfo = client.server_info()
-    if 'versionArray' in sinfo and sinfo['versionArray'][:2] == [2, 4]:
-        client.admin.command(
-            'setParameter', textSearchEnabled=True)
-
-        for host, port in client.secondaries:
-            client = single_client(host, port)
-            client.admin.command('setParameter', textSearchEnabled=True)
-
-
 def assertRaisesExactly(cls, fn, *args, **kwargs):
     """
     Unlike the standard assertRaises, this checks that a function raises a
@@ -339,12 +332,40 @@ def assertRaisesExactly(cls, fn, *args, **kwargs):
 
 
 @contextlib.contextmanager
-def ignore_deprecations():
+def _ignore_deprecations():
     with warnings.catch_warnings():
         warnings.simplefilter("ignore", DeprecationWarning)
         yield
 
 
+def ignore_deprecations(wrapped=None):
+    """A context manager or a decorator."""
+    if wrapped:
+        @functools.wraps(wrapped)
+        def wrapper(*args, **kwargs):
+            with _ignore_deprecations():
+                return wrapped(*args, **kwargs)
+
+        return wrapper
+
+    else:
+        return _ignore_deprecations()
+
+
+class DeprecationFilter(object):
+
+    def __init__(self, action="ignore"):
+        """Start filtering deprecations."""
+        self.warn_context = warnings.catch_warnings()
+        self.warn_context.__enter__()
+        warnings.simplefilter(action, DeprecationWarning)
+
+    def stop(self):
+        """Stop filtering deprecations."""
+        self.warn_context.__exit__()
+        self.warn_context = None
+
+
 def read_from_which_host(
         client,
         pref,
@@ -496,3 +517,31 @@ def lazy_client_trial(reset, target, test, get_client):
             lazy_collection = lazy_client.pymongo_test.test
             run_threads(lazy_collection, target)
             test(lazy_collection)
+
+
+def gevent_monkey_patched():
+    """Check if gevent's monkey patching is active."""
+    # In Python 3.6 importing gevent.socket raises an ImportWarning.
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", ImportWarning)
+        try:
+            import socket
+            import gevent.socket
+            return socket.socket is gevent.socket.socket
+        except ImportError:
+            return False
+
+
+def eventlet_monkey_patched():
+    """Check if eventlet's monkey patching is active."""
+    try:
+        import threading
+        import eventlet
+        return (threading.current_thread.__module__ ==
+                'eventlet.green.threading')
+    except ImportError:
+        return False
+
+
+def is_greenthread_patched():
+    return gevent_monkey_patched() or eventlet_monkey_patched()
